package edu.uoc.jsev.b4b.ui;


import android.support.test.espresso.contrib.DrawerActions;
import android.support.test.espresso.contrib.DrawerMatchers;
import android.support.test.espresso.contrib.NavigationViewActions;
import android.support.test.espresso.contrib.RecyclerViewActions;
import android.support.test.espresso.intent.rule.IntentsTestRule;
import android.support.test.runner.AndroidJUnit4;
import android.support.test.uiautomator.UiObjectNotFoundException;
import android.support.test.uiautomator.UiScrollable;
import android.support.test.uiautomator.UiSelector;
import android.test.suitebuilder.annotation.LargeTest;
import android.view.Gravity;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.login.LoginActivity;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailActivity;
import edu.uoc.jsev.b4b.ui.mybooks.newbook.NewBookActivity;

import static android.support.test.espresso.Espresso.onData;
import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.action.ViewActions.closeSoftKeyboard;
import static android.support.test.espresso.action.ViewActions.pressImeActionButton;
import static android.support.test.espresso.action.ViewActions.typeText;
import static android.support.test.espresso.assertion.ViewAssertions.matches;
import static android.support.test.espresso.intent.Intents.intended;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasComponent;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasExtraWithKey;
import static android.support.test.espresso.matcher.RootMatchers.isDialog;
import static android.support.test.espresso.matcher.RootMatchers.withDecorView;
import static android.support.test.espresso.matcher.ViewMatchers.hasDescendant;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withContentDescription;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static android.support.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import static org.hamcrest.Matchers.instanceOf;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.not;

/**
 * Clase de prueba para verificar los procesos
 * de registro y borrado de un libro.
 * La prueba asume que los servicios de localización están desactivados,
 * para poder cerrar el diálogo que aparece al entrar en la pantalla principal
 * de la aplicación preguntando si se desean activar.
 * Asume también que, de inicio, el usuario de pruebas no tiene aún
 * ningún libro registrado en su perfil.
 */
@LargeTest
@RunWith(AndroidJUnit4.class)
public class AddDeleteBookTest extends BaseTest {

    // Constantes para rellenar los campos de la pantalla.
    private static final String NEW_BOOK_TITLE = "Test book";
    private static final String NEW_BOOK_AUTHOR = "Test author";
    private static final String NEW_BOOK_PUBLISHER = "Test publisher";
    private static final String NEW_BOOK_GENRE = "Novela";
    private static final String NEW_BOOK_LANGUAGE = "Español";

    // Actividad inicial de la prueba.
    @Rule
    public IntentsTestRule<LoginActivity> mActivityTestRule = new IntentsTestRule<>(LoginActivity.class);

    /**
     * Prueba principal. Realiza lo siguiente:
     *      - Autentica al usuario.
     *      - Navega hacia el listado de libros del usuario.
     *      - Navega hacia la pantalla de nuevo libro.
     *      - Registra un nuevo libro.
     *      - Navega hacia el detalle del libro introducido.
     *      - Elimina el libro introducido.
     */
    @Test
    public void addDeleteBookTest() throws UiObjectNotFoundException {
        login();
        navigateToMyBooks();
        navigateToNewBook();
        addBook();
        navigateToBookDetail();
        deleteBook();
        assertDeletion();
    }

    private void login() {
        // Autenticamos al usuario.
        onView(withId(R.id.email_text))
                .perform(typeText(USER_EMAIL), closeSoftKeyboard());
        onView(withId(R.id.password_text))
                .perform(typeText(USER_PASSWD), pressImeActionButton());
    }

    private void navigateToMyBooks() {
        // Verificamos que se ha navegado a la pantalla principal.
        intended(hasComponent(MainActivity.class.getName()));
        registerIdlingResource();

        // Verificamos que el usuario se ha autenticado
        // y que no tiene ningún libro registrado todavía.
        assertLoggedIn(USER_EMAIL);
        assertBooksNumber(0);

        // Pulsamos sobre la opción "Mis libros".
        onView(withText(android.R.string.cancel))
                .inRoot(isDialog())
                .check(matches(isDisplayed()))
                .perform(click());

        onView(withId(R.id.drawer_layout))
                .check(matches(DrawerMatchers.isClosed(Gravity.LEFT)))
                .perform(DrawerActions.open())
                .check(matches(DrawerMatchers.isOpen()));

        onView(withId(R.id.nav_view))
                .perform(NavigationViewActions.navigateTo(R.id.nav_books));
    }

    private void navigateToNewBook() {
        // Verificamos que el listado de libros no muestra ningnuno.
        onView(withId(R.id.empty_list))
                .check(matches(isDisplayed()));

        // Pulsamos sobre el boton flotante, para dar de alta un libro.
        onView(withId(R.id.fab))
                .check(matches(isDisplayed()))
                .perform(click());
    }

    private void addBook() throws UiObjectNotFoundException {
        // Verificamos que estamos en la pantalla de alta de libro.
        intended(hasComponent(NewBookActivity.class.getName()));
        registerIdlingResource();

        // Introducimos los campos obligatorios.
        onView(withId(R.id.title_text))
                .perform(typeText(NEW_BOOK_TITLE), closeSoftKeyboard());
        onView(withId(R.id.author_text))
                .perform(typeText(NEW_BOOK_AUTHOR), closeSoftKeyboard());
        onView(withId(R.id.publisher_text))
                .perform(typeText(NEW_BOOK_PUBLISHER), closeSoftKeyboard());

        // Nos desplazamos hacia abajo.
        UiScrollable scrollView = new UiScrollable(new UiSelector().resourceId("edu.uoc.jsev.b4b:id/nested_scroll_view"));
        scrollView.scrollForward();

        // Seleccionamos un género del desplegable.
        onView(withId(R.id.genre))
                .perform(click());
        onData(allOf(is(instanceOf(String.class)), is(NEW_BOOK_GENRE)))
                .inRoot(withDecorView(not(is(mActivityTestRule.getActivity().getWindow().getDecorView()))))
                .perform(click());
        onView(withId(R.id.genre_text))
                .check(matches(withText(NEW_BOOK_GENRE)));

        // Seleccionamos un idioma del desplegable.
        onView(withId(R.id.language))
                .perform(click());
        onData(allOf(is(instanceOf(String.class)), is(NEW_BOOK_LANGUAGE)))
                .inRoot(withDecorView(not(is(mActivityTestRule.getActivity().getWindow().getDecorView()))))
                .perform(click());
        onView(withId(R.id.language_text))
                .check(matches(withText(NEW_BOOK_LANGUAGE)));

        // Seleccionamos el estado del libro.
        onView(withId(R.id.state_regular))
                .perform(click());

        // Pulsamos sobre el botón de alta de libro.
        onView(withId(R.id.action_add_book))
                .perform(click());
    }

    private void navigateToBookDetail() {
        // Verificamos que, tras registrar el libro, estamos en la pantalla del listado,
        // y que el número de libros del usuario se ha incrementado.
        intended(hasComponent(MainActivity.class.getName()));
        assertBooksNumber(1);

        // Verificamos que el libro introducido se está mostrando.
        onView(withId(R.id.empty_list))
                .check(matches(not(isDisplayed())));

        // Pulsamos sobre el libro, para ir a su detalle.
        onView(withId(R.id.list))
                .perform(RecyclerViewActions.actionOnItem(hasDescendant(withText(NEW_BOOK_TITLE)), click()));
    }

    private void deleteBook() {
        // Verificamos que, tras pulsar sobre el elemento del listado,
        // nos encontramos en la pantalla de detalle del libro,
        // y que los datos mostrados son los del libro seleccionado.
        intended(allOf(hasComponent(BookDetailActivity.class.getName()), hasExtraWithKey(BookDetailActivity.EXTRA_BOOK_DATA)));
        registerIdlingResource();
        onView(withId(R.id.author))
                .check(matches(withText(NEW_BOOK_AUTHOR)));

        // Pulsamos sobre el bóton de eliminar.
        onView(allOf(withId(R.id.fab), withContentDescription(R.string.action_delete_book)))
                .perform(click());

        // Confirmamos el borrado en el diálogo.
        onView(withText(android.R.string.ok))
                .inRoot(isDialog())
                .check(matches(isDisplayed()))
                .perform(click());
    }

    private void assertDeletion() {
        // Verificamos que, tras eliminar el libro, volvemos al listado.
        intended(hasComponent(MainActivity.class.getName()));

        // Verificamos que el listado no está mostrando ningún libro,
        // y que el número de libros del usuario se ha decrementado.
        onView(withId(R.id.empty_list))
                .check(matches(isDisplayed()));
        assertBooksNumber(0);
    }
}
