package edu.uoc.jsev.b4b.ui;

import android.app.Activity;
import android.support.test.InstrumentationRegistry;
import android.support.test.espresso.IdlingRegistry;
import android.support.test.espresso.IdlingResource;
import android.support.test.rule.GrantPermissionRule;
import android.support.test.runner.lifecycle.ActivityLifecycleMonitorRegistry;
import android.support.test.runner.lifecycle.Stage;
import android.support.test.uiautomator.UiDevice;
import android.util.Log;

import com.backendless.Backendless;
import com.backendless.BackendlessUser;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Rule;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.backendless.model.Usuario;
import edu.uoc.jsev.b4b.util.ViewVisibilityIdlingResource;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

/**
 * Clase base para los test automáticos de interfaz de usuario.
 * Configura las pruebas para su ejecución y proporciona una
 * serie de utilidades comunes a todas ellas.
 */
public abstract class BaseTest {

    // Email y contraseña del usuario de pruebas por defecto.
    protected static final String USER_EMAIL = "espresso@test.com";
    protected static final String USER_PASSWD = "123456";

    // Referencia a la actividad que se está ejecutando actualmente.
    private Activity mCurrentActivity;

    // Lista de IdlingResources registrados. En caso de que la prueba
    // transcurra en varias actividades es necesario registrar un IdlingResource
    // por cada una de ellas, ya que lo que hacen es controlar si la actividad actual
    // está mostrando, o no, el indicador de progreso.
    private static List<ViewVisibilityIdlingResource> mIdlingResources;

    // Necesario para las partes que se prueban con UI Automator.
    protected static UiDevice mUiDevice;

    // Proporcionamos el permiso de acceso a la localización, para que la aplicación
    // no lo pida durante las pruebas al acceder a la pantalla principal.
    @Rule
    public GrantPermissionRule mPermissionRule = GrantPermissionRule.grant(android.Manifest.permission.ACCESS_FINE_LOCATION);

    /**
     * Al iniciar un test inicializamos la lista de IdlingResources
     * y la intrumentación con UI Automator.
     */
    @BeforeClass
    public static void init() {
        mUiDevice = UiDevice.getInstance(InstrumentationRegistry.getInstrumentation());
        mIdlingResources = new ArrayList<>();
    }

    /**
     * Registramos el IdlingResource de la actividad actual antes de cada prueba.
     */
    @Before
    public void registerIdlingResource() {
        ViewVisibilityIdlingResource idlingResource = new ViewVisibilityIdlingResource(getCurrentActivity(), R.id.progress);
        mIdlingResources.add(idlingResource);

        IdlingRegistry.getInstance().register(idlingResource);
    }

    /**
     * Eliminamos todos IdlingResources registrados después de cada prueba.
     */
    @After
    public void unregisterIdlingResources() {
        for(IdlingResource idlingResource : mIdlingResources) {
            IdlingRegistry.getInstance().unregister(idlingResource);
        }
        mIdlingResources.clear();
    }

    /**
     * Método auxiliar para pausar la ejecución de las pruebas.
     * @param millis Número de milisegundos de la pausa.
     */
    protected void sleep(long millis) {
        try {
            Thread.sleep(millis);
        }
        catch (InterruptedException e) {
            Log.w("->>>", e);
        }
    }

    /**
     * Comprueba que el usuario pasado como parámetro está autenticado.
     * @param expectedEmail El email del usuario que se espera
     *                      esté autenticado.
     */
    protected void assertLoggedIn(String expectedEmail) {
        BackendlessUser currentUser = Backendless.UserService.CurrentUser();
        assertNotNull(currentUser);
        assertEquals(expectedEmail, currentUser.getEmail());
    }

    /**
     * Comprueba que no hay ningún usuario autenticado.
     */
    protected void assertLoggedOut() {
        BackendlessUser currentUser = Backendless.UserService.CurrentUser();
        assertNull(currentUser);
    }

    /**
     * Comprueba que el número de libros del usuario de prueba
     * es el pasado como parámetro.
     * @param expectedNumber El número de libros esperado.
     */
    protected void assertBooksNumber(int expectedNumber) {
        BackendlessUser currentUser = Backendless.UserService.CurrentUser();
        assertEquals(expectedNumber ,(int) currentUser.getProperty(Usuario.BOOKS_NUMBER_KEY));
    }

    /**
     * Obtiene la actividad que se está ejecutando actualmente en la prueba.
     * @return La actividad que se está ejecutando actualmente en la prueba.
     */
    private Activity getCurrentActivity() {
        InstrumentationRegistry.getInstrumentation().runOnMainSync(new Runnable() {
            public void run() {
                Collection<Activity> resumedActivities = ActivityLifecycleMonitorRegistry.getInstance().getActivitiesInStage(Stage.RESUMED);
                if (resumedActivities.iterator().hasNext()){
                    mCurrentActivity = resumedActivities.iterator().next();
                }
            }
        });

        return mCurrentActivity;
    }
}
