package edu.uoc.jsev.b4b.ui;

import android.support.test.espresso.intent.rule.IntentsTestRule;
import android.support.test.espresso.matcher.ViewMatchers;
import android.support.test.filters.LargeTest;
import android.support.test.runner.AndroidJUnit4;
import android.support.test.uiautomator.UiObject;
import android.support.test.uiautomator.UiObjectNotFoundException;
import android.support.test.uiautomator.UiSelector;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.registration.RegistrationActivity;

import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.action.ViewActions.closeSoftKeyboard;
import static android.support.test.espresso.action.ViewActions.typeText;
import static android.support.test.espresso.assertion.ViewAssertions.matches;
import static android.support.test.espresso.intent.Intents.intended;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasComponent;
import static android.support.test.espresso.matcher.RootMatchers.withDecorView;
import static android.support.test.espresso.matcher.ViewMatchers.isDescendantOfA;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static android.support.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import static org.hamcrest.Matchers.not;

/**
 * Clase de prueba para la actividad de registro.
 */
@RunWith(AndroidJUnit4.class)
@LargeTest
public class RegistrationActivityTest extends BaseTest {

    // Constantes para rellenar los campos de la pantalla.
    private static final String NEW_USER_NAME = "Another Espresso Test";
    private static final String NEW_USER_EMAIL = "another_espresso@test.com";
    private static final String NEW_USER_CITY = "Sevilla";
    private static final String NEW_USER_PLACE = "UOC Sede";
    private static final String NEW_USER_PASSWD = "123456";

    // Actividad inicial de la prueba.
    @Rule
    public IntentsTestRule<RegistrationActivity> mActivityTestRule = new IntentsTestRule<>(RegistrationActivity.class);

    /**
     * Comprueba que se muestran los mensajes correspondientes
     * si se intenta hacer el registro sin rellenar los campos obligatorios.
     */
    @Test
    public void emptyRegistrationTest() {
        // Pulsamos sobre el botón "Crear cuenta".
        onView(ViewMatchers.withId(R.id.register_button))
                .perform(closeSoftKeyboard(), click());

        // Validamos los mensajes.
        onView(allOf(isDescendantOfA(withId(R.id.full_name)), withText(R.string.error_mandatory_field)))
                .check(matches(isDisplayed()));
        onView(allOf(isDescendantOfA(withId(R.id.email)), withText(R.string.error_mandatory_field)))
                .check(matches(isDisplayed()));
        onView(allOf(isDescendantOfA(withId(R.id.password)), withText(R.string.error_mandatory_field)))
                .check(matches(isDisplayed()));
        onView(withId(R.id.location_error))
                .check(matches(allOf(isDisplayed(), withText(R.string.error_mandatory_field))));
    }

    /**
     * Comprueba que se muestran los mensajes correspondientes
     * si se intenta registrar la cuenta introduciendo valores
     * con formatos no válidos.
     */
    @Test
    public void fieldsFormatRegistrationTest() {
        // Introducimos el nombre del usuario.
        onView(withId(R.id.full_name_text))
                .perform(typeText(NEW_USER_NAME), closeSoftKeyboard());

        // Introducimos un email no válido.
        onView(withId(R.id.email_text))
                .perform(typeText(NEW_USER_EMAIL.replaceAll("@", "")), closeSoftKeyboard());

        // Introducimos un password con menos caracteres de los requeridos.
        onView(withId(R.id.password_text))
                .perform(typeText(NEW_USER_PASSWD.substring(3)), closeSoftKeyboard());
        onView(withId(R.id.repeat_password_text))
                .perform(typeText(NEW_USER_PASSWD.substring(3)), closeSoftKeyboard());

        // Pulsamos sobre el botón "Crear cuenta".
        onView(withId(R.id.register_button))
                .perform(closeSoftKeyboard(), click());

        // Verificamos que se muestran los mensajes.
        onView(allOf(isDescendantOfA(withId(R.id.email)), withText(R.string.error_invalid_email)))
                .check(matches(isDisplayed()));
        onView(allOf(isDescendantOfA(withId(R.id.password)), withText(R.string.error_invalid_password)))
                .check(matches(isDisplayed()));
    }

    /**
     * Comprueba que se muestra el mensaje correspondiente
     * si se intenta registrar la cuenta y el campo de confirmación
     * de la contraseña no coincide con la contraseña introducida.
     */
    @Test
    public void passwordsDontMatchTest() {
        // Introducimos un password válido.
        onView(withId(R.id.password_text))
                .perform(typeText(NEW_USER_PASSWD), closeSoftKeyboard());

        // Introducimos una confirmación de password que no coincide.
        onView(withId(R.id.repeat_password_text))
                .perform(typeText(NEW_USER_PASSWD.substring(1)), closeSoftKeyboard());

        // Pulsamos sobre el botón "Crear cuenta".
        onView(withId(R.id.register_button))
                .perform(closeSoftKeyboard(), click());

        // Verificamos que se muestra el mensaje.
        onView(allOf(isDescendantOfA(withId(R.id.repeat_password)), withText(R.string.error_password_matching)))
                .check(matches(isDisplayed()));
    }

    /**
     * Comprueba que se muestra el mensaje correspondiente
     * si se intenta registrar una cuenta con un email ya
     * registrado.
     */
    @Test
    public void duplicatedRegistrationTest() throws UiObjectNotFoundException {
        // Rellenamos los datos con los de un usuario que ya existe.
        fillRegistrationForm(USER_EMAIL);

        // Verificamos que se muestra el mensaje.
        onView(withText(R.string.error_incorrect_registration))
                .inRoot(withDecorView(not(mActivityTestRule.getActivity().getWindow().getDecorView())))
                .check(matches(isDisplayed()));
    }

    /**
     * Comprueba que el proceso de registro es correcto,
     * y que el usuario se autentica una vez se ha creado
     * la cuenta.
     * Para que la prueba sea correcta, el correo electrónico
     * que se proporcione en la prueba no debe estar dado de alta
     * aún en la aplicación.
     */
    @Test
    public void validRegistrationTest() throws UiObjectNotFoundException {
        // Rellenamos los datos de forma correcta con los de un usuario nuevo.
        fillRegistrationForm(NEW_USER_EMAIL);

        // Verificamos que, una vez hecho el registro, el usuario
        // ha navegado a la pantalla principal y se encuentra autenticado.
        intended(hasComponent(MainActivity.class.getName()));
        assertLoggedIn(NEW_USER_EMAIL);
    }

    /**
     * Método auxiliar para rellenar los datos de la pantalla.
     */
    private void fillRegistrationForm(String userEmail) throws UiObjectNotFoundException {
        // Rellena los campos nombre, email, contraseña y confirmación de contraseña.
        onView(withId(R.id.full_name_text))
                .perform(typeText(NEW_USER_NAME), closeSoftKeyboard());
        onView(withId(R.id.email_text))
                .perform(typeText(userEmail), closeSoftKeyboard());
        onView(withId(R.id.password_text))
                .perform(typeText(NEW_USER_PASSWD), closeSoftKeyboard());
        onView(withId(R.id.repeat_password_text))
                .perform(typeText(NEW_USER_PASSWD), closeSoftKeyboard());
        onView(withId(R.id.place_autocomplete_search_input))
                .perform(click());

        // Selecciona una localización en la herramienta de autocompletado de Google Places.
        // Para simular esta interacción es necesario utilizar UI Automator, ya que Espresso
        // no la reconoce correctamente.
        String searchHint = mActivityTestRule.getActivity().getString(R.string.place_autocomplete_search_hint);
        UiObject placeSearchText = mUiDevice.findObject(new UiSelector().text(searchHint));
        placeSearchText.setText(NEW_USER_CITY + " " + NEW_USER_PLACE);
        UiObject selectedPlace = mUiDevice.findObject(new UiSelector().textStartsWith(NEW_USER_PLACE));
        selectedPlace.click();

        // Pulsamos sobre el botón "Crear cuenta".
        onView(withId(R.id.register_button))
                .perform(click());
    }
}
