package edu.uoc.jsev.b4b.data.backendless.service;

import com.backendless.Backendless;
import com.backendless.async.callback.AsyncCallback;
import com.backendless.exceptions.BackendlessFault;
import com.backendless.persistence.DataQueryBuilder;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Libro;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Servicio de acceso al repositorio de libros en Backendless.
 */
public class BookService {

    // Instancia singleton.
    private static BookService instance;

    private BookService() {

    }

    // Devuelve el singleton.
    public static BookService getInstance() {
        if(instance == null) {
            instance = new BookService();
        }

        return instance;
    }

    /**
     * Obtiene la lista de libros del usuario pasado como parámetro.
     *
     * @param userId ID del usuario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de libros del usuario.
     */
    public void getUserBooks(String userId, final ServiceCallback<List<Libro>> callback) {
        DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                .setPageSize(100)
                .setWhereClause("ownerId='" + userId + "' AND eliminado = FALSE")
                .setSortBy("titulo");

        Backendless.Data.of(Libro.class).find(queryBuilder, new AsyncCallback<List<Libro>>() {
            @Override
            public void handleResponse(List<Libro> response) {
                callback.onSuccess(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Comprueba si un usuario ya tiene registrado un libro
     * con el código ISBN pasado como parámetro.
     *
     * @param isbn Código ISBN.
     * @param userId ID del usuario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback true
     *                 si ya hay un libro registrado con dicho ISBN, y false
     *                 en caso contrario. Si el ISBN es vacío se devuelve
     *                 false siempre.
     */
    public void bookExists(String isbn, String userId, final ServiceCallback<Boolean> callback) {
        if(Utils.isEmptyString(isbn)) {
            callback.onSuccess(false);
            return;
        }

        DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                .setWhereClause("ownerId = '" + userId + "' " +
                        "AND isbn='" + isbn + "' " +
                        "AND eliminado = FALSE");

        Backendless.Data.of(Libro.class).getObjectCount(queryBuilder, new AsyncCallback<Integer>() {
            @Override
            public void handleResponse(Integer bookCount) {
                callback.onSuccess(bookCount > 0);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Inserta/actualiza un libro en la base de datos del backend.
     * @param book Libro a guardar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el objeto
     *                 insertado/actualizado en la base de datos.
     */
    public void saveBook(Libro book, final ServiceCallback<Libro> callback) {
        Backendless.Data.of(Libro.class).save(book, new AsyncCallback<Libro>() {
            @Override
            public void handleResponse(final Libro bookResponse) {
                callback.onSuccess(bookResponse);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Devuelve el código numérico para el estado "malo".
     * @return El código numérico para el estado "malo".
     */
    public int getStateBad() {
        return Libro.STATE_BAD;
    }

    /**
     * Devuelve el código numérico para el estado "regular".
     * @return El código numérico para el estado "regular".
     */
    public int getStateRegular() {
        return Libro.STATE_REGULAR;
    }

    /**
     * Devuelve el código numérico para el estado "bueno".
     * @return El código numérico para el estado "bueno".
     */
    public int getStateGood() {
        return Libro.STATE_GOOD;
    }
}
