package edu.uoc.jsev.b4b.data.backendless.service;

import android.util.Log;

import com.backendless.Backendless;
import com.backendless.async.callback.AsyncCallback;
import com.backendless.exceptions.BackendlessFault;
import com.backendless.messaging.MessageStatus;
import com.backendless.persistence.DataQueryBuilder;

import java.util.Arrays;
import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Genero;
import edu.uoc.jsev.b4b.data.backendless.model.Idioma;

/**
 * Servicio de acceso a repositorios horizontales a otros servicios en Backendless,
 * tales como el envío de correo electrónico, la carga de imágenes en el servidor
 * o el acceso al repositorio de géneros de libros e idiomas.
 */
public class CommonsService {

    // Ruta base del directorio en Backendless para almacenar las imágenes de los libros.
    private static final String BASE_PATH = "b4b/";

    // Instancia singleton.
    private static CommonsService instance;

    // Listas para cachear géneros de libros e idiomas.
    private List<Genero> mGenres;
    private List<Idioma> mLanguages;

    private CommonsService() {

    }

    // Devuelve el singleton.
    public static CommonsService getInstance() {
        if(instance == null) {
            instance = new CommonsService();
        }

        return instance;
    }

    /**
     * Obtiene la lista de todos los géneros literarios disponibles en Backendless.
     * La primera vez que se invoca este método obtiene los datos del servidor y los
     * cachea en una variable miembro, para que en sucesivas llamadas no sea necesario
     * invocar de nuevo al servidor.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de géneros literarios.
     */
    public void getAllGenres(final ServiceCallback<List<Genero>> callback) {
        if(mGenres == null) {
            DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                    .setPageSize(100)
                    .setSortBy("nombre");
            Backendless.Data.of(Genero.class).find(queryBuilder, new AsyncCallback<List<Genero>>() {
                @Override
                public void handleResponse(List<Genero> response) {
                    mGenres = response;
                    callback.onSuccess(response);
                }

                @Override
                public void handleFault(BackendlessFault fault) {
                    callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
                }
            });
        }
        else {
            callback.onSuccess(mGenres);
        }
    }

    /**
     * Obtiene la lista de todos los idiomas disponibles en Backendless.
     * La primera vez que se invoca este método obtiene los datos del servidor y los
     * cachea en una variable miembro, para que en sucesivas llamadas no sea necesario
     * invocar de nuevo al servidor.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de idiomas.
     */
    public void getAllLanguages(final ServiceCallback<List<Idioma>> callback) {
        if(mLanguages == null) {
            DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                    .setPageSize(100)
                    .setSortBy("nombre");
            Backendless.Data.of(Idioma.class).find(queryBuilder, new AsyncCallback<List<Idioma>>() {
                @Override
                public void handleResponse(List<Idioma> response) {
                    mLanguages = response;
                    callback.onSuccess(response);
                }

                @Override
                public void handleFault(BackendlessFault fault) {
                    callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
                }
            });
        }
        else {
            callback.onSuccess(mLanguages);
        }
    }

    /**
     * Almacena una imagen en el sistema de archivos remoto de Backendless.
     *
     * @param picture Imagen a guardar.
     * @param userDir Directorio remoto en el que se almacena la imagen.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback una
     *                 cadena de texto con la ruta de descarga de la imagen.
     */
    public void uploadPicture(byte[] picture, String userDir, final ServiceCallback<String> callback) {
        String path = BASE_PATH + userDir;
        String filename = System.currentTimeMillis() + ".png";
        Backendless.Files.saveFile(path, filename, picture, new AsyncCallback<String>() {
            @Override
            public void handleResponse(String response) {
                callback.onSuccess(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Envía un mensaje de correo electrónico con el asunto y cuerpo indicados
     * a las direcciones pasadas como parámetro.
     * @param subject Asunto del correo.
     * @param message Cuerpo del correo.
     * @param recipients Direcciones de correo a las que se envía el mensaje.
     */
    public void sendEmail(final String subject, String message, String... recipients) {
        Backendless.Messaging.sendTextEmail(subject, message, Arrays.asList(recipients), new AsyncCallback<MessageStatus>() {
            @Override
            public void handleResponse(MessageStatus response) {
                Log.i("--->", "Mensaje enviado: " + subject);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                Log.w("--->", "No se pudo enviar el mensaje: " + fault.getDetail());
            }
        });
    }
}
