package edu.uoc.jsev.b4b.data.backendless.service;

import com.backendless.Backendless;
import com.backendless.BackendlessUser;
import com.backendless.async.callback.AsyncCallback;
import com.backendless.exceptions.BackendlessFault;
import com.backendless.persistence.DataQueryBuilder;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;

/**
 * Servicio para la búsqueda de usuarios/libros por criterios.
 */
public class SearchService extends UserService {

    // Instancia singleton.
    private static SearchService instance;

    private SearchService() {

    }

    // Devuelve el singleton.
    public static SearchService getInstance() {
        if(instance == null) {
            instance = new SearchService();
        }

        return instance;
    }

    /**
     * Devuelve la lista de usuarios que comparten algún libro
     * que verifique los criterios indicados en los parámetros.
     * Si se especifican varios criterios, la búsqueda se realiza
     * mediante un operador lógico "AND".
     * @param title Búsqueda de libros por título.
     * @param author Búsqueda de libros por autor.
     * @param genre Búsqueda de libros por genero literario.
     * @param distance Búsqueda de libros por distancia con el usuario
     *                 propietario del libro. Se devolverán los usuarios
     *                 que se encuentren en un radio igual o inferior a la
     *                 distancia especificada (en kilómetros).
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de usuarios que comparten algún libro que verifique los
     *                 criterios de búsqueda.
     */
    public void findUsers(String title, String author, String genre, int distance, final ServiceCallback<List<BackendlessUser>> callback) {
        DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                .setPageSize(100)
                .setWhereClause(buildWhereClause(title, author, genre, distance));

        Backendless.Data.of(BackendlessUser.class).find(queryBuilder, new AsyncCallback<List<BackendlessUser>>() {
            @Override
            public void handleResponse(List<BackendlessUser> response) {
                callback.onSuccess(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Método auxiliar que construye la cláusula "WHERE" que se ejecutará en Backendless
     * para realiazar la búsqueda de usuarios.
     */
    private String buildWhereClause(String title, String author, String genre, int distance) {
        String whereClause = "distance(" +
                getCurrentUserLatitude() + "," +
                getCurrentUserLongitude() + "," +
                "location.latitude, location.longitude) <= km(" + distance + ") ";
        whereClause += "AND objectId != '" + getCurrentUserId() + "' ";
        whereClause += "AND objectId in " +
                "(Libro[eliminado = FALSE " +
                "AND genero LIKE '%" + genre + "%' " +
                "AND autor LIKE '%" + author + "%' " +
                "AND titulo LIKE '%" + title + "%'].ownerId)";

        return whereClause;
    }
}
