package edu.uoc.jsev.b4b.data.backendless.service;

import com.backendless.Backendless;
import com.backendless.async.callback.AsyncCallback;
import com.backendless.exceptions.BackendlessFault;
import com.backendless.persistence.DataQueryBuilder;

import java.util.Arrays;
import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Intercambio;
import edu.uoc.jsev.b4b.data.backendless.model.Libro;

/**
 * Servicio de acceso al repositorio de intercambios en Backendless.
 */
public class SwapService {

    // Instancia singleton.
    private static SwapService instance;

    private SwapService() {

    }

    // Devuelve el singleton.
    public static SwapService getInstance() {
        if(instance == null) {
            instance = new SwapService();
        }

        return instance;
    }

    /**
     * Obtiene la lista de intercambios en los que participa
     * el usuario pasado como parámetro, bien como origen o
     * bien como destinatario.
     * @param userId ID del usuario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de intercambios en los que participa el usuario.
     */
    public void getUserSwaps(String userId, final ServiceCallback<List<Intercambio>> callback) {
        DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                .setPageSize(100)
                .setWhereClause("ownerId='" + userId + "' " +
                        "OR libroDestino.ownerId='" + userId + "'")
                .setSortBy("created desc");

        Backendless.Data.of(Intercambio.class).find(queryBuilder, new AsyncCallback<List<Intercambio>>() {
            @Override
            public void handleResponse(List<Intercambio> response) {
                callback.onSuccess(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Calcula la valoración media de los intecambios en los que participa el usuario.
     * En el cálculo de la media solo se tienen en cuenta los intercambios en los que
     * el usuario ya ha recibido una valoración por parte del otro usuario implicado.
     * @param userId ID del usuario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la valoración
     *                 media de los intercambios en los que participa el usuario.
     */
    public void getUserRating(final String userId, final ServiceCallback<Double> callback) {
        DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                .setPageSize(100)
                .setWhereClause("(ownerId='" + userId + "' AND valoracionOrigen > 0) " +
                        "OR (libroDestino.ownerId='" + userId + "' AND valoracionDestino > 0) " +
                        "AND estado=" + getStateAccepted());

        Backendless.Data.of(Intercambio.class).find(queryBuilder, new AsyncCallback<List<Intercambio>>() {
            @Override
            public void handleResponse(List<Intercambio> response) {
                int count = 0;
                double sum = 0d;
                for(Intercambio swap : response) {
                    sum += swap.getOwnerId().equals(userId) ?
                            swap.getValoracionOrigen() : swap.getValoracionDestino();
                    count++;
                }

                double rating = count > 0 ? sum/count : 0d;
                callback.onSuccess(rating);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Inserta un nuevo intercambio en la base de datos de Backendless.
     * @param swap Datos del intercambio a guardar.
     * @param bookId ID del libro solicitado al usuario destinatario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el objeto
     *                 intercambio almacenado en la base de datos.
     */
    public void addSwap(Intercambio swap, final String bookId, final ServiceCallback<Intercambio> callback) {
        // Insertamos el intercambio.
        Backendless.Data.of(Intercambio.class).save(swap, new AsyncCallback<Intercambio>() {
            @Override
            public void handleResponse(final Intercambio newSwapResponse) {
                Libro destinationBook = new Libro();
                destinationBook.setObjectId(bookId);
                // Añadimos la relación entre el intercambio y el libro destino.
                Backendless.Data.of(Intercambio.class).setRelation(newSwapResponse,
                        Intercambio.DESTINATION_BOOK_KEY,
                        Arrays.asList(destinationBook),
                        new AsyncCallback<Integer>() {
                            @Override
                            public void handleResponse(Integer response) {
                                // Obtenemos el objeto intercambio almacenado en la base de datos.
                                Backendless.Data.of(Intercambio.class).findById(newSwapResponse.getObjectId(), new AsyncCallback<Intercambio>() {
                                    @Override
                                    public void handleResponse(Intercambio response) {
                                        callback.onSuccess(response);
                                    }

                                    @Override
                                    public void handleFault(BackendlessFault fault) {
                                        callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
                                    }
                                });
                            }

                            @Override
                            public void handleFault(BackendlessFault fault) {
                                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
                            }
                        });
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Actualiza los datos en Backendless de un intercambio existente.
     * @param swap El intercambio con los datos actualizados.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el objeto
     *                 intercambio actualizado en la base de datos.
     */
    public void updateSwap(Intercambio swap, final ServiceCallback<Intercambio> callback) {
        Backendless.Data.of(Intercambio.class).save(swap, new AsyncCallback<Intercambio>() {
            @Override
            public void handleResponse(Intercambio response) {
                callback.onSuccess(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Establece en Backendless la relación entre un intercambio existente
     * y el libro solicitado por el usuario destinatario al usuario origen.
     * @param swap Intercambio con el que se relacionará el libro.
     * @param bookId ID del libro solicitado por el usuario destinatario al usuario origen.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el objeto
     *                 intercambio actualizado en la base de datos.
     */
    public void setRelatedBook(final Intercambio swap, String bookId, final ServiceCallback<Intercambio> callback) {
        Libro sourceBook = new Libro();
        sourceBook.setObjectId(bookId);

        Backendless.Data.of(Intercambio.class).setRelation(swap, Intercambio.SOURCE_BOOK_KEY,
                Arrays.asList(sourceBook), new AsyncCallback<Integer>() {
            @Override
            public void handleResponse(Integer response) {
                callback.onSuccess(swap);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Comprueba si un libro interviene ya en algún intercambio
     * que esté aún pendiente con el usuario pasado como parámetro .
     * @param bookId ID del libro a comprobar.
     * @param userId ID del usuario a comprobar. Si es null, el usuario no se
     *               comprueba, y se verifica que el libro no se esté usando
     *               en ningún intercambio pendiente, independientemente del usuario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback true
     *                 si el libro participa en algún intercambio que está
     *                 aún pendiente con el usuario, y false en caso contrario.
     */
    public void isBookInPendingSwaps(String bookId, String userId, final ServiceCallback<Boolean> callback) {
        String whereClause = "(libroDestino.objectId = '" + bookId + "' " +
                "OR libroOrigen.objectId = '" + bookId + "') ";
        if(userId != null) {
            whereClause = "((libroDestino.objectId = '" + bookId + "' AND ownerId = '" + userId + "') " +
                    "OR (libroOrigen.objectId = '" + bookId + "' AND libroDestino.ownerId = '" + userId + "')) ";
        }
        whereClause += "AND estado = " + getStatePending();

        DataQueryBuilder queryBuilder = DataQueryBuilder.create()
                .setWhereClause(whereClause);

        Backendless.Data.of(Intercambio.class).getObjectCount(queryBuilder, new AsyncCallback<Integer>() {
            @Override
            public void handleResponse(Integer swapCount) {
                callback.onSuccess(swapCount > 0);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Devuelve el código numérico para el estado "pendiente".
     * @return El código numérico para el estado "pendiente".
     */
    public int getStatePending() {
        return Intercambio.STATE_PENDING;
    }

    /**
     * Devuelve el código numérico para el estado "cancelado".
     * @return El código numérico para el estado "cancelado".
     */
    public int getStateCanceled() {
        return Intercambio.STATE_CANCELED;
    }

    /**
     * Devuelve el código numérico para el estado "aceptado".
     * @return El código numérico para el estado "aceptado".
     */
    public int getStateAccepted() {
        return Intercambio.STATE_ACCEPTED;
    }
}
