package edu.uoc.jsev.b4b.data.backendless.service;

import android.util.Log;

import com.backendless.Backendless;
import com.backendless.BackendlessUser;
import com.backendless.async.callback.AsyncCallback;
import com.backendless.exceptions.BackendlessFault;
import com.backendless.geo.GeoPoint;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Usuario;

/**
 * Servicio de acceso al repositorio de usuarios en Backendless.
 */
public class UserService {

    // Instancia singleton.
    private static UserService instance;

    protected UserService() {

    }

    // Devuelve el singleton.
    public static UserService getInstance() {
        if(instance == null) {
            instance = new UserService();
        }

        return instance;
    }

    /**
     * Obtiene el usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     * @return El usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     */
    public BackendlessUser getCurrentUser() {
        return Backendless.UserService.CurrentUser();
    }

    /**
     * Obtiene el nombre del usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     * @return El nombre del usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     */
    public String getCurrentUserFullName() {
        BackendlessUser currentUser = getCurrentUser();
        return currentUser != null ? currentUser.getProperty(Usuario.NAME_KEY).toString() : null;
    }

    /**
     * Obtiene el email del usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     * @return El email del usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     */
    public String getCurrentUserEmail() {
        BackendlessUser currentUser = getCurrentUser();
        return currentUser != null ? currentUser.getEmail() : null;
    }

    /**
     * Obtiene el ID del usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     * @return El ID del usuario autenticado actualmente en la aplicación,
     * o null si no hay ninguno autenticado.
     */
    public String getCurrentUserId() {
        BackendlessUser currentUser = getCurrentUser();
        return currentUser != null ? currentUser.getObjectId() : null;
    }

    /**
     * Obtiene la latitud de la última localización conocida del usuario
     * autenticado actualmente en la aplicación, o null si no hay ninguno autenticado.
     * @return La latitud de la última localización conocida del usuario
     * autenticado actualmente en la aplicación, o null si no hay ninguno autenticado.
     */
    public Double getCurrentUserLatitude() {
        GeoPoint location = getCurrentUserCoordinates();
        Double lat = null;

        if(location != null) {
            lat = location.getLatitude();
        }

        return lat;
    }

    /**
     * Obtiene la longitud de la última localización conocida del usuario
     * autenticado actualmente en la aplicación, o null si no hay ninguno autenticado.
     * @return La longitud de la última localización conocida del usuario
     * autenticado actualmente en la aplicación, o null si no hay ninguno autenticado.
     */
    public Double getCurrentUserLongitude() {
        GeoPoint location = getCurrentUserCoordinates();
        Double lon = null;

        if(location != null) {
            lon = location.getLongitude();
        }

        return lon;
    }

    /**
     * Actualiza los datos del usuario autenticado actualmente en la aplicación.
     * @param currentUser Usuario actualmente autenticado en la aplicación.
     */
    public void setCurrentUser(BackendlessUser currentUser) {
        Backendless.UserService.setCurrentUser(currentUser);
    }

    /**
     * Verifica las credenciales de un usuario para poder acceder a la aplicación.
     * @param email Email del usuario.
     * @param password Contraseña del usuario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la autenticación es correcta se informa al callback
     *                 por medio de su método onSuccess. Si la autenticación
     *                 falla o se produce algún error se informa al callback
     *                 por medio de su método onError.
     */
    public void login(final String email, final String password, final ServiceCallback<Void> callback) {

        Backendless.UserService.login(email, password, new AsyncCallback<BackendlessUser>() {
            @Override
            public void handleResponse(BackendlessUser response) {
                callback.onSuccess(null);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getMessage()));
            }
        });
    }

    /**
     * Cierra la sesión del usuario actualmente autenticado en la aplicación.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la desconexión es correcta se informa al callback
     *                 por medio de su método onSuccess. Si se produce algún
     *                 error en la desconexión se informa al callback a través
     *                 de su método onError.
     */
    public void logout(final ServiceCallback<Void> callback) {
        Backendless.UserService.logout(new AsyncCallback<Void>() {
            @Override
            public void handleResponse(Void response) {
                callback.onSuccess(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getMessage()));
            }
        });
    }

    /**
     * Registra en el servidor Backendless una nueva cuenta de usuario.
     * @param user Datos del nuevo usuario a registrar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el usuario
     *                 insertado en la base de datos.
     */
    public void registerAccount(BackendlessUser user, final ServiceCallback<BackendlessUser> callback) {
        Backendless.UserService.register(user, new AsyncCallback<BackendlessUser>() {
            @Override
            public void handleResponse(BackendlessUser userResponse) {
                callback.onSuccess(userResponse);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Añade a un usuario en el servidor la información relativa
     * a su última localización conocida.
     * @param user Datos del usuario a actualizar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito y se asocia correctamente la información
     *                 de geolocalización al usuario, se informa al callback por medio
     *                 de su método onSuccess.
     */
    public void addInitialLocation(final BackendlessUser user, final ServiceCallback<Void> callback) {
        GeoPoint userLocation = (GeoPoint) user.getProperty(Usuario.LOCATION_KEY);
        // Registramos el objeto con la localización del usuario.
        Backendless.Geo.savePoint(userLocation, new AsyncCallback<GeoPoint>() {
            @Override
            public void handleResponse(GeoPoint geoResponse) {
                final List<GeoPoint> userLocation = Arrays.asList(geoResponse);
                // Asociamos el usuario con su localización.
                Backendless.Data.of(BackendlessUser.class).setRelation(user, Usuario.LOCATION_KEY, userLocation, new AsyncCallback<Integer>() {
                    @Override
                    public void handleResponse(Integer response) {
                        callback.onSuccess(null);
                    }

                    @Override
                    public void handleFault(BackendlessFault fault) {
                        callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
                    }
                });
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Actualiza en Backendless los datos de un usuario existente.
     * @param user Datos del usuario a actualizar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el usuario
     *                 actualizado en la base de datos.
     */
    public void updateUser(final BackendlessUser user, final ServiceCallback<BackendlessUser> callback) {
        String whereClause = "objectId='" + user.getObjectId() + "'";
        Map<String, Object> changes = new HashMap<>();
        changes.put(Usuario.NAME_KEY, user.getProperty(Usuario.NAME_KEY));
        changes.put(Usuario.BOOKS_NUMBER_KEY, user.getProperty(Usuario.BOOKS_NUMBER_KEY));
        changes.put(Usuario.RATING_KEY, user.getProperty(Usuario.RATING_KEY));

        Backendless.Data.of(BackendlessUser.class).update(whereClause, changes, new AsyncCallback<Integer>() {
            @Override
            public void handleResponse(Integer response) {
                callback.onSuccess(user);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Actualiza en Backendless las coordenadas de la última localización
     * conocida del usuario autenticado actualmente en la aplicación.
     * @param latitude Nueva latitud del usuario.
     * @param longitude Nueva longitud del usuario.
     */
    public void updateUserLastKnownLocation(double latitude, double longitude) {
        BackendlessUser currentUser = getCurrentUser();
        GeoPoint location = (GeoPoint) currentUser.getProperty(Usuario.LOCATION_KEY);
        location.setLatitude(latitude);
        location.setLongitude(longitude);

        // Actualizamos la información localmente en la aplicación.
        setCurrentUser(currentUser);

        // Actualizamos la localización remotamente en el servidor.
        Backendless.Geo.savePoint(location, new AsyncCallback<GeoPoint>() {
            @Override
            public void handleResponse(GeoPoint response) {
                Log.i("Updated location", response.getLatitude() + ", " + response.getLongitude());
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                Log.w("Location update error", fault.getDetail());
            }
        });
    }

    /**
     * Obtiene un usuario de Backendless a partir de su ID.
     * @param userId ID del usuario a buscar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el usuario
     *                 cuyo ID se pasa como parámetro.
     */
    public void findUserById(String userId, final ServiceCallback<BackendlessUser> callback) {
        Backendless.Data.of(BackendlessUser.class).findById(userId, new AsyncCallback<BackendlessUser>() {
            @Override
            public void handleResponse(BackendlessUser backendlessUser) {
                callback.onSuccess(backendlessUser);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                callback.onError(new ServiceFault(fault.getDetail(), fault.getCode()));
            }
        });
    }

    /**
     * Método auxiliar que obtiene la última localización conocida
     * del usuario autenticado actualmente en la aplicación.
     */
    private GeoPoint getCurrentUserCoordinates() {
        BackendlessUser currentUser = getCurrentUser();
        GeoPoint location = null;

        if(currentUser != null) {
            Object locationProperty = currentUser.getProperty(Usuario.LOCATION_KEY);

            // En condiciones normales se obtiene del currentUser.
            if(locationProperty instanceof GeoPoint) {
                location = (GeoPoint) currentUser.getProperty(Usuario.LOCATION_KEY);
            }
            // Pero en ocasiones, si al SDK de Backendless no le ha dado tiempo
            // a actualizar los datos del usuario actual en el SharedPreferences,
            // la devuelve a través de un mapa. Esto tiene pinta de ser un bug
            // del SDK de Backendless.
            else {
                Map<String, Object> locationAsMap = (Map) locationProperty;
                location = new GeoPoint((double) locationAsMap.get("latitude"), (double) locationAsMap.get("longitude"));
            }
        }

        return location;
    }
}
