package edu.uoc.jsev.b4b.ui.adapter.recyclerview;

import android.support.annotation.NonNull;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;

/**
 * Adaptador para mostrar elementos en listados
 * gestionados a través de un RecyclerView.
 * Es común a los dos tipos de listados existentes
 * en la aplicación: libros e intercambios.
 * Los datos mostrados en el listado no son gestionados
 * por el adaptador, sino a través de un presentador asociado,
 * al que se le solicitan los mismos.
 */
public class RecyclerViewAdapter extends RecyclerView.Adapter<RecyclerViewAdapter.ViewHolder> {

    // Posibles tipos de elementos de los listados.
    public static final int SWAP_VIEW_TYPE = 0; // Vista para intercambios.
    public static final int BOOK_VIEW_TYPE = 1; // Vista para libros.

    // Tipo de elemento mostrado en el listado.
    private int mViewType;

    // Presentador.
    private RecyclerViewContract.Presenter mPresenter;

    // Listener para gestionar las pulsaciones sobre los elementos del listado.
    private RecyclerViewItemClickListener mListener;

    public RecyclerViewAdapter(RecyclerViewContract.Presenter presenter, RecyclerViewItemClickListener listener, int viewType) {
        mPresenter = presenter;
        mListener = listener;
        mViewType = viewType;
    }

    @Override
    public int getItemViewType(int position) {
        // Devuelve el tipo de elemento mostrado en el listado,
        // para utilizar un tipo de vista u otro.
        return mViewType;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        // Inflamos el layout de la vista en función del tipo de elemento mostrado.
        int layoutResId;
        if(viewType == SWAP_VIEW_TYPE) {
            layoutResId = R.layout.cardview_list_swap;
        }
        else {
            layoutResId = R.layout.cardview_list_book;
        }

        View view = LayoutInflater.from(parent.getContext()).inflate(layoutResId, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull final ViewHolder holder, int position) {
        // Solicitamos al presentador los datos del elemento
        // situado en una posición determinada.
        mPresenter.bindDataAtPosition(holder, position);

        holder.mView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mListener != null) {
                    // Notificamos al callback (actividad o fragmento en el que se
                    // encuentra el listado) que el usuario ha pulsado sobre
                    // un elemento.
                    mListener.onItemClick(holder.mItem);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        // Solicitamos al presentador el número de elementos del listado.
        return mPresenter.getDataCount();
    }

    /**
     * Indica si el número de elementos es cero.
     * @return true si el número de elementos es cero,
     * false en caso contrario.
     */
    public boolean isEmpty() {
        return getItemCount() == 0;
    }

    /**
     * Holder para los elementos de los listados.
     * Implementa una interfaz RecyclerViewContract.RowView
     * para aislar completamente al presentador asociado
     * al adaptador de cualquier componente relacionado
     * con la plataforma Android, y así hacerlo independiente.
     */
    public class ViewHolder extends RecyclerView.ViewHolder implements RecyclerViewContract.RowView {
        // Referencias a las vistas.
        private final View mView;
        private final ImageView mImageView;
        private final TextView mTitleView;
        private final TextView mSubtitleView;
        private final TextView mExtraInfoView;

        // Entidad asociada al elemento.
        private UiEntity mItem;

        public ViewHolder(View view) {
            super(view);
            mView = view;
            mImageView = view.findViewById(R.id.image);
            mTitleView = view.findViewById(R.id.title);
            mSubtitleView = view.findViewById(R.id.subtitle);
            mExtraInfoView = view.findViewById(R.id.extra_info);
        }

        /**
         * @see RecyclerViewContract.RowView#setImage(String)
         */
        @Override
        public void setImage(String url) {
            Picasso.get()
                    .load(url)
                    .placeholder(R.drawable.book_placeholder)
                    .into(mImageView);
        }

        /**
         * @see RecyclerViewContract.RowView#setImage(int, int)
         */
        @Override
        public void setImage(int iconResId, int colorResId) {
            mImageView.setImageResource(iconResId);
            mImageView.setColorFilter(ContextCompat.getColor(mImageView.getContext(), colorResId));
        }

        /**
         * @see RecyclerViewContract.RowView#setTitle(String)
         */
        @Override
        public void setTitle(String title) {
            mTitleView.setText(title);
        }

        /**
         * @see RecyclerViewContract.RowView#setSubtitle(String)
         */
        @Override
        public void setSubtitle(String subtitle) {
            mSubtitleView.setText(subtitle);
        }

        /**
         * @see RecyclerViewContract.RowView#setExtraInfo(String)
         */
        @Override
        public void setExtraInfo(String extraInfo) {
            mExtraInfoView.setText(extraInfo);
        }

        /**
         * @see RecyclerViewContract.RowView#setData(UiEntity)
         */
        @Override
        public void setData(UiEntity data) {
            mItem = data;
        }
    }
}
