package edu.uoc.jsev.b4b.ui.adapter.recyclerview;

import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador
 * de aquellas pantallas que contienen listados
 * gestionados por un RecyclerView.
 */
public interface RecyclerViewContract {

    /**
     * Vista de las pantallas con listado.
     */
    interface View extends BaseContract.View {
        /**
         * Notifica al adaptador un cambio en los datos
         * del listado, para que se refresque la información
         * mostrada.
         */
        void refreshData();

        /**
         * Navega a la actividad de detalle, tras pulsar sobre
         * un elemento del listado, pasándole como parámetro
         * los datos del ítem pulsado.
         * @param item Datos del ítem pulsado.
         * @param params Parámetros adicionales que se pueden
         *               enviar a la actividad de detalle.
         */
        void navigateToDetail(UiEntity item, Object... params);

        /**
         * Muestra un mensaje indicando que se ha producido
         * un error al cargar los datos del listado.
         */
        void showLoadDataError();
    }

    /**
     * Vista de los elementos (ítems) del listado.
     */
    interface RowView {
        /**
         * Establece la imagen del elemento y su color,
         * a partir de un ID de recurso estático.
         * @param iconResId Id de recurso de la imagen.
         * @param colorResId Id de recurso del color
         *                   con el que se coloreará
         *                   la imagen.
         */
        void setImage(int iconResId, int colorResId);

        /**
         * Establece la imagen del elemento,
         * a partir de su URL de descarga.
         * @param url URL de descarga de la imagen.
         */
        void setImage(String url);

        /**
         * Establece el título del elemento.
         * @param title El título.
         */
        void setTitle(String title);

        /**
         * Establece el subtítulo del elemento.
         * @param subtitle El subtítulo.
         */
        void setSubtitle(String subtitle);

        /**
         * Establece la información adicional del elemento,
         * que se mostrará en la parte inferior del mismo.
         * @param extraInfo La información adicional.
         */
        void setExtraInfo(String extraInfo);

        /**
         * Establece los datos de la entidad asociada al elemento.
         * @param data Los datos de la entidad.
         */
        void setData(UiEntity data);
    }

    /**
     * Presentador de las pantallas con listado.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * Obtiene los datos a cargar en el listado.
         */
        void loadData();

        /**
         * El usuario ha pulsado sobre un elemento del listado.
         * @param data Los datos asociados al ítem del listado pulsado.
         */
        void dataClicked(UiEntity data);

        /**
         * Carga los datos asociados al elemento
         * de una determinada posición del listado.
         * @param rowView Vista del elemento del listado
         *                sobre la que se mostrarán los datos.
         * @param position Posición del listado a cargar.
         */
        void bindDataAtPosition(RowView rowView, int position);

        /**
         * Devuelve el número total de elementos
         * contenidos en el listado.
         * @return El número de elementos del listado.
         */
        int getDataCount();
    }
}
