package edu.uoc.jsev.b4b.ui.base;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.content.Context;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.view.Menu;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Toast;

import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;

/**
 * Actividad base de la que extienden todas las actividades de la aplicación.
 * Implementa una serie de funcionalidades comunes a todas ellas.
 */
public abstract class BaseActivity extends AppCompatActivity
        implements BaseContract.View, AlertDialogFragment.AlertDialogListener {

    // Duración por defecto de los mensajes de tipo Snackbar.
    public static final int SNACKBAR_LENGTH_DEFAULT = 6000;

    // Referencias a las vistas.
    protected FloatingActionButton mFab;
    protected View mProgressView;

    // Referencia al menú de la actividad.
    protected Menu mMenu;

    /**
     * @see BaseContract.View#showProgress(boolean)
     */
    @Override
    public void showProgress(final boolean show) {
        int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);

        if(mFab != null) mFab.setEnabled(!show);
        if(mMenu != null) mMenu.setGroupEnabled(0, !show);

        mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
        mProgressView.animate().setDuration(shortAnimTime).alpha(
                show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            }
        });
    }

    /**
     * @see BaseContract.View#hideKeyboard()
     */
    @Override
    public void hideKeyboard() {
        View view = this.getCurrentFocus();
        if (view != null) {
            InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

    /**
     * @see BaseContract.View#showToastMessage(String)
     */
    @Override
    public void showToastMessage(String message) {
        Toast.makeText(getApplicationContext(), message, Toast.LENGTH_LONG).show();
    }

    /**
     * @see BaseContract.View#showToastMessage(int)
     */
    @Override
    public void showToastMessage(int resId) {
        showToastMessage(getString(resId));
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogPositiveClick(String, Object...)
     */
    @Override
    public void onDialogPositiveClick(String tag, Object... values) {
        // Por defecto no hacemos nada.
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        // Por defecto no hacemos nada.
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogItemClick(String, int)
     */
    @Override
    public void onDialogItemClick(String tag, int which) {
        // Por defecto no hacemos nada.
    }

    /**
     * Muestra una ventana de diálogo con texto.
     * @param titleResId Id de recurso del título del diálogo.
     * @param messageResId Id de recurso del mensaje del diálogo.
     * @param tag Tag asociado para identificar al diálogo.
     */
    protected void showAlertDialog(int titleResId, int messageResId, String tag) {
        AlertDialogFragment dialog = AlertDialogFragment.newInstance(titleResId, messageResId);
        dialog.show(getSupportFragmentManager(), tag);
    }

    /**
     * Muestra una ventana de diálogo con una selección de opciones.
     * @param titleResId Id de recurso del título del diálogo.
     * @param items Ids de recurso de las opciones mostradas en el diálogo.
     * @param tag Tag asociado para identificar al diálogo.
     */
    protected void showOptionsDialog(int titleResId, int[] items, String tag) {
        AlertDialogFragment dialog = AlertDialogFragment.newInstance(titleResId, items);
        dialog.show(getSupportFragmentManager(), tag);
    }

    /**
     * Descarta y cierra el diálogo identificado por el tag pasado como parámetro.
     * @param tag Tag del diálogo a cerrar.
     */
    protected void dismissDialog(String tag) {
        Fragment fragment = getSupportFragmentManager().findFragmentByTag(tag);
        if(fragment != null) {
            ((DialogFragment) fragment).dismiss();
        }
    }
}
