package edu.uoc.jsev.b4b.ui.base;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Genero;
import edu.uoc.jsev.b4b.data.backendless.model.Idioma;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.uimodel.mapper.BookMapper;
import edu.uoc.jsev.b4b.data.uimodel.mapper.OpenLibraryMapper;
import edu.uoc.jsev.b4b.data.uimodel.mapper.SwapMapper;
import edu.uoc.jsev.b4b.data.uimodel.mapper.UserMapper;

/**
 * Implementación base de la que extienden todas las implementaciones
 * de interactores de la aplicación.
 * Implementa las funcionalidades comunes a todos ellos.
 */
public abstract class BaseInteractorImpl implements BaseInteractor {

    // Servicio de acceso al repositorio de usuarios.
    protected UserService mUserService;

    // Servicio de acceso a los repositorios horizontales.
    protected CommonsService mCommonsService;

    // Objetos para mapear entidades de UI <---> entidades de negocio.
    protected static UserMapper userMapper = UserMapper.getInstance();
    protected static BookMapper bookMapper = BookMapper.getInstance();
    protected static SwapMapper swapMapper = SwapMapper.getInstance();
    protected static OpenLibraryMapper openLibraryMapper = OpenLibraryMapper.getInstance();

    protected BaseInteractorImpl(UserService userService, CommonsService commonsService){
        this.mUserService = userService;
        this.mCommonsService = commonsService;
    }

    /**
     * @see BaseInteractor#getCurrentUserFullName()
     */
    @Override
    public String getCurrentUserFullName() {
        return mUserService.getCurrentUserFullName();
    }

    /**
     * @see BaseInteractor#getCurrentUserEmail()
     */
    @Override
    public String getCurrentUserEmail() {
        return mUserService.getCurrentUserEmail();
    }

    /**
     * @see BaseInteractor#getCurrentUserId()
     */
    @Override
    public String getCurrentUserId() {
        return mUserService.getCurrentUserId();
    }

    /**
     * @see BaseInteractor#getCurrentUserLatitude()
     */
    @Override
    public Double getCurrentUserLatitude() {
        return mUserService.getCurrentUserLatitude();
    }

    /**
     * @see BaseInteractor#getCurrentUserLongitude()
     */
    @Override
    public Double getCurrentUserLongitude() {
        return mUserService.getCurrentUserLongitude();
    }

    /**
     * @see BaseInteractor#getAllGenres(ServiceCallback)
     */
    @Override
    public void getAllGenres(final ServiceCallback<List<String>> callback) {
        mCommonsService.getAllGenres(new ServiceCallback<List<Genero>>() {
            @Override
            public void onSuccess(List<Genero> response) {
                List<String> genres = new ArrayList<>();
                for(Genero genre : response) {
                    genres.add(genre.getNombre());
                }
                callback.onSuccess(genres);
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /**
     * @see BaseInteractor#getAllLanguages(ServiceCallback)
     */
    @Override
    public void getAllLanguages(final ServiceCallback<List<String>> callback) {
        mCommonsService.getAllLanguages(new ServiceCallback<List<Idioma>>() {
            @Override
            public void onSuccess(List<Idioma> response) {
                List<String> languages = new ArrayList<>();
                for(Idioma language : response) {
                    languages.add(language.getNombre());
                }
                callback.onSuccess(languages);
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /**
     * @see BaseInteractor#uploadPicture(byte[], ServiceCallback)
     */
    @Override
    public void uploadPicture(byte[] picture, ServiceCallback<String> callback) {
        mCommonsService.uploadPicture(picture, getCurrentUserId(), callback);
    }

    /**
     * @see BaseInteractor#sendEmail(String, String, String...)
     */
    @Override
    public void sendEmail(final String subject, String message, String... recipients) {
        mCommonsService.sendEmail(subject, message, recipients);
    }
}
