package edu.uoc.jsev.b4b.ui.dialog;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.view.View;

import edu.uoc.jsev.b4b.R;

/**
 * Fragmento para generar una ventana de diálogo de forma genérica.
 */
public class AlertDialogFragment extends DialogFragment {

    // Parámetros de configuración del fragmento.
    private static final String ARG_TITLE = "title";
    private static final String ARG_MESSAGE = "message";
    private static final String ARG_ITEMS = "items";
    private static final String ARG_LAYOUT = "layout";

    // Vista personalizada del diálogo,
    // en caso de no mostrar texto u opciones de selección.
    protected View mCustomView;

    // Callback que gestionará los eventos de pulsación
    // sobre los elementos del diálogo.
    protected AlertDialogListener mListener;

    public AlertDialogFragment() {

    }

    // Métodos fábrica.
    public static AlertDialogFragment newInstance(int titleResId, int messageResId) {
        return newInstance(getArgs(titleResId, messageResId));
    }

    public static AlertDialogFragment newInstance(int titleResId, int[] items) {
        return newInstance(getArgs(titleResId, items));
    }

    public static AlertDialogFragment newInstance(int titleResId, int messageResId, int layoutResId) {
        return newInstance(getArgs(titleResId, messageResId, layoutResId));
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());

        // Construimos el diálogo a partir de los argumentos.
        // Título y mensaje.
        builder.setTitle(getString(getArguments().getInt(ARG_TITLE)));
        builder.setIcon(R.mipmap.ic_launcher);
        builder.setCancelable(true);
        if(getArguments().containsKey(ARG_MESSAGE)) {
            builder.setMessage(getString(getArguments().getInt(ARG_MESSAGE)));
        }

        // Vista personalizada, en caso de suministrarse.
        if(getArguments().containsKey(ARG_LAYOUT)) {
            int layoutId = getArguments().getInt(ARG_LAYOUT);
            mCustomView = getActivity().getLayoutInflater().inflate(layoutId, null);
            builder.setView(mCustomView);
        }

        // Lista de opciones, en caso de suministrarse.
        // Callbacks para atender las pulsaciones sobre los botones
        // del diálogo y/o sobre las opciones de selección.
        if(getArguments().containsKey(ARG_ITEMS)) {
            CharSequence sources[] = getSources(getArguments().getIntArray(ARG_ITEMS));
            builder.setItems(sources, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialogInterface, int which) {
                    mListener.onDialogItemClick(getTag(), which);
                }
            });
        }
        else {
            builder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialogInterface, int i) {
                    mListener.onDialogPositiveClick(getTag());
                }
            });
        }

        builder.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                mListener.onDialogNegativeClick(getTag());
            }
        });

        return builder.create();
    }

    @Override
    public void show(FragmentManager manager, String tag) {
        // Mostramos el diálogo.
        Fragment fragment = manager.findFragmentByTag(tag);
        if(fragment == null) {
            super.show(manager, tag);
        }
    }

    @Override
    public void onCancel(DialogInterface dialog) {
        // Cancelar el diálogo (pulsar fuera de él o pulsar sobre el botón "Back")
        // es equivalente a pulsar sobre el botón "Cancelar".
        mListener.onDialogNegativeClick(getTag());
    }

    @Override
    public void onAttach(Context context) {
        // El contexto contenedor debe implementar los métodos
        // para poder gestionar los eventos sobre el diálogo.
        super.onAttach(context);
        try {
            mListener = (AlertDialogListener) context;
        }
        catch (ClassCastException e) {
            throw new ClassCastException(context.toString() + " debe implementar AlertDialogListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    // Métodos auxiliares para construir el diálogo a partir de los argumentos.
    protected static Bundle getArgs(int titleResId, int messageResId) {
        Bundle args = new Bundle();
        args.putInt(ARG_TITLE, titleResId);
        args.putInt(ARG_MESSAGE, messageResId);
        return args;
    }

    protected static Bundle getArgs(int titleResId, int[] items) {
        Bundle args = new Bundle();
        args.putInt(ARG_TITLE, titleResId);
        args.putIntArray(ARG_ITEMS, items);
        return args;
    }

    protected static Bundle getArgs(int titleResId, int messageResId, int layoutResId) {
        Bundle args = getArgs(titleResId, messageResId);
        args.putInt(ARG_LAYOUT, layoutResId);
        return args;
    }

    private static AlertDialogFragment newInstance(Bundle args) {
        AlertDialogFragment fragment = new AlertDialogFragment();
        fragment.setArguments(args);
        return fragment;
    }

    private CharSequence[] getSources(int[] items) {
        CharSequence sources[] = new CharSequence[items.length];
        for(int i=0; i<items.length; i++) {
            sources[i] = getString(items[i]);
        }

        return sources;
    }

    /**
     * Interfaz que debe implementar la actividad o fragmento
     * desde el que se cree el diálogo para gestionar los eventos
     * que se generen sobre él.
     */
    public interface AlertDialogListener {
        /**
         * Evento lanzado cuando se pulsa sobre el botón positivo del diálogo.
         * @param tag Tag que identifica al diálogo.
         * @param values En caso de disponer de un layout personalizado,
         *               el diálogo devolverá en este campo los valores
         *               almacenados en las vistas contenidas en el layout.
         */
        void onDialogPositiveClick(String tag, Object... values);

        /**
         * Evento lanzado cuando se pulsa sobre el botón negativo del diálogo.
         * @param tag Tag que identifica al diálogo.
         */
        void onDialogNegativeClick(String tag);

        /**
         * Evento lanzado cuando se pulsa sobre una de las opciones de selección
         * del diálogo.
         * @param tag Tag que identifica al diálogo.
         * @param which Entero que identifica la opción pulsada.
         */
        void onDialogItemClick(String tag, int which);
    }
}
