package edu.uoc.jsev.b4b.ui.main;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.NavigationView;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;
import edu.uoc.jsev.b4b.ui.login.LoginActivity;
import edu.uoc.jsev.b4b.ui.map.MapFragment;
import edu.uoc.jsev.b4b.ui.mybooks.MyBooksFragment;
import edu.uoc.jsev.b4b.ui.swaps.SwapsFragment;
import edu.uoc.jsev.b4b.Injection;

/**
 * Actividad de la pantalla principal de la aplicación.
 * A través de un NavigationView se encarga de gestionar
 * los accesos a las opciones principales, que se cargan
 * por medio de fragments. El contenido de los fragments
 * se carga la primera vez que se accede a cada opción,
 * para mejorar el rendimiento. Esto se consigue mostrando
 * y ocultando los fragments, en lugar de crearlos cada vez.
 */
public class MainActivity extends BaseActivity
        implements MainContract.View,
        NavigationView.OnNavigationItemSelectedListener,
        MainFragment.OnMainFragmentInteractionListener {

    // Nombre del extra que puede recibir la actividad desde otras pantallas.
    public static final String EXTRA_DATA_OBJECT = "dataObject";

    // Tags para identificar a los diferentes fragments.
    private static final String TAG_MAP_FRAGMENT = "mapFragment";
    private static final String TAG_MYBOOKS_FRAGMENT = "myBooksFragment";
    private static final String TAG_SWAPS_FRAGMENT = "swapsFragment";
    private static final String TAG_LOGOUT_DIALOG_FRAGMENT = "logoutDialog";

    // Referencias a las vistas.
    private Toolbar mToolbar;
    private NavigationView mNavigationView;
    private DrawerLayout mDrawer;
    private TextView mCurrentUserFullNameView;
    private TextView mCurrentUserEmailView;

    // Referencia al fragment visible actualmente.
    private MainFragment mCurrentFragment;

    // Presentador.
    private MainContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Inyectamos el presentador.
        mPresenter = Injection.provideMainPresenter(this);

        // Configuramos las vistas.
        setupFab();
        setupToolbar();
        setupDrawer();
        setupNavMenu();

        mProgressView = findViewById(R.id.progress);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();
    }

    @Override
    public void onBackPressed() {
        // Informamos al presentador de que el usuario ha pulsado el botón "Back".
        mPresenter.backButtonPressed();
    }

    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem item) {
        int id = item.getItemId();

        // Informamos al presentador cuando el usuario pulsa
        // sobre una de las opciones del menú de navegación principal.
        if (id == R.id.nav_map) {
            mPresenter.mapOptionClicked();
        }
        else if (id == R.id.nav_books) {
            mPresenter.myBooksOptionClicked();
        }
        else if (id == R.id.nav_swaps) {
            mPresenter.swapsOptionClicked();
        }
        else if (id == R.id.nav_exit) {
            mPresenter.logoutOptionClicked();
        }

        return true;
    }

    @Override
    public void onNewIntent(Intent intent) {
        super.onNewIntent(intent);

        if (intent != null && intent.getAction() != null) {
            String action = intent.getAction();
            UiEntity dataObject = (UiEntity) intent.getSerializableExtra(EXTRA_DATA_OBJECT);

            // Se vuelve de una actividad que solicita que se refresque el listado con un nuevo objeto.
            if(action.equals(Intent.ACTION_INSERT)) {
                mCurrentFragment.addObjectToList(dataObject);
            }
            // Se vuelve de una actividad que solicita que se refresque un determinado objeto del listado.
            else if(action.equals(Intent.ACTION_EDIT)) {
                mCurrentFragment.updateObjectInList(dataObject);
            }
            // Se vuelve de una actividad que solicita que se refresque el listado eliminando un objeto.
            else if(action.equals(Intent.ACTION_DELETE)) {
                mCurrentFragment.removeObjectFromList(dataObject);
            }
            // Se vuelve de una actividad que solicita que se refresque un listado que no está visible con un nuevo objeto.
            // Esta situación solo se va a producir cuando el usuario solicite un nuevo intercambio. Al volver está
            // activo el fragment del mapa. Mostramos el de intercambios y añadimos el nuevo objeto a la lista.
            else if(action.equals(Intent.ACTION_PICK_ACTIVITY)) {
                String tag = null;
                if(dataObject instanceof UiSwap) {
                    tag = TAG_SWAPS_FRAGMENT;
                }

                if(tag != null) {
                    MainFragment newCurrentFragment = (MainFragment) getSupportFragmentManager().findFragmentByTag(tag);
                    mPresenter.swapsOptionClicked();
                    if(newCurrentFragment != null) {
                        mCurrentFragment.addObjectToList(dataObject);
                    }
                }
            }
        }
    }

    /**
     * @see MainContract.View#fabButtonClicked()
     */
    @Override
    public void fabButtonClicked() {
        if(mCurrentFragment != null) {
            mCurrentFragment.fabButtonClicked();
        }
    }

    /**
     * @see MainContract.View#setCurrentUserFullName(String)
     */
    @Override
    public void setCurrentUserFullName(String fullName) {
        mCurrentUserFullNameView.setText(fullName);
    }

    /**
     * @see MainContract.View#setCurrentUserEmail(String)
     */
    @Override
    public void setCurrentUserEmail(String email) {
        mCurrentUserEmailView.setText(email);
    }

    /**
     * @see MainContract.View#backButtonNormalAction()
     */
    @Override
    public void backButtonNormalAction() {
        super.onBackPressed();
    }

    /**
     * @see MainContract.View#showMap()
     */
    @Override
    public void showMap() {
        mNavigationView.setCheckedItem(R.id.nav_map);
        loadFragment(TAG_MAP_FRAGMENT);
    }

    /**
     * @see MainContract.View#showMyBooks()
     */
    @Override
    public void showMyBooks() {
        mNavigationView.setCheckedItem(R.id.nav_books);
        loadFragment(TAG_MYBOOKS_FRAGMENT);
    }

    /**
     * @see MainContract.View#showSwaps()
     */
    @Override
    public void showSwaps() {
        mNavigationView.setCheckedItem(R.id.nav_swaps);
        loadFragment(TAG_SWAPS_FRAGMENT);
    }

    /**
     * @see MainContract.View#showLogoutDialog()
     */
    @Override
    public void showLogoutDialog() {
        showAlertDialog(R.string.action_logout, R.string.prompt_logout_confirmation, TAG_LOGOUT_DIALOG_FRAGMENT);
    }

    /**
     * @see MainContract.View#showLogoutError()
     */
    @Override
    public void showLogoutError() {
        showToastMessage(R.string.error_logout);
    }

    /**
     * @see MainContract.View#dismissLogoutDialog()
     */
    @Override
    public void dismissLogoutDialog() {
        dismissDialog(TAG_LOGOUT_DIALOG_FRAGMENT);
    }

    /**
     * @see MainContract.View#isNavigationDrawerOpen()
     */
    @Override
    public boolean isNavigationDrawerOpen() {
        return mDrawer.isDrawerOpen(GravityCompat.START);
    }

    /**
     * @see MainContract.View#closeNavigationDrawer()
     */
    @Override
    public void closeNavigationDrawer() {
        mDrawer.closeDrawer(GravityCompat.START);
    }

    /**
     * @see MainContract.View#navigateToLogin()
     */
    @Override
    public void navigateToLogin() {
        startActivity(new Intent(this, LoginActivity.class));
        finish();
    }

    /**
     * @see MainFragment.OnMainFragmentInteractionListener#setTitle(String)
     */
    @Override
    public void setTitle(String title) {
        mToolbar.setTitle(title);
    }

    /**
     * @see MainFragment.OnMainFragmentInteractionListener#showFabButton(boolean)
     */
    @Override
    public void showFabButton(boolean show) {
        if(show) {
            mFab.show();
        }
        else {
            mFab.hide();
        }
    }

    /**
     * @see MainFragment.OnMainFragmentInteractionListener#setFabButtonIcon(int)
     */
    @Override
    public void setFabButtonIcon(int resId) {
        mFab.setImageDrawable(getResources().getDrawable(resId));
    }

    /**
     * Gestiona la pulsación sobre el botón "Aceptar" de los diálogos.
     * @see AlertDialogFragment.AlertDialogListener#onDialogPositiveClick(String, Object...)
     */
    @Override
    public void onDialogPositiveClick(String tag, Object... values) {
        // El usuario pulsa "Aceptar" sobre el diálogo de logout.
        // Este diálogo lo abre la actividad.
        if(tag.equals(TAG_LOGOUT_DIALOG_FRAGMENT)) {
            mPresenter.logout();
        }
        // En cualquier otro caso, es el fragmento visible el que ha abierto el diálogo.
        // Se delega en él la gestión del evento.
        else {
            mCurrentFragment.onDialogPositiveClick(tag, values);
        }
    }

    /**
     * Gestiona la pulsación sobre el botón "Cancelar" de los diálogos.
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        // El usuario pulsa "Cancelar" sobre el diálogo de logout.
        // Este diálogo lo abre la actividad.
        if (tag.equals(TAG_LOGOUT_DIALOG_FRAGMENT)) {
            mPresenter.logoutCanceled();
        }
        // En cualquier otro caso, es el fragmento visible el que ha abierto el diálogo.
        // Se delega en él la gestión del evento.
        else {
            mCurrentFragment.onDialogNegativeClick(tag);
        }
    }

    /*** MÉTODOS AUXILIARES ***/


    /**
     * Configuración del botón flotante.
     */
    private void setupFab() {
        mFab = findViewById(R.id.fab);
        mFab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.mainActionClicked();
            }
        });
    }

    /**
     * Configuración de la barra de herramientas.
     */
    private void setupToolbar() {
        mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
    }

    /**
     * Configuración del panel lateral de navegación.
     */
    private void setupDrawer() {
        mDrawer = findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, mDrawer, mToolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        mDrawer.addDrawerListener(toggle);
        toggle.syncState();
    }

    /**
     * Configuración de las opciones de navegación principal.
     */
    private void setupNavMenu() {
        mNavigationView = findViewById(R.id.nav_view);
        mNavigationView.setNavigationItemSelectedListener(this);

        View headerView = mNavigationView.getHeaderView(0);
        mCurrentUserFullNameView = headerView.findViewById(R.id.current_user_name);
        mCurrentUserEmailView = headerView.findViewById(R.id.current_user_email);
    }

    /**
     * Método auxiliar para cargar/mostrar el fragment seleccionado por el usuario.
     * El contenido de los fragments se carga la primera vez que se accede a cada
     * opción, para mejorar el rendimiento. Esto se consigue mostrando y ocultando
     * los fragments, en lugar de crearlos cada vez.
     * @param tag Tag del fragment a cargar/mostrar.
     */
    private void loadFragment(String tag) {
        // Si pulsamos la opción que ya está seleccionada, no hacemos nada.
        if(mCurrentFragment != null && mCurrentFragment.getTag().equals(tag)) {
            return;
        }

        FragmentManager fragmentManager = getSupportFragmentManager();
        MainFragment selectedFragment = (MainFragment) fragmentManager.findFragmentByTag(tag);
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();

        // Si se pulsa la opción por primera vez, cargamos el fragment correspondiente.
        if(selectedFragment == null) {
            if (tag.equals(TAG_MAP_FRAGMENT)) {
                selectedFragment = MapFragment.newInstance();
            } else if (tag.equals(TAG_MYBOOKS_FRAGMENT)) {
                selectedFragment = MyBooksFragment.newInstance();
            } else if (tag.equals(TAG_SWAPS_FRAGMENT)) {
                selectedFragment = SwapsFragment.newInstance();
            }
            fragmentTransaction.add(R.id.fragment_container, selectedFragment, tag);
        }
        // Si no, mostramos la opción pulsada y ocultamos la anterior.
        else {
            fragmentTransaction.show(selectedFragment);
        }

        if(mCurrentFragment != null) {
            fragmentTransaction.hide(mCurrentFragment);
        }

        fragmentTransaction.commit();
        mCurrentFragment = selectedFragment;
    }
}
