package edu.uoc.jsev.b4b.ui.main;

import android.content.Context;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewAdapter;
import edu.uoc.jsev.b4b.ui.base.BaseFragment;

/**
 * Fragmento del que extienden todos los fragmentos que se cargan en la actividad principal.
 * Implementa una serie de funcionalidades comunes a todos ellos.
 */
public abstract class MainFragment extends BaseFragment implements SwipeRefreshLayout.OnRefreshListener {

    // Callback para gestionar los eventos generados en el fragment.
    protected OnMainFragmentInteractionListener mCallback;

    // Referencia a las vistas.
    protected SwipeRefreshLayout mSwipeRefreshLayout;
    protected View mEmptyListView;
    protected RecyclerView mRecyclerView;
    protected RecyclerViewAdapter mAdapter;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);

        // Nos aseguramos de que la actividad contenedora implementa la interfaz de callback.
        // En caso contrario, lanzamos una excepción.
        try {
            mCallback = (OnMainFragmentInteractionListener) context;
        }
        catch (ClassCastException e) {
            throw new ClassCastException(context.toString() + " debe implementar OnMainFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mCallback = null;
    }

    @Override
    public void onRefresh() {
        // Implementación por defecto: ya se muestra el ProgressDialog,
        // así que desactivamos el refresco del SwipeRefreshLayout.
        if(mSwipeRefreshLayout != null) {
            mSwipeRefreshLayout.setRefreshing(false);
        }
    }

    /**
     * Método que utilizará la actividad contenedora para indicarle
     * al fragment que el usuario ha pulsado sobre el botón flotante.
     * Cada fragment proporcionará su implementación concreta de este
     * método.
     */
    public abstract void fabButtonClicked();

    /**
     * Método que utilizará la actividad contenedora para indicarle
     * al fragment que añada un objeto al listado, en caso de que
     * el fragment esté mostrando un RecyclerView.
     * @param dataObject El objeto a añadir al listado.
     */
    public void addObjectToList(UiEntity dataObject) {
        // Implementación por defecto: no hacemos nada.
        // La implementación concreta del fragment proporcionará
        // el comportamiento según sea necesario.
    }

    /**
     * Método que utilizará la actividad contenedora para indicarle
     * al fragment que actualice los datos de un objeto del listado,
     * en caso de que el fragment esté mostrando un RecyclerView.
     * @param dataObject El objeto a actualizar en el listado.
     */
    public void updateObjectInList(UiEntity dataObject) {
        // Implementación por defecto: no hacemos nada.
        // La implementación concreta del fragment proporcionará
        // el comportamiento según sea necesario.
    }

    /**
     * Método que utilizará la actividad contenedora para indicarle
     * al fragment que elimine un objeto del listado, en caso de que
     * el fragment esté mostrando un RecyclerView.
     * @param dataObject El objeto a eliminar del listado.
     */
    public void removeObjectFromList(UiEntity dataObject) {
        // Implementación por defecto: no hacemos nada.
        // La implementación concreta del fragment proporcionará
        // el comportamiento según sea necesario.
    }

    /**
     * Configura el RecyclerView, en caso de existir.
     */
    protected void setupRecyclerView(View view) {
        mSwipeRefreshLayout = view.findViewById(R.id.refresh_layout);
        mSwipeRefreshLayout.setColorSchemeResources(R.color.colorPrimary, R.color.colorPrimaryLight, R.color.colorPrimaryDark);
        mSwipeRefreshLayout.setOnRefreshListener(this);

        mRecyclerView = view.findViewById(R.id.list);
        mRecyclerView.setLayoutManager(new LinearLayoutManager(view.getContext()));
        mRecyclerView.setAdapter(mAdapter);

        mEmptyListView = view.findViewById(R.id.empty_list);
    }

    /**
     * Muestra/oculta un mensaje de lista vacía,
     * en caso de existir un RecyclerView sin elementos.
     * @param show Si es true, se muestra el mensaje de lista vacía,
     *             si es false, se muestran los elementos del listado.
     */
    protected void showEmptyList(boolean show) {
        mEmptyListView.setVisibility(show ? View.VISIBLE : View.GONE);
        mRecyclerView.setVisibility(show ? View.GONE : View.VISIBLE);
    }

    /**
     * Las actividades que contienen este fragmento deben implementar esta interfaz,
     * para comunicarles las interacciones que tengan lugar en el fragmento.
     */
    public interface OnMainFragmentInteractionListener {
        /**
         * Establece el título de la actividad contenedora.
         * @param title El título a establecer.
         */
        void setTitle(String title);

        /**
         * Muestra/oculta el botón flotante de la actividad contenedora.
         * @param show true para mostrar el botón, false para ocultarlo.
         */
        void showFabButton(boolean show);

        /**
         * Establece el icono del botón flotante de la actividad contenedora.
         * @param resId Id de recurso del icono a establecer.
         */
        void setFabButtonIcon(int resId);
    }
}
