package edu.uoc.jsev.b4b.ui.map;

import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla del mapa.
 */
public interface MapContract {

    /**
     * Vista de la pantalla del mapa.
     */
    interface View extends BaseContract.View {
        /**
         * Establece el título de la pantalla.
         */
        void setTitle();

        /**
         * Muestra/oculta el botón flotante.
         * @param show true para mostrarlo, false para ocultarlo.
         */
        void showFabButton(boolean show);

        /**
         * Comprueba si se tiene permiso para acceder
         * a la localización del dispositivo.
         * @return true si se tiene permiso, false en
         * caso contrario.
         */
        boolean isLocationPermissionGranted();

        /**
         * Solicita al usuario permiso para acceder
         * a su localización. Si ya se le había solicitado
         * antes pero lo denegó, se le vuelve a pedir
         * mediante un mensaje explicativo de por qué
         * es necesario el permiso.
         */
        void requestLocationPermission();

        /**
         * Solicita al usuario permiso para acceder
         * a su localización.
         */
        void requestLocationPermissionNoRationale();

        /**
         * Comprueba si el servicio de localización
         * del dispositivo está o no activado.
         * @return true si lo está, false en caso contrario.
         */
        boolean isLocationServiceEnabled();

        /**
         * Muestra un diálogo para preguntar al usuario
         * si desea activar el servicio de localización,
         * en caso de que esté desactivado.
         */
        void showLocationSettingsDialog();

        /**
         * Navega a la configuración de los servicios
         * de localización del dispositivo, para que
         * el usuario los active.
         */
        void navigateToLocationSettings();

        /**
         * Cierra el diálogo para abrir la configuración
         * de los servicios de localización.
         */
        void dismissLocationSettingsDialog();

        /**
         * Obtiene la ubicación actual del dispositivo
         * haciendo uso de sus servicios de localización.
         */
        void getDeviceLocation();

        /**
         * Detiene la recepción de actualizaciones de la
         * localización del dispositivo.
         */
        void stopLocationUpdates();

        /**
         * Centra el mapa en la posición pasada como parámetro
         * y con el nivel de zoom especificado.
         * @param latitude Latitud de la posición en la que centrar el mapa.
         * @param longitude Longitud de la posición en la que centrar el mapa.
         * @param zoom Nivel de zoom del mapa.
         */
        void centerMap(double latitude, double longitude, int zoom);

        /**
         * Elimina del mapa todos los marcadores existentes.
         */
        void removeAllMarkers();

        /**
         * Añade al mapa un nuevo marcador de usuario.
         * @param name Nombre del usuario.
         * @param books Número de libros compartidos por el usuario.
         * @param rating Valoración media del usuario.
         * @param latitude Latitud de la posición del usuario.
         * @param longitude Longitud de la posición del usuario.
         * @param tag Identificador del marcador. En él se establecerá
         *            el ID del usuario.
         */
        void addMarker(String name, String books, String rating, double latitude, double longitude, String tag);

        /**
         * Muestra un mensaje indicando que no se han encontrado
         * usuarios con los criterios de búsqueda especificados.
         */
        void showNoUsersFoundMessage();

        /**
         * Muestra un mensaje indicando que se ha producido
         * un error al realizar la búsqueda de usuarios.
         */
        void showUsersSearchError();

        /**
         * Navega hacia la pantalla de detalle de usuario.
         * @param user Datos del usuario sobre el que se ha pulsado,
         *             para mostrarlos en la actividad destino.
         */
        void navigateToUserDetail(UiUser user);
    }

    /**
     * Presentador de la pantalla del mapa.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * Inicializa el mapa.
         */
        void setupMap();

        /**
         * El usuario ha concedido permiso para acceder a su localización.
         */
        void locationPermissionGranted();

        /**
         * El usuario ha denegado el permiso para acceder a su localización.
         */
        void locationPermissionDenied();

        /**
         * El usuario ha pulsado sobre el botón de acción tras
         * solicitarle de nuevo el permiso para acceder a su localización.
         */
        void locationPermissionRationaleActionClicked();

        /**
         * El usuario ha descartado el mensaje tras solicitarle
         * de nuevo el permiso para acceder a su localización.
         */
        void locationPermissionRationaleDismissed();

        /**
         * El usuario quiere abrir la configuración
         * de los servicios de localización para activarlos.
         */
        void locationSettingsOk();

        /**
         * El usuario no quiere abrir la configuración
         * de los servicios de localización para activarlos.
         */
        void locationSettingsCanceled();

        /**
         * Se recibe una actualización de la localización
         * actual del usuario.
         * @param latitude Latitud de la localización.
         * @param longitude Longitud de la localización.
         * @param validLocation Boolean que indica si la actualización
         *                      es o no fiable.
         */
        void updateUserLocation(double latitude, double longitude, boolean validLocation);

        /**
         * El usuario ha regresado de la pantalla de congfiguración
         * de los servicios de localización del dispositivo.
         */
        void navigatedFromLocationSettings();

        /**
         * El usuario solicita realizar una búsqueda.
         * @param title Título a buscar.
         * @param author Autor a buscar.
         * @param genre Género literario a buscar.
         * @param distance Distancia máxima en la que buscar.
         */
        void search(String title, String author, String genre, int distance);

        /**
         * El usuario ha pulsado sobre la ventana de información
         * de un marcador de usuario ubicado sobre el mapa.
         * @param tag El identificador de la ventana de información pulsada,
         *            que coincide con el Id del usuario asociado.
         */
        void infoWindowClicked(String tag);
    }
}
