package edu.uoc.jsev.b4b.ui.map;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.adapter.map.MapInfoWindowAdapter;
import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;
import edu.uoc.jsev.b4b.ui.main.MainFragment;
import edu.uoc.jsev.b4b.ui.map.search.SearchFragment;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailActivity;
import edu.uoc.jsev.b4b.util.ContextUtils;
import edu.uoc.jsev.b4b.Injection;

/**
 * Fragmento para mostrar en un mapa la ubicación
 * de otros usuarios que comparten libros.
 * El fragmento se encarga también de determinar
 * la localización actual del usuario, para realizar
 * las búsquedas respecto a esa posición.
 */
public class MapFragment extends MainFragment
        implements MapContract.View, SearchFragment.OnInteractionListener, OnMapReadyCallback {

    // Códigos para solicitar permisos y recibir respuestas de actividades.
    private static final int LOCATION_PERMISSION_REQUEST_CODE = 100;
    private static final int LOCATION_SETTINGS_REQUEST_CODE = 101;

    // Tags para identificar los diálogos.
    private static final String TAG_LOCATION_SETTINGS_DIALOG_FRAGMENT = "locationSettingsDialog";

    // Referencias a las vistas.
    private ViewGroup mRootView;
    private SupportMapFragment mMapFragment;
    private GoogleMap mMap;

    // Cliente para determinar la localización actual del usuario.
    private FusedLocationProviderClient mFusedLocationProviderClient;
    private LocationCallback mLocationCallback;

    // Presentador.
    private MapContract.Presenter mPresenter;

    public MapFragment() {
        // Required empty public constructor
    }

    // Método fábrica.
    public static MapFragment newInstance() {
        return new MapFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Inicializamos el cliente para obtener la ubicación.
        mFusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(getContext());

        // Inyectamos el presentador.
        mPresenter = Injection.provideMapPresenter(this);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_map, container, false);

        // Referencias a las vistas.
        mRootView = view.findViewById(R.id.map);

        // Añadimos el mapa.
        if (mMapFragment == null) {
            mMapFragment = SupportMapFragment.newInstance();
            mMapFragment.getMapAsync(this);
        }

        getChildFragmentManager().beginTransaction()
                .replace(R.id.map, mMapFragment)
                .commit();

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();

        return view;
    }

    @Override
    public void onHiddenChanged(boolean hidden) {
        // Volvemos a inicializar la pantalla (título y acción del FAB)
        // cada vez que el fragmento vuelve a estar visible.
        // Como solo se carga la primera vez, solo entra una vez por
        // onCreateView.
        super.onHiddenChanged(hidden);
        if(!hidden) mPresenter.setup();
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        // Una vez se ha cargado el mapa, lo configuramos.
        mMap = googleMap;
        mMap.getUiSettings().setMapToolbarEnabled(false);
        mMap.getUiSettings().setMyLocationButtonEnabled(false);

        // Adaptador personalizado al pulsar sobre un marcador.
        mMap.setInfoWindowAdapter(new MapInfoWindowAdapter(getContext()));
        mMap.setOnInfoWindowClickListener(new GoogleMap.OnInfoWindowClickListener() {
            @Override
            public void onInfoWindowClick(Marker marker) {
                mPresenter.infoWindowClicked(marker.getTag().toString());
            }
        });

        // Callback para recibir actualizaciones de la ubicación del usuario.
        mLocationCallback = new LocationCallback() {
            @Override
            public void onLocationResult(LocationResult locationResult) {
                super.onLocationResult(locationResult);

                boolean validLocation = false;
                double latitude = 0d;
                double longitude = 0d;

                // Comprobamos la localización recibida.
                if(locationResult != null && locationResult.getLastLocation() != null) {
                    latitude = locationResult.getLastLocation().getLatitude();
                    longitude = locationResult.getLastLocation().getLongitude();
                    validLocation = true;
                }

                // Informamos al presentador de que se ha recibido una nueva localización.
                mPresenter.updateUserLocation(latitude, longitude, validLocation);
            }
        };

        // Solicitamos al presentador que inicialice el mapa.
        mPresenter.setupMap();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        // Se vuelve de solicitar el permiso para acceder a la localización.
        if (requestCode == LOCATION_PERMISSION_REQUEST_CODE) {
            // El usuario da permiso de acceso a la localización.
            if (grantResults.length == 1
                    && permissions[0].equals(Manifest.permission.ACCESS_FINE_LOCATION)
                    && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                mPresenter.locationPermissionGranted();
            }
            // El usuario deniega el permiso de acceso a la localización.
            else {
                mPresenter.locationPermissionDenied();
            }
        }
        else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        // Se vuelve desde la configuración de la localización.
        if(requestCode == LOCATION_SETTINGS_REQUEST_CODE) {
            // Indicamos al presentador que se ha vuelto
            // desde la configuración de la localización,
            // para que compruebe si se ha activado o no.
            mPresenter.navigatedFromLocationSettings();
        }
        else {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }

    /**
     * @see MainFragment#fabButtonClicked()
     */
    @Override
    public void fabButtonClicked() {
        // Abrimos la ventana de búsqueda.
        SearchFragment fragment = SearchFragment.newInstance();
        fragment.show(getChildFragmentManager(), fragment.getTag());
    }

    /**
     * @see MapContract.View#setTitle()
     */
    @Override
    public void setTitle() {
        // Delega en la actividad contenedora el cambio.
        mCallback.setTitle(getString(R.string.app_name));
    }

    /**
     * @see MapContract.View#showFabButton(boolean)
     */
    @Override
    public void showFabButton(boolean show) {
        // Delega en la actividad contenedora la configuración del botón.
        mCallback.setFabButtonIcon(R.drawable.ic_search_white_24dp);
        mCallback.showFabButton(show);
    }

    /**
     * @see MapContract.View#isLocationPermissionGranted()
     */
    @Override
    public boolean isLocationPermissionGranted() {
        return ContextUtils.isPermissionGranted(getContext(), Manifest.permission.ACCESS_FINE_LOCATION);
    }

    /**
     * @see MapContract.View#requestLocationPermission()
     */
    @Override
    public void requestLocationPermission() {
        // Si ya hemos solicitado anteriormente el permiso de acceso a la localización
        // mostramos al usuario un mensaje explicado que es necesario acceder a él.
        if (shouldShowRequestPermissionRationale(Manifest.permission.ACCESS_FINE_LOCATION)) {

            Snackbar snackbar = Snackbar.make(mRootView, R.string.prompt_location_permission_required, SNACKBAR_LENGTH_DEFAULT);
            snackbar.setAction(android.R.string.ok, new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    // Informamos al presentador de que el usuario ha pulsado
                    // sobre el botón de acción del Snackbar.
                    mPresenter.locationPermissionRationaleActionClicked();
                }
            });
            snackbar.addCallback(new Snackbar.Callback() {
                @Override
                public void onDismissed(Snackbar transientBottomBar, int event) {
                    // Informamos al presentador de que el usuario ha descartado
                    // el Snackbar sin pulsar en el botón de acción.
                    if(event != DISMISS_EVENT_ACTION) {
                        mPresenter.locationPermissionRationaleDismissed();
                    }
                }
            });
            snackbar.show();
        }
        // Si no se ha pedido antes, solicitamos acceder a la localización.
        else {
            requestLocationPermissionNoRationale();
        }
    }

    /**
     * @see MapContract.View#requestLocationPermissionNoRationale()
     */
    @Override
    public void requestLocationPermissionNoRationale() {
        requestPermissions(new String[]{ Manifest.permission.ACCESS_FINE_LOCATION }, LOCATION_PERMISSION_REQUEST_CODE);
    }

    /**
     * @see MapContract.View#isLocationServiceEnabled()
     */
    @Override
    public boolean isLocationServiceEnabled() {
        return ContextUtils.isLocationEnabled(getContext());
    }

    /**
     * @see MapContract.View#showLocationSettingsDialog()
     */
    @Override
    public void showLocationSettingsDialog() {
        showAlertDialog(R.string.action_location_settings, R.string.prompt_location_settings, TAG_LOCATION_SETTINGS_DIALOG_FRAGMENT);
    }

    /**
     * @see MapContract.View#navigateToLocationSettings()
     */
    @Override
    public void navigateToLocationSettings() {
        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
        startActivityForResult(intent, LOCATION_SETTINGS_REQUEST_CODE);
    }

    /**
     * @see MapContract.View#dismissLocationSettingsDialog()
     */
    @Override
    public void dismissLocationSettingsDialog() {
        dismissDialog(TAG_LOCATION_SETTINGS_DIALOG_FRAGMENT);
    }

    /**
     * @see MapContract.View#getDeviceLocation()
     */
    @Override
    public void getDeviceLocation() {
        try {
            // Solicitamos actualizaciones de ubicación del dispositivo.
            LocationRequest locationRequest = new LocationRequest();
            locationRequest.setInterval(5000);
            locationRequest.setFastestInterval(2500);
            locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);

            mFusedLocationProviderClient.requestLocationUpdates(locationRequest, mLocationCallback, null);
        }
        catch(SecurityException e) {
            Log.e("Exception: %s", e.getMessage());
        }
    }

    /**
     * @see MapContract.View#stopLocationUpdates()
     */
    @Override
    public void stopLocationUpdates() {
        mFusedLocationProviderClient.removeLocationUpdates(mLocationCallback);
    }

    /**
     * @see MapContract.View#centerMap(double, double, int)
     */
    @Override
    public void centerMap(double latitude, double longitude, int zoom) {
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(latitude, longitude), zoom));
    }

    /**
     * @see MapContract.View#removeAllMarkers()
     */
    @Override
    public void removeAllMarkers() {
        mMap.clear();
    }

    /**
     * @see MapContract.View#addMarker(String, String, String, double, double, String)
     */
    @Override
    public void addMarker(String name, String books, String rating, double latitude, double longitude, String tag) {
        Marker marker = mMap.addMarker(new MarkerOptions()
                .title(name)
                .snippet(getString(R.string.label_map_marker_snippet, books, rating))
                .position(new LatLng(latitude, longitude)));
        marker.setTag(tag);
    }

    /**
     * @see MapContract.View#showNoUsersFoundMessage()
     */
    @Override
    public void showNoUsersFoundMessage() {
        showToastMessage(R.string.message_no_users_found);
    }

    /**
     * @see MapContract.View#showUsersSearchError()
     */
    @Override
    public void showUsersSearchError() {
        showToastMessage(R.string.error_searching_users);
    }

    /**
     * @see SearchFragment.OnInteractionListener#search()
     */
    @Override
    public void search(String title, String author, String genre, int distance) {
        // Solicita al presentador la búsqueda de usuarios con los criterios especificados.
        mPresenter.search(title, author, genre, distance);
    }

    /**
     * @see MapContract.View#navigateToUserDetail(UiUser)
     */
    @Override
    public void navigateToUserDetail(UiUser user) {
        // Pasamos a la actividad de detalle los datos del usuario seleccionado.
        Intent intent = new Intent(getContext(), UserDetailActivity.class);
        intent.putExtra(UserDetailActivity.EXTRA_USER_DATA, user);
        startActivity(intent);
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogPositiveClick(String, Object...)
     */
    @Override
    public void onDialogPositiveClick(String tag, Object... values) {
        if(tag.equals(TAG_LOCATION_SETTINGS_DIALOG_FRAGMENT)) {
            // El usuario acepta abrir la pantalla de configuración
            // para activar la localización.
            mPresenter.locationSettingsOk();
        }
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        if(tag.equals(TAG_LOCATION_SETTINGS_DIALOG_FRAGMENT)) {
            // El usuario rechaza abrir la pantalla de configuración
            // para activar la localización.
            mPresenter.locationSettingsCanceled();
        }
    }
}
