package edu.uoc.jsev.b4b.ui.map;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;
import edu.uoc.jsev.b4b.util.MapUtils;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador de la pantalla del mapa.
 */
public class MapPresenter extends BasePresenter implements MapContract.Presenter {

    // Radio por defecto para las búsquedas por distancia (en kms).
    private static final int DEFAULT_SEARCH_RADIUS = 45;

    // Vista e interactor asociados.
    private MapContract.View mView;
    private MapInteractor mInteractor;

    // Lista de respaldo para almacenar
    // los usuarios mostrados sobre el mapa.
    private List<UiUser> mUsers;

    // Variables de respaldo para almacenar
    // las coordenadas de la localización actual del usuario.
    private double mCurrentLatitude;
    private double mCurrentLongitude;

    public MapPresenter(MapContract.View view, MapInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;

        // Por defecto serán las de la última localización
        // conocida registrada en el servidor.
        this.mCurrentLatitude = mInteractor.getCurrentUserLatitude();
        this.mCurrentLongitude = mInteractor.getCurrentUserLongitude();
    }

    /**
     * @see MapContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Establecemos el título de la actividad contenedora
        // y mostramos el botón flotante, para realizar búsquedas.
        mView.setTitle();
        mView.showFabButton(true);
    }

    /**
     * @see MapContract.Presenter#setupMap()
     */
    @Override
    public void setupMap() {
        // Tenemos permiso para acceder a la localización.
        if(mView.isLocationPermissionGranted()) {
            // Comprobamos si la localización está activada.
            if(!mView.isLocationServiceEnabled()) {
                // Si no lo está, preguntamos al usuario si quiere activarla.
                mView.showLocationSettingsDialog();
            }
            // Si lo está, obtenemos la localización actual del usuario.
            else {
                getDeviceLocation();
            }
        }
        // Si no tenemos permiso, lo solicitamos al usuario.
        else {
            mView.requestLocationPermission();
        }
    }

    /**
     * @see MapContract.Presenter#locationPermissionGranted()
     */
    @Override
    public void locationPermissionGranted() {
        // Si el usuario nos da permiso para acceder a su localización
        // iniciamos de nuevo el proceso de configuración del mapa.
        setupMap();
    }

    /**
     * @see MapContract.Presenter#locationPermissionDenied()
     */
    @Override
    public void locationPermissionDenied() {
        // Si el usuario nos deniega el permiso de acceso a su localización
        // realizamos la búsqueda de usuarios utilizando su última localización
        // conocida.
        doInitialSearch();
    }

    /**
     * @see MapContract.Presenter#locationPermissionRationaleActionClicked()
     */
    @Override
    public void locationPermissionRationaleActionClicked() {
        // Volvemos a solicitar permiso al usuario para que
        // nos deje acceder a su localización.
        mView.requestLocationPermissionNoRationale();
    }

    /**
     * @see MapContract.Presenter#locationPermissionRationaleDismissed()
     */
    @Override
    public void locationPermissionRationaleDismissed() {
        // El usuario nos vuelve a denegar el permiso de acceso
        // a su localización, por lo que realizamos la búsqueda
        // de usuarios utilizando su última localización conocida.
        doInitialSearch();
    }

    /**
     * @see MapContract.Presenter#locationSettingsOk()
     */
    @Override
    public void locationSettingsOk() {
        // Abrimos la configuración de la localización,
        // para que el usuario la active.
        mView.navigateToLocationSettings();
    }


    /**
     * @see MapContract.Presenter#locationSettingsCanceled()
     */
    @Override
    public void locationSettingsCanceled() {
        // Cerramos el diálogo. Al no estar activada
        // la localización realizamos la búsqueda
        // de usuarios utilizando la última localización
        // conocida del usuario.
        mView.dismissLocationSettingsDialog();
        doInitialSearch();
    }

    /**
     * @see MapContract.Presenter#updateUserLocation(double, double, boolean)
     */
    @Override
    public void updateUserLocation(double latitude, double longitude, boolean validLocation) {
        // Obtenemos la actualización de la ubicación del usuario.
        // Si es fiable la actualizamos, localmente y en el servidor,
        // para tenerla disponible la próxima vez que acceda.
        // Si no es fiable, utilizamos la última conocida.
        if(validLocation) {
            mCurrentLatitude = latitude;
            mCurrentLongitude = longitude;

            mInteractor.updateUserLastKnownLocation(latitude, longitude);
        }

        // Detenemos la recepción de actualizaciones de la localización,
        // para ahorra batería en el dispositivo.
        mView.stopLocationUpdates();

        // Realizamos la búsqueda inicial.
        doInitialSearch();
    }

    /**
     * @see MapContract.Presenter#navigatedFromLocationSettings()
     */
    @Override
    public void navigatedFromLocationSettings() {
        // Al volver de la pantalla de activación de la localización
        // volvemos a evaluar si el usuario la ha activado o no.
        if(mView.isLocationServiceEnabled()) {
            // Si la ha activado obtenemos su localización.
            getDeviceLocation();
        }
        else {
            // Si no la ha activado realizamos la búsqueda
            // inicial utilizando la última localización conocida.
            doInitialSearch();
        }
    }

    /**
     * @see MapContract.Presenter#search(String, String, String, int)
     */
    @Override
    public void search(String title, String author, String genre, final int distance) {
        mView.showProgress(true);

        // Realizamos la búsqueda de usuarios con los criterios especificados.
        mInteractor.findUsers(title, author, genre, distance, new ServiceCallback<List<UiUser>>() {
            @Override
            public void onSuccess(List<UiUser> response) {
                // Si la búsqueda tiene éxito, eliminamos
                // los usuarios que hubiera ubicados sobre el mapa
                // y centramos el mapa en la posición del usuario.
                mUsers = response;
                mView.showProgress(false);
                mView.removeAllMarkers();
                mView.centerMap(mCurrentLatitude, mCurrentLongitude,
                        MapUtils.getZoomLevelFromDistance(distance));

                // Si la búsqueda no devuelve usuarios,
                // mostramos un mensaje.
                if(response.isEmpty()) {
                    mView.showNoUsersFoundMessage();
                    return;
                }

                // Si devuelve usuarios, mostramos su posición sobre el mapa.
                for(UiUser user : response) {
                    mView.addMarker(user.getNombre(), String.valueOf(user.getNumeroLibros()),
                            Utils.formatDoubleIfGreaterThan1(user.getValoracion()), user.getLatitud(),
                            user.getLongitud(), user.getId());
                }
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce un error en la búsqueda
                // mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showUsersSearchError();
            }
        });
    }

    /**
     * @see MapContract.Presenter#infoWindowClicked(String)
     */
    @Override
    public void infoWindowClicked(String tag) {
        // Navegamos a la pantalla de detalle
        // del usuario sobre el que se ha pulsado.
        UiUser selectedUser = new UiUser();
        selectedUser.setId(tag);
        selectedUser = mUsers.get(mUsers.indexOf(selectedUser));

        mView.navigateToUserDetail(selectedUser);
    }

    /***  MÉTODOS AUXILIARES ***/

    /**
     * Realiza la búsqueda inicial. Busca todos los usuarios
     * que comparten algún libro que se encuentren a 45 km
     * o menos de la posición del usuario.
     */
    private void doInitialSearch() {
        search("", "", "", DEFAULT_SEARCH_RADIUS);
    }

    /**
     * Obtiene la ubicación actual del dispositivo.
     */
    private void getDeviceLocation(){
        mView.showProgress(true);
        mView.getDeviceLocation();
    }
}
