package edu.uoc.jsev.b4b.ui.map.search;

import java.util.List;

/**
 * Contrato entre la vista y el presentador
 * del filtro de búsqueda de usuarios.
 */
public interface SearchContract {

    /**
     * Vista del filtro de búsqueda de usuarios.
     */
    interface View {
        /**
         * Elimina las opciones del desplagable de géneros literarios.
         */
        void clearGenres();

        /**
         * Establece la lista de géneros literarios
         * en su correspondiente desplegable.
         * @param genres La lista de géneros.
         */
        void setGenres(List<String> genres);

        /**
         * Establece la distancia máxima de búsqueda que el usuario
         * puede introducir en el filtro.
         * @param maxDistance La distancia máxima.
         */
        void setMaxSearchDistance(int maxDistance);

        /**
         * Actualiza el valor de la barra de búsqueda
         * por distancia del filtro.
         * @param distance El nuevo valor de la barra de distancia.
         */
        void updateDistanceSeekBar(int distance);

        /**
         * Actualiza el texto que indica el valor actual
         * de la barra de búsqueda por distancia del filtro.
         * @param distance El nuevo valor del texto.
         */
        void updateDistanceText(int distance);

        /**
         * Limpia los errores de validación
         * de los campos de texto de la pantalla.
         */
        void resetValidationErrors();

        /**
         * Muestra un mensaje de error indicando que
         * el título introducido en los criterios no
         * tiene la longitud mínima requerida.
         */
        void showInvalidTitleError();

        /**
         * Muestra un mensaje de error indicando que
         * el autor introducido en los criterios no
         * tiene la longitud mínima requerida.
         */
        void showInvalidAuthorError();

        /**
         * Solicita al listener del fragment que realice la búsqueda
         * de usuarios con los criterios pasados como parámetro.
         * @param title El título a buscar.
         * @param author El autor a buscar.
         * @param genre El género literario a buscar.
         * @param distance La distancia máxima (en kms) en la que buscar.
         */
        void delegateSearch(String title, String author, String genre, int distance);

        /**
         * Cierra el panel de búsqueda.
         */
        void dismissDialog();
    }

    /**
     * Presentador del filtro de búsqueda de usuarios.
     */
    interface Presenter {
        /**
         * Inicializa el panel de búsqueda.
         */
        void setup();

        /**
         * El usuario ha modificado el valor de la distancia de búsqueda.
         * @param distance La distancia proporcionada por el usuario.
         */
        void distanceChanged(int distance);

        /**
         * El usuario ha pulsado sobre el botón "Buscar",
         * proporcionando los criterios de búsqueda pasados
         * como parámetros.
         * @param title El título a buscar.
         * @param author El autor a buscar.
         * @param genre El género literario a buscar.
         * @param distance La distancia máxima (en kms) en la que buscar.
         */
        void search(String title, String author, String genre, int distance);
    }
}
