package edu.uoc.jsev.b4b.ui.map.search;

import android.annotation.SuppressLint;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.BottomSheetDialogFragment;
import android.support.design.widget.TextInputLayout;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.TextView;

import java.util.List;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.util.ContextUtils;
import edu.uoc.jsev.b4b.Injection;

/**
 * Fragmento que proporciona un filtro para buscar usuarios que compartan
 * libros que cumplan unos determinados criterios.
 * El fragmento se despliega desde la parte inferior de la pantalla,
 * a través de un BottomSheetDialogFragment.
 */
public class SearchFragment extends BottomSheetDialogFragment implements SearchContract.View {

    // Referencias a las vistas.
    private TextInputLayout mTitleLayout;
    private TextInputLayout mAuthorLayout;
    private AutoCompleteTextView mGenreText;
    private TextView mDistanceLabel;
    private SeekBar mDistanceSeekBar;
    private TextView mDistanceText;

    // Adaptador para mostrar la lista de selección de géneros literarios.
    private ArrayAdapter<String> mGenreAdapter;

    // Presentador.
    private SearchContract.Presenter mPresenter;

    // Callback al que se enviarán los criterios
    // de búsqueda introducidos por el usuario.
    private OnInteractionListener mListener;

    // Método fábrica.
    public static SearchFragment newInstance() {
        return new SearchFragment();
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        try {
            // Inyectamos el presentador y configuramos el fragment contenedor
            // como receptor de los criterios introducidos por el usuario.
            mPresenter = Injection.provideSearchPresenter(this);
            mListener = (OnInteractionListener) getParentFragment();
            setStyle(BottomSheetDialogFragment.STYLE_NORMAL, R.style.AppTheme_BottomSheetDialog);
        }
        catch(ClassCastException e) {
            throw new ClassCastException(getParentFragment().toString() + " debe implementar " + OnInteractionListener.class.getName());
        }
    }

    @Nullable
    @Override
    @SuppressLint("ClickableViewAccessibility")
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.dialog_search, container, false);

        // Configuramos las vistas.
        mTitleLayout = view.findViewById(R.id.title);
        mAuthorLayout = view.findViewById(R.id.author);
        mDistanceLabel = view.findViewById(R.id.distance_label);
        mDistanceText = view.findViewById(R.id.distance_text);

        // Lista de selección de géneros literarios.
        // Simulamos un desplegable sobrescribiendo el evento onTouch.
        // Cuando el usuario pulse el control se mostrará la lista de géneros.
        mGenreText = view.findViewById(R.id.genre);
        mGenreAdapter = new ArrayAdapter<>(getContext(), android.R.layout.simple_spinner_dropdown_item);
        mGenreText.setAdapter(mGenreAdapter);
        mGenreText.setKeyListener(null);
        mGenreText.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if(event.getAction() == MotionEvent.ACTION_UP) {
                    ((AutoCompleteTextView) v).showDropDown();
                    return true;
                }
                return false;
            }
        });

        // Barra de selección de la distancia máxima de búsqueda.
        mDistanceSeekBar = view.findViewById(R.id.distance_seekbar);
        mDistanceSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                // Informamos al presentador de que el usuario
                // ha modificado la distancia de búsqueda.
                if(fromUser) {
                    mPresenter.distanceChanged(progress);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        // Botón principal. Pasa los criterios introducidos al listener,
        // y delega en él la búsqueda.
        Button searchButton = view.findViewById(R.id.search_button);
        searchButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                search();
            }
        });

        // Coloreamos los iconos de los TextView.
        TextView[] textViews = new TextView[] { mTitleLayout.getEditText(), mAuthorLayout.getEditText(),
                mGenreText, mDistanceLabel };
        ContextUtils.tintCompoundDrawables(textViews, R.color.colorPrimary);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();

        return view;
    }

    /**
     * @see SearchContract.View#clearGenres()
     */
    @Override
    public void clearGenres() {
        mGenreAdapter.clear();
    }

    /**
     * @see SearchContract.View#setGenres(List)
     */
    @Override
    public void setGenres(List<String> genres) {
        mGenreAdapter.addAll(genres);
    }

    /**
     * @see SearchContract.View#setMaxSearchDistance(int)
     */
    @Override
    public void setMaxSearchDistance(int distance) {
        mDistanceSeekBar.setMax(distance);
    }

    /**
     * @see SearchContract.View#updateDistanceSeekBar(int)
     */
    @Override
    public void updateDistanceSeekBar(int distance) {
        mDistanceSeekBar.setProgress(distance);
    }

    /**
     * @see SearchContract.View#updateDistanceText(int)
     */
    @Override
    public void updateDistanceText(int distance) {
        mDistanceText.setText(getString(R.string.label_distance_kms, String.valueOf(distance)));
    }

    /**
     * @see SearchContract.View#resetValidationErrors()
     */
    @Override
    public void resetValidationErrors() {
        mTitleLayout.setErrorEnabled(false);
        mAuthorLayout.setErrorEnabled(false);
    }

    /**
     * @see SearchContract.View#showInvalidTitleError()
     */
    @Override
    public void showInvalidTitleError() {
        mTitleLayout.setError(getString(R.string.error_invalid_criterion));
    }

    /**
     * @see SearchContract.View#showInvalidAuthorError()
     */
    @Override
    public void showInvalidAuthorError() {
        mAuthorLayout.setError(getString(R.string.error_invalid_criterion));
    }

    /**
     * @see SearchContract.View#delegateSearch(String, String, String, int)
     */
    @Override
    public void delegateSearch(String title, String author, String genre, int distance) {
        mListener.search(title, author, genre, distance);
    }

    /**
     * @see SearchContract.View#dismissDialog()
     */
    @Override
    public void dismissDialog() {
        dismiss();
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Pasa los datos al presentador para que se realize la búsqueda,
     * después de haber pulsado el usuario sobre el botón correspondiente.
     */
    private void search() {
        String title = mTitleLayout.getEditText().getText().toString();
        String author = mAuthorLayout.getEditText().getText().toString();
        String genre = mGenreText.getText().toString();
        int distance = mDistanceSeekBar.getProgress();

        mPresenter.search(title, author, genre, distance);
    }

    /**
     * Interfaz que debe implementar la actividad o fragmento
     * desde el que se cree el filtro de búsqueda para gestionar
     * los eventos que se generen sobre él.
     */
    public interface OnInteractionListener {
        /**
         * Evento lanzado cuando se pulsa sobre el botón "Buscar".
         * Proporciona los criterios de búsqueda introducidos por
         * el usuario.
         * @param title Título a buscar.
         * @param author Autor a buscar.
         * @param genre Género literario a buscar.
         * @param distance Distancia máxima (en kms) en la que buscar.
         */
        void search(String title, String author, String genre, int distance);
    }
}
