package edu.uoc.jsev.b4b.ui.map.search;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador del filtro de búsqueda de usuarios.
 */
public class SearchPresenter implements SearchContract.Presenter {

    // Distancias mínima y máxima de búsqueda en kilómetros.
    private static final int MIN_SEARCH_DISTANCE = 15;
    private static final int MAX_SEARCH_DISTANCE = MIN_SEARCH_DISTANCE * 65; // 975 km

    // Vista e interactor asociados.
    private SearchContract.View mView;
    private SearchInteractor mInteractor;

    public SearchPresenter(SearchContract.View view, SearchInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see SearchContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Cargamos el desplegable de géneros literarios,
        // establecemos la distancia máxima de búsqueda
        // y la distancia de búsqueda por defecto.
        getGenres();
        mView.setMaxSearchDistance(MAX_SEARCH_DISTANCE);
        distanceChanged(MIN_SEARCH_DISTANCE * 3);
    }

    /**
     * @see SearchContract.Presenter#distanceChanged(int)
     */
    @Override
    public void distanceChanged(int distance) {
        // Normalizamos la distancia introducida
        // por el usuario para que sea múltiplo
        // de la distancia mínima y para que no
        // sea menor que ésta.
        distance = normalizeDistance(distance);
        mView.updateDistanceText(distance);
        mView.updateDistanceSeekBar(distance);
    }

    /**
     * @see SearchContract.Presenter#search(String, String, String, int)
     */
    @Override
    public void search(String title, String author, String genre, int distance) {
        boolean cancel = false;
        mView.resetValidationErrors();

        // Validamos los criterios introducidos antes de hacer la búsqueda:

        // Si se introduce un título debe tener un mínimo de 4 caracteres.
        if (!Utils.isEmptyString(title) && !Utils.hasMinLength(title, 4)) {
            mView.showInvalidTitleError();
            cancel = true;
        }

        // Si se introduce un autor debe tener un mínimo de 4 caracteres.
        if (!Utils.isEmptyString(author) && !Utils.hasMinLength(author, 4)) {
            mView.showInvalidAuthorError();
            cancel = true;
        }

        // Si los criterios son correctos realizamos la búsqueda
        // y cerramos el panel de filtro.
        if(!cancel) {
            mView.delegateSearch(title, author, genre, normalizeDistance(distance));
            mView.dismissDialog();
        }

    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Obtiene la lista de géneros literarios
     * y los carga en el campo desplegable del panel de filtro.
     */
    private void getGenres() {
        final List<String> genres = new ArrayList<>();
        genres.add("");
        mView.clearGenres();
        mView.setGenres(genres);

        mInteractor.getAllGenres(new ServiceCallback<List<String>>() {
            @Override
            public void onSuccess(List<String> response) {
                // Carga la lista de géneros en el desplegable.
                genres.addAll(response);
                mView.clearGenres();
                mView.setGenres(genres);
            }

            @Override
            public void onError(ServiceFault fault) {

            }
        });
    }

    /**
     * Normaliza la distancia de búsqueda introducida por el usuario.
     * Se asegura de que sea múltiplo de la distancia mínima y que
     * no sea inferior a ésta.
     * @param distance La distancia introducida por el usuario.
     * @return La distancia normalizada. Será un múltiplo de la distancia
     * mínima, o la propia distancia mínima, en caso de que el usuario introdujera
     * un valor menor a ésta.
     */
    private int normalizeDistance(int distance) {
        distance = distance / MIN_SEARCH_DISTANCE;
        distance = distance * MIN_SEARCH_DISTANCE;
        return distance >= MIN_SEARCH_DISTANCE ? distance : MIN_SEARCH_DISTANCE;
    }
}
