package edu.uoc.jsev.b4b.ui.map.userdetail;

import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v4.app.NavUtils;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.TextView;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewAdapter;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewItemClickListener;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailActivity;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailActivity;
import edu.uoc.jsev.b4b.util.ContextUtils;
import edu.uoc.jsev.b4b.Injection;

/**
 * Actividad de la pantalla de detalle de usuario.
 * Muestra información básica de otro usuario, como su
 * nombre, número de libros que comparte, distancia a la
 * que se encuentra y valoración media. Además, lista el
 * conjunto de libros compartidos por el usuario, a través
 * de un RecyclerView cuyos datos se pueden refrescar por
 * medio de un SwipeRefreshLayout.
 */
public class UserDetailActivity extends BaseActivity
        implements UserDetailContract.View, RecyclerViewItemClickListener, SwipeRefreshLayout.OnRefreshListener {

    // Nombre de los extras que puede recibir la actividad desde otras pantallas.
    public static final String EXTRA_USER_DATA = "userData";
    public static final String EXTRA_SWAP_ID = BookDetailActivity.EXTRA_SWAP_ID;

    // Referencias a las vistas.
    private TextView mBooksNumberView;
    private TextView mDistanceView;
    private TextView mRatingView;
    private CollapsingToolbarLayout mToolbarLayout;

    // Referencias a las vistas para mostrar el listado de libros del usuario.
    private SwipeRefreshLayout mSwipeRefreshLayout;
    private View mEmptyListView;
    private RecyclerView mRecyclerView;
    private RecyclerViewAdapter mAdapter;

    // Presentador.
    private UserDetailContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_detail);

        // Inyectamos el presentador.
        mPresenter = Injection.provideUserDetailPresenter(this);

        // Configuramos las vistas.
        mBooksNumberView = findViewById(R.id.books_number);
        mDistanceView = findViewById(R.id.distance);
        mRatingView = findViewById(R.id.rating);
        mToolbarLayout = findViewById(R.id.toolbar_layout);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        // Configuramos el RecyclerView del listado.
        mAdapter = new RecyclerViewAdapter(mPresenter, this, RecyclerViewAdapter.BOOK_VIEW_TYPE);
        setupRecyclerView();

        // Coloreamos los iconos de los TextView.
        TextView[] textViews = new TextView[] { mBooksNumberView, mDistanceView, mRatingView };
        ContextUtils.tintCompoundDrawables(textViews, R.color.colorAccent);

        mProgressView = findViewById(R.id.progress);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();

        // Solicitamos al presentador los libros del usuario.
        mPresenter.loadData();
    }

    @Override
    public boolean onSupportNavigateUp() {
        // Navegación al pulsar el botón "Up".
        mPresenter.upButtonPressed();
        return true;
    }

    @Override
    public void onBackPressed() {
        // Navegación al pulsar el botón "Back".
        mPresenter.backButtonPressed();
    }

    @Override
    public void onRefresh() {
        // Al solicitar el refresco por medio del SwipeRefreshLayout,
        // volvemos a pedir los datos al presentador.
        mPresenter.loadData();
    }

    /**
     * @see UserDetailContract.View#showProgress(boolean)
     */
    @Override
    public void showProgress(boolean show) {
        // Sobrescribimos el comportamiento por defecto del método
        // para que cuando se muestre el indicador de progreso
        // tras refrescar con el SwipeRefreshLayout, no aparezca
        // el indicador que muestra el propio componente, ya que
        // en ese caso aparecerían dos a la vez.
        super.showProgress(show);
        mSwipeRefreshLayout.setRefreshing(false);
    }

    /**
     * @see UserDetailContract.View#getUserData()
     */
    @Override
    public UiUser getUserData() {
        return (UiUser) getIntent().getSerializableExtra(EXTRA_USER_DATA);
    }

    /**
     * @see UserDetailContract.View#getParams()
     */
    @Override
    public Object[] getParams() {
        return new String[] { getIntent().getStringExtra(EXTRA_SWAP_ID) };
    }

    /**
     * @see UserDetailContract.View#setName(String)
     */
    @Override
    public void setName(String name) {
        mToolbarLayout.setTitle(name);
    }

    /**
     * @see UserDetailContract.View#setBooksNumber(String)
     */
    @Override
    public void setBooksNumber(String booksNumber) {
        mBooksNumberView.setText(booksNumber);
    }

    /**
     * @see UserDetailContract.View#setDistance(String)
     */
    @Override
    public void setDistance(String distance) {
        mDistanceView.setText(getString(R.string.label_distance_kms, distance));
    }

    /**
     * @see UserDetailContract.View#setRating(String)
     */
    @Override
    public void setRating(String rating) {
        mRatingView.setText(rating);
    }

    /**
     * @see UserDetailContract.View#navigateToMap()
     */
    @Override
    public void navigateToMap() {
        // Si se ha llegado a esta pantalla desde la búsqueda de usuarios en el mapa,
        // la navegación atrás regresaría al mapa.
        Intent intent = NavUtils.getParentActivityIntent(this);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);
        NavUtils.navigateUpTo(this, intent);
    }

    /**
     * @see UserDetailContract.View#navigateToSwapDetail()
     */
    @Override
    public void navigateToSwapDetail() {
        // Si se ha llegado a esta pantalla desde el detalle de un intercambio,
        // para consultar los libros del usuario y solicitarle uno,
        // la navegación atrás regresaría a la pantalla de detalle de intercambio.
        Intent intent = new Intent(this, SwapDetailActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);
        NavUtils.navigateUpTo(this, intent);
    }

    /**
     * @see RecyclerViewItemClickListener#onItemClick(UiEntity)
     */
    @Override
    public void onItemClick(UiEntity item) {
        mPresenter.dataClicked(item);
    }

    /**
     * @see RecyclerViewContract.View#refreshData()
     */
    @Override
    public void refreshData() {
        mAdapter.notifyDataSetChanged();
        showEmptyList(mAdapter.isEmpty());
    }

    /**
     * @see UserDetailContract.View#navigateToDetail(UiEntity, Object...)
     */
    @Override
    public void navigateToDetail(UiEntity item, Object... params) {
        // Pasamos a la actividad de detalle de libro los datos del elemento pulsado.
        // Si se llega a esta actividad desde el detalle de un intercambio,
        // para consultar los libros del usuario y solicitarle uno,
        // pasamos también el Id del intercambio, para actualizar sus datos
        // en caso de que finalmente se le acabe pidiendo el libro al usuario.
        String swapId = params.length > 0 ? (String) params[0] : "";

        Intent intent = new Intent(this, BookDetailActivity.class);
        intent.putExtra(BookDetailActivity.EXTRA_BOOK_DATA, item);
        intent.putExtra(BookDetailActivity.EXTRA_SWAP_ID, swapId);

        startActivity(intent);
    }

    /**
     * @see RecyclerViewContract.View#showLoadDataError()
     */
    @Override
    public void showLoadDataError() {
        showToastMessage(R.string.error_getting_books);
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Configura el RecyclerView del listado y su adaptador.
     */
    private void setupRecyclerView() {
        mSwipeRefreshLayout = findViewById(R.id.refresh_layout);
        mSwipeRefreshLayout.setColorSchemeResources(R.color.colorPrimary, R.color.colorPrimaryLight, R.color.colorPrimaryDark);
        mSwipeRefreshLayout.setOnRefreshListener(this);

        mRecyclerView = findViewById(R.id.list);
        mRecyclerView.setLayoutManager(new LinearLayoutManager(this));
        mRecyclerView.setAdapter(mAdapter);

        mEmptyListView = findViewById(R.id.empty_list);
    }

    /**
     * Muestra/oculta un mensaje de lista vacía,
     * en caso de que el usuario no disponga de ningún libro.
     * @param show Si es true, se muestra el mensaje de lista vacía,
     *             si es false, se muestran los elementos del listado.
     */
    private void showEmptyList(boolean show) {
        mEmptyListView.setVisibility(show ? View.VISIBLE : View.GONE);
        mRecyclerView.setVisibility(show ? View.GONE : View.VISIBLE);
    }
}
