package edu.uoc.jsev.b4b.ui.mybooks;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewAdapter;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewItemClickListener;
import edu.uoc.jsev.b4b.ui.main.MainFragment;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailActivity;
import edu.uoc.jsev.b4b.ui.mybooks.newbook.NewBookActivity;
import edu.uoc.jsev.b4b.Injection;

/**
 * Fragmento de la pantalla de listado de libros.
 * El listado se implementa a través de un RecyclerView
 * cuyos datos se pueden refrescar por medio de un
 * SwipeRefreshLayout.
 */
public class MyBooksFragment extends MainFragment
        implements MyBooksContract.View, RecyclerViewItemClickListener {

    // Presentador.
    private MyBooksContract.Presenter mPresenter;

    public MyBooksFragment() {
    }

    // Método fábrica.
    public static MyBooksFragment newInstance() {
        return new MyBooksFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Inyectamos el presentador.
        mPresenter = Injection.provideMyBooksPresenter(this);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_mybooks, container, false);

        // Configuramos el recyclerview.
        mAdapter = new RecyclerViewAdapter(mPresenter, this, RecyclerViewAdapter.BOOK_VIEW_TYPE);
        setupRecyclerView(view);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();

        // Solicitamos al presentador los libros del usuario.
        mPresenter.loadData();

        return view;
    }

    @Override
    public void onHiddenChanged(boolean hidden) {
        // Volvemos a inicializar la pantalla (título y acción del FAB)
        // cada vez que el fragmento vuelve a estar visible.
        // Como solo se carga la primera vez, solo entra una vez por
        // onCreateView.
        super.onHiddenChanged(hidden);
        if(!hidden) mPresenter.setup();
    }

    @Override
    public void onRefresh() {
        // Al solicitar el refresco por medio del SwipeRefreshLayout,
        // volvemos a pedir los datos al presentador.
        super.onRefresh();
        mPresenter.loadData();
    }

    /**
     * Añade al listado un nuevo libro.
     * @see MainFragment#addObjectToList(UiEntity)
     */
    @Override
    public void addObjectToList(UiEntity dataObject) {
        super.addObjectToList(dataObject);
        mPresenter.addData((UiBook) dataObject);
    }

    /**
     * Elimina un libro del listado.
     * @see MainFragment#removeObjectFromList(UiEntity)
     */
    @Override
    public void removeObjectFromList(UiEntity dataObject) {
        super.removeObjectFromList(dataObject);
        mPresenter.removeData((UiBook) dataObject);
    }

    /**
     * @see MyBooksContract.View#setTitle()
     */
    @Override
    public void setTitle() {
        // Delega en la actividad contenedora el cambio.
        mCallback.setTitle(getString(R.string.title_fragment_mybooks));
    }

    /**
     * @see MyBooksContract.View#showFabButton(boolean)
     */
    @Override
    public void showFabButton(boolean show) {
        // Delega en la actividad contenedora la configuración del botón.
        mCallback.setFabButtonIcon(R.drawable.ic_plus_white_24dp);
        mCallback.showFabButton(show);
    }

    /**
     * @see RecyclerViewContract.View#refreshData()
     */
    @Override
    public void refreshData() {
        // Notificamos al adaptador los cambios en los datos
        // y evaluamos si es necesario o no mostrar la vista
        // del listado vacío.
        mAdapter.notifyDataSetChanged();
        showEmptyList(mAdapter.isEmpty());
    }

    /**
     * @see RecyclerViewContract.View#showLoadDataError()
     */
    @Override
    public void showLoadDataError() {
        showToastMessage(R.string.error_getting_books);
    }

    /**
     * @see RecyclerViewItemClickListener#onItemClick(UiEntity)
     */
    @Override
    public void onItemClick(UiEntity item) {
        mPresenter.dataClicked(item);
    }

    /**
     * @see MainFragment#fabButtonClicked()
     */
    @Override
    public void fabButtonClicked() {
        mPresenter.mainActionClicked();
    }

    /**
     * @see MyBooksContract.View#navigateToDetail(UiEntity, Object...)
     */
    @Override
    public void navigateToDetail(UiEntity item, Object... params) {
        // Pasamos a la actividad de detalle los datos del libro pulsado.
        Intent intent = new Intent(getContext(), BookDetailActivity.class);
        intent.putExtra(BookDetailActivity.EXTRA_BOOK_DATA, item);
        startActivity(intent);
    }

    /**
     * @see MyBooksContract.View#navigateToNewBook()
     */
    @Override
    public void navigateToNewBook() {
        startActivity(new Intent(getContext(), NewBookActivity.class));
    }
}
