package edu.uoc.jsev.b4b.ui.mybooks.bookdetail;

import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v4.app.NavUtils;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailActivity;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailActivity;
import edu.uoc.jsev.b4b.util.ContextUtils;
import edu.uoc.jsev.b4b.Injection;

/**
 * Actividad de la pantalla de detalle de un libro.
 * Desde esta actividad, aparte de visualizar sus datos,
 * se puede eliminar un libro, si es propiedad del usuario,
 * o solicitarlo, si es propiedad de otro usuario.
 */
public class BookDetailActivity extends BaseActivity implements BookDetailContract.View {

    // Nombre de los extras que puede recibir la actividad desde otras pantallas.
    public static final String EXTRA_BOOK_DATA = "bookData";
    public static final String EXTRA_SWAP_ID = "swapId";

    // Tags para identificar los diálogos.
    private static final String TAG_DELETION_DIALOG_FRAGMENT = "deletionDialog";
    private static final String TAG_SWAP_DIALOG_FRAGMENT = "swapDialog";

    // Referencias a las vistas.
    private ImageView mPictureView;
    private TextView mAuthorView;
    private TextView mPublisherView;
    private TextView mGenreView;
    private TextView mLanguageView;
    private TextView mIsbnView;
    private TextView mStateLabelView;
    private ImageView mStateView;
    private CollapsingToolbarLayout mToolbarLayout;

    // Presentador.
    private BookDetailContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_book_detail);

        // Inyectamos el presentador.
        mPresenter = Injection.provideBookDetailPresenter(this);

        // Configuramos las vistas.
        mPictureView = findViewById(R.id.picture);
        mAuthorView = findViewById(R.id.author);
        mPublisherView = findViewById(R.id.publisher);
        mGenreView = findViewById(R.id.genre);
        mLanguageView = findViewById(R.id.language);
        mIsbnView = findViewById(R.id.isbn);
        mToolbarLayout = findViewById(R.id.toolbar_layout);
        mStateLabelView = findViewById(R.id.state_label);
        mStateView = findViewById(R.id.state);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        mProgressView = findViewById(R.id.progress);
        mFab = findViewById(R.id.fab);
        mFab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.mainActionClicked();
            }
        });

        // Coloreamos los iconos de los TextView.
        TextView[] textViews = new TextView[] { mAuthorView, mPublisherView, mGenreView,
                mLanguageView, mIsbnView, mStateLabelView };
        ContextUtils.tintCompoundDrawables(textViews, R.color.colorPrimary);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();
    }

    @Override
    public boolean onSupportNavigateUp() {
        // Navegación al pulsar el botón "Up".
        mPresenter.upButtonPressed();
        return true;
    }

    @Override
    public void onBackPressed() {
        // Navegación al pulsar el botón "Back".
        mPresenter.backButtonPressed();
    }

    /**
     * @see BookDetailContract.View#getBookData()
     */
    @Override
    public UiBook getBookData() {
        return (UiBook) getIntent().getSerializableExtra(EXTRA_BOOK_DATA);
    }

    /**
     * @see BookDetailContract.View#getExtraSwapId()
     */
    @Override
    public String getExtraSwapId() {
        return getIntent().getStringExtra(EXTRA_SWAP_ID);
    }

    /**
     * @see BookDetailContract.View#setupDeleteFab()
     */
    @Override
    public void setupDeleteFab() {
        mFab.setImageResource(R.drawable.ic_delete_white_24dp);
    }

    /**
     * @see BookDetailContract.View#setupSwapFab()
     */
    @Override
    public void setupSwapFab() {
        mFab.setImageResource(R.drawable.ic_swap_horizontal_white_24dp);
    }

    /**
     * @see BookDetailContract.View#setPicture(String)
     */
    @Override
    public void setPicture(String url) {
        Picasso.get()
                .load(url)
                .placeholder(R.drawable.book_placeholder)
                .into(mPictureView);
    }

    /**
     * @see BookDetailContract.View#setTitle(String)
     */
    @Override
    public void setTitle(String title) {
        mToolbarLayout.setTitle(title);
    }

    /**
     * @see BookDetailContract.View#setAuthor(String)
     */
    @Override
    public void setAuthor(String author) {
        mAuthorView.setText(author);
    }

    /**
     * @see BookDetailContract.View#setPublisher(String)
     */
    @Override
    public void setPublisher(String publisher) {
        mPublisherView.setText(publisher);
    }

    /**
     * @see BookDetailContract.View#setGenre(String)
     */
    @Override
    public void setGenre(String genre) {
        mGenreView.setText(genre);
    }

    /**
     * @see BookDetailContract.View#setLanguage(String)
     */
    @Override
    public void setLanguage(String language) {
        mLanguageView.setText(language);
    }

    /**
     * @see BookDetailContract.View#setIsbn(String)
     */
    @Override
    public void setIsbn(String isbn) {
        mIsbnView.setText(isbn);
    }

    /**
     * @see BookDetailContract.View#setStateGood()
     */
    @Override
    public void setStateGood() {
        mStateView.setImageResource(R.drawable.ic_emoticon_happy_white_48dp);
        mStateView.setContentDescription(getString(R.string.label_book_state_good));
    }

    /**
     * @see BookDetailContract.View#setStateRegular()
     */
    @Override
    public void setStateRegular() {
        mStateView.setImageResource(R.drawable.ic_emoticon_neutral_white_48dp);
        mStateView.setContentDescription(getString(R.string.label_book_state_regular));
    }

    /**
     * @see BookDetailContract.View#setStateBad()
     */
    @Override
    public void setStateBad() {
        mStateView.setImageResource(R.drawable.ic_emoticon_sad_white_48dp);
        mStateView.setContentDescription(getString(R.string.label_book_state_bad));
    }

    /**
     * @see BookDetailContract.View#showDeletionDialog()
     */
    @Override
    public void showDeletionDialog() {
        showAlertDialog(R.string.action_delete_book, R.string.prompt_deletion_confirmation, TAG_DELETION_DIALOG_FRAGMENT);
    }

    /**
     * @see BookDetailContract.View#showSwapDialog()
     */
    @Override
    public void showSwapDialog() {
        showAlertDialog(R.string.action_request_swap, R.string.prompt_swap_confirmation, TAG_SWAP_DIALOG_FRAGMENT);
    }

    /**
     * @see BookDetailContract.View#dismissDeletionDialog()
     */
    @Override
    public void dismissDeletionDialog() {
        dismissDialog(TAG_DELETION_DIALOG_FRAGMENT);
    }

    /**
     * @see BookDetailContract.View#dismissSwapDialog()
     */
    @Override
    public void dismissSwapDialog() {
        dismissDialog(TAG_SWAP_DIALOG_FRAGMENT);
    }

    /**
     * @see BookDetailContract.View#showBookDeletionOk()
     */
    @Override
    public void showBookDeletionOk() {
        showToastMessage(R.string.message_book_deletion_ok);
    }

    /**
     * @see BookDetailContract.View#showBookDeletionNotPossible()
     */
    @Override
    public void showBookDeletionNotPossible() {
        showToastMessage(R.string.message_book_deletion_not_possible);
    }

    /**
     * @see BookDetailContract.View#showBookDeletionError()
     */
    @Override
    public void showBookDeletionError() {
        showToastMessage(R.string.error_deleting_book);
    }

    /**
     * @see BookDetailContract.View#showSwapRequestOk()
     */
    @Override
    public void showSwapRequestOk() {
        showToastMessage(R.string.message_swap_request_ok);
    }

    /**
     * @see BookDetailContract.View#showSwapRequestNotPossible()
     */
    @Override
    public void showSwapRequestNotPossible() {
        showToastMessage(R.string.message_swap_request_not_possible);
    }

    /**
     * @see BookDetailContract.View#showSwapRequestError()
     */
    @Override
    public void showSwapRequestError() {
        showToastMessage(R.string.error_requesting_swap);
    }

    /**
     * @see BookDetailContract.View#navigateToMyBooks(UiBook)
     */
    @Override
    public void navigateToMyBooks(UiBook book) {
        // Si estamos consultando los datos de un libro propio
        // podemos navegar hacia atrás a la lista de libros.
        Intent intent = NavUtils.getParentActivityIntent(this);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);

        // Informamos al listado de libros de si se debe actualizar.
        // Si se envía un libro significa que ha sido eliminado. Se
        // lo enviamos a la actividad listado para que actualice sus datos
        // y lo elimine también del listado.
        if(book != null) {
            intent.setAction(Intent.ACTION_DELETE);
            intent.putExtra(MainActivity.EXTRA_DATA_OBJECT, book);
        }

        NavUtils.navigateUpTo(this, intent);
    }

    /**
     * @see BookDetailContract.View#navigateToUserDetail()
     */
    @Override
    public void navigateToUserDetail() {
        // Si estamos consultando los datos de un libro de otro usuario
        // podemos navegar hacia atrás a la pantalla de detalle de dicho usuario.
        Intent intent = new Intent(this, UserDetailActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);
        NavUtils.navigateUpTo(this, intent);
    }

    /**
     * @see BookDetailContract.View#navigateToSwaps(UiSwap)
     */
    @Override
    public void navigateToSwaps(UiSwap swap) {
        // Si estamos consultando los datos de un libro de otro usuario
        // y se lo solicitamos, al terminar el proceso navegamos a la pantalla
        // de listado de intercambios.
        Intent intent = new Intent(this, MainActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);

        // Informamos a la pantalla principal de que debe mostrar el listado de intercambios.
        // Enviamos el intercambio a la actividad, para que actualice sus datos
        // y los muestre con la información actualizada, incluyendo el nuevo intercambio.
        if(swap != null) {
            intent.setAction(Intent.ACTION_PICK_ACTIVITY);
            intent.putExtra(MainActivity.EXTRA_DATA_OBJECT, swap);
        }

        startActivity(intent);
    }

    /**
     * @see BookDetailContract.View#navigateToSwapDetail(String, String)
     */
    @Override
    public void navigateToSwapDetail(String bookTitle, String bookPicture) {
        // Si estamos consultando los datos de un libro de otro usuario
        // para aceptar un intercambio que nos ha solicitado, al terminar
        // el proceso navegamos a la pantalla de detalle de intercambio.
        // Enviamos a la pantalla los datos del libro que hemos seleccionado,
        // para que se muestre la información actualizada en la pantalla
        // de detalle del intercambio.
        Intent intent = new Intent(this, SwapDetailActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);
        intent.setAction(Intent.ACTION_EDIT);
        intent.putExtra(SwapDetailActivity.EXTRA_REQUESTED_BOOK_TITLE, bookTitle);
        intent.putExtra(SwapDetailActivity.EXTRA_REQUESTED_BOOK_PICTURE, bookPicture);

        startActivity(intent);
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogPositiveClick(String, Object...)
     */
    @Override
    public void onDialogPositiveClick(String tag, Object... values) {
        // Se ha pulsado sobre "Aceptar" en el diálogo
        // de confirmación de borrado del libro.
        if(tag.equals(TAG_DELETION_DIALOG_FRAGMENT)) {
            mPresenter.delete();
        }
        // Se ha pulsado sobre "Aceptar" en el diálogo
        // de confirmación de solicitud de intercambio del libro.
        else if(tag.equals(TAG_SWAP_DIALOG_FRAGMENT)) {
            mPresenter.requestSwap();
        }
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        // Se ha pulsado sobre "Cancelar" en el diálogo
        // de confirmación de borrado del libro.
        if (tag.equals(TAG_DELETION_DIALOG_FRAGMENT)) {
            mPresenter.deleteCanceled();
        }
        // Se ha pulsado sobre "Cancelar" en el diálogo
        // de confirmación de solicitud de intercambio del libro.
        else if(tag.equals(TAG_SWAP_DIALOG_FRAGMENT)) {
            mPresenter.requestSwapCanceled();
        }
    }
}
