package edu.uoc.jsev.b4b.ui.mybooks.bookdetail;

import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla de detalle de libro.
 */
public interface BookDetailContract {

    /**
     * Vista de la pantalla de detalle de libro.
     */
    interface View extends BaseContract.View {
        /**
         * Devuelve los datos del libro a mostar en pantalla.
         * @return El libro a mostrar.
         */
        UiBook getBookData();

        /**
         * En caso de acceder a la pantalla para aceptar
         * un intercambio, devuelve el id de dicho intercambio.
         * @return El id del intercambio a aceptar.
         */
        String getExtraSwapId();

        /**
         * Configura el botón flotante para que
         * elimine el libro al ser pulsado.
         * Esto ocurrirá cuando se acceda a la pantalla
         * para visualizar un libro del propio usuario.
         */
        void setupDeleteFab();

        /**
         * Configura el botón flotante para que
         * solicite el libro al ser pulsado.
         * Esto ocurrirá cuando se acceda a la pantalla
         * para visualizar un libro de otro usuario.
         */
        void setupSwapFab();

        /**
         * Establece en la pantalla la fotografía del libro.
         * @param url URL de descarga de la fotografía.
         */
        void setPicture(String url);

        /**
         * Establece en la pantalla el título del libro.
         * @param title El título del libro.
         */
        void setTitle(String title);

        /**
         * Establece en la pantalla el autor del libro.
         * @param author El autor del libro.
         */
        void setAuthor(String author);

        /**
         * Establece en la pantalla la editorial del libro.
         * @param publisher La editorial del libro.
         */
        void setPublisher(String publisher);

        /**
         * Establece en la pantalla el género literario del libro.
         * @param genre El género literario del libro.
         */
        void setGenre(String genre);

        /**
         * Establece en la pantalla el idioma del libro.
         * @param language El idioma del libro.
         */
        void setLanguage(String language);

        /**
         * Establece en la pantalla el código ISBN del libro.
         * @param isbn El código ISBN del libro.
         */
        void setIsbn(String isbn);

        /**
         * Establece en la pantalla el estado
         * del libro como bueno.
         */
        void setStateGood();

        /**
         * Establece en la pantalla el estado
         * del libro como regular.
         */
        void setStateRegular();

        /**
         * Establece en la pantalla el estado
         * del libro como malo.
         */
        void setStateBad();

        /**
         * Muestra un diálogo de confirmación
         * para eliminar el libro.
         */
        void showDeletionDialog();

        /**
         * Muestra un diálogo de confirmación
         * para solicitar el intercambio del libro.
         */
        void showSwapDialog();

        /**
         * Descarta el diálogo de eliminación del libro.
         */
        void dismissDeletionDialog();

        /**
         * Descarta el diálogo de solicitud de intercambio del libro.
         */
        void dismissSwapDialog();

        /**
         * Muestra un mensaje indicando que el libro
         * se ha eliminado correctamente.
         */
        void showBookDeletionOk();

        /**
         * Muestra un mensaje indicando que el libro
         * no se puede eliminar por estar usándose
         * en algún intercambio todavía pendiente.
         */
        void showBookDeletionNotPossible();

        /**
         * Muestra un mensaje indicando que se
         * ha producido un error al eliminar el libro.
         */
        void showBookDeletionError();

        /**
         * Muestra un mensaje indicando que el libro
         * se ha solicitado correctamente.
         */
        void showSwapRequestOk();

        /**
         * Muestra un mensaje indicando que el libro
         * no puede solicitarse por haber sido ya solicitado
         * en algún intercambio todavía pendiente.
         */
        void showSwapRequestNotPossible();

        /**
         * Muestra un mensaje indicando que se
         * ha producido un error al solicitar el libro.
         */
        void showSwapRequestError();

        /**
         * Navega a la pantalla del listado de libros
         * (cuando se navega hacia atrás estando consultando
         * un libro propio).
         * @param book Datos del libro eliminado, para
         *             eliminarlo también de la actividad listado.
         *             Si no se ha eliminado el libro,
         *             el parámetro es null y la actividad listado
         *             no realiza ninguna acción.
         */
        void navigateToMyBooks(UiBook book);

        /**
         * Navega a la pantalla de detalle de usuario
         * (cuando se navega hacia atrás estando consultando
         * un libro de otro usuario).
         */
        void navigateToUserDetail();

        /**
         * Navega a la pantalla de listado de intercambios
         * (cuando se ha iniciado un nuevo intercambio).
         * @param swap Datos del intercambio iniciado, para
         *             incluirlo también de la actividad listado.
         */
        void navigateToSwaps(UiSwap swap);

        /**
         * Navega a la pantalla de detalle de intercambio
         * (cuando se ha aceptado un intercambio solicitado
         * por otro usuario).
         * @param bookTitle Título del libro seleccionado, para actualizar
         *                  la información de la pantalla de detalle de intercambio.
         * @param bookPicture URL de la imagen del libro seleccionado, para actualizar
         *                  la información de la pantalla de detalle de intercambio.
         */
        void navigateToSwapDetail(String bookTitle, String bookPicture);
    }

    /**
     * Presentador de la pantalla de detalle de libro.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * El usuario ha pulsado sobre el fotón flotante.
         */
        void mainActionClicked();

        /**
         * El usuario ha pulsado sobre el botón "Aceptar"
         * en el diálogo de confirmación de eliminación del libro.
         */
        void delete();

        /**
         * El usuario ha pulsado sobre el botón "Cancelar"
         * en el diálogo de confirmación de eliminación del libro.
         */
        void deleteCanceled();

        /**
         * El usuario ha pulsado sobre el botón "Aceptar"
         * en el diálogo de confirmación de solicitud del libro.
         */
        void requestSwap();

        /**
         * El usuario ha pulsado sobre el botón "Cancelar"
         * en el diálogo de confirmación de solicitud del libro.
         */
        void requestSwapCanceled();
    }
}
