package edu.uoc.jsev.b4b.ui.mybooks.bookdetail;

import android.util.Log;

import com.backendless.BackendlessUser;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Intercambio;
import edu.uoc.jsev.b4b.data.backendless.model.Libro;
import edu.uoc.jsev.b4b.data.backendless.model.Usuario;
import edu.uoc.jsev.b4b.data.backendless.service.BookService;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.SwapService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.messaging.MailTemplate;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BaseBooksInteractorImpl;

/**
 * Clase de implementación del interactor de la pantalla de detalle de libro.
 */
public class BookDetailInteractorImpl extends BaseBooksInteractorImpl implements BookDetailInteractor {

    // Instancia singleton.
    private static BookDetailInteractor instance;

    // Servicio para acceder al repositorio de intercambios.
    private SwapService mSwapService;

    private BookDetailInteractorImpl(UserService userService,
                                     CommonsService commonsService,
                                     BookService bookService,
                                     SwapService swapService) {
        super(userService, commonsService, bookService);
        this.mSwapService = swapService;
    }

    // Devuelve el singleton.
    public static BookDetailInteractor getInstance(UserService userService,
                                                   CommonsService commonsService,
                                                   BookService bookService,
                                                   SwapService swapService) {
        if (instance == null) {
            instance = new BookDetailInteractorImpl(userService, commonsService, bookService, swapService);
        }
        return instance;
    }

    /**
     * @see BookDetailInteractor#deleteBook(UiBook, ServiceCallback)
     */
    @Override
    public void deleteBook(final UiBook uiBook, final ServiceCallback<Boolean> callback) {
        // Antes de eliminar el libro comprobamos que no
        // se esté usando en algún intercambio pendiente.
        mSwapService.isBookInPendingSwaps(uiBook.getId(), null, new ServiceCallback<Boolean>() {
            @Override
            public void onSuccess(Boolean isBookInPendingSwaps) {
                if(isBookInPendingSwaps) {
                    // Si se está usando, no dejamos borrarlo.
                    callback.onSuccess(false);
                    return;
                }

                Libro book = bookMapper.from(uiBook);
                book.setEliminado(true);

                // Para eliminar el libro actualizamos su campo "eliminado".
                // Se trata de un borrado lógico, para mantener la integridad
                // referencial con otras tablas.
                mBookService.saveBook(book, new ServiceCallback<Libro>() {
                    @Override
                    public void onSuccess(Libro bookResponse) {
                        final BackendlessUser currentUser = mUserService.getCurrentUser();
                        int booksNumber = (Integer) currentUser.getProperty(Usuario.BOOKS_NUMBER_KEY) - 1;
                        currentUser.setProperty(Usuario.BOOKS_NUMBER_KEY, booksNumber);

                        // Si el borrado es correcto actualizamos el contador
                        // de libros del usuario, decrementándolo en uno.
                        mUserService.updateUser(currentUser, new ServiceCallback<BackendlessUser>() {
                            @Override
                            public void onSuccess(BackendlessUser response) {
                                mUserService.setCurrentUser(currentUser);
                                callback.onSuccess(true);
                            }

                            @Override
                            public void onError(ServiceFault fault) {
                                callback.onError(fault);
                            }
                        });
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        callback.onError(fault);
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /**
     * @see BookDetailInteractor#requestNewSwap(String, String, ServiceCallback)
     */
    @Override
    public void requestNewSwap(final String userId, final String bookId, final ServiceCallback<UiSwap> callback) {
        // Antes de solicitarlo comprobamos que no se haya solicitado ya
        // y esté aún pendiente de aceptar por parte del otro usuario.
        mSwapService.isBookInPendingSwaps(bookId, getCurrentUserId(), new ServiceCallback<Boolean>() {
            @Override
            public void onSuccess(Boolean isBookInPendingSwaps) {
                if(isBookInPendingSwaps) {
                    // Si está solicitado y pendiente, no lo volvemos a pedir.
                    callback.onSuccess(null);
                    return;
                }
                // Creamos el intercambio.
                doRequestNewSwap(userId, bookId, callback);
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    @Override
    public void acceptSwap(final String swapId, final String bookId, final ServiceCallback<UiSwap> callback) {
        // Antes de solicitarlo comprobamos que no se haya solicitado ya
        // y esté aún pendiente de aceptar por parte del otro usuario.
        mSwapService.isBookInPendingSwaps(bookId, getCurrentUserId(), new ServiceCallback<Boolean>() {
            @Override
            public void onSuccess(Boolean isBookInPendingSwaps) {
                if(isBookInPendingSwaps) {
                    // Si está solicitado y pendiente, no lo volvemos a pedir.
                    callback.onSuccess(null);
                    return;
                }
                // Aceptamos el intercambio solicitando el libro.
                doAcceptSwap(swapId, bookId, callback);
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Solicitud de nuevo intercambio.
     */
    private void doRequestNewSwap(String userId, final String bookId, final ServiceCallback<UiSwap> callback) {
        mUserService.findUserById(userId, new ServiceCallback<BackendlessUser>() {
            @Override
            public void onSuccess(final BackendlessUser destinationUser) {
                final Intercambio swap = new Intercambio();
                swap.setUsuarioOrigen(getCurrentUserFullName());
                swap.setUsuarioDestino(destinationUser.getProperty(Usuario.NAME_KEY).toString());

                // Insertamos el nuevo intercambio.
                mSwapService.addSwap(swap, bookId, new ServiceCallback<Intercambio>() {
                    @Override
                    public void onSuccess(Intercambio swapResponse) {
                        callback.onSuccess(swapMapper.from(swapResponse));

                        // Enviamos un correo al usuario destinatario,
                        // para informarle de que se le ha solicitado un libro.
                        sendEmail(MailTemplate.SWAP_REQUESTED.getSubject(),
                                MailTemplate.SWAP_REQUESTED.getMessage(getCurrentUserFullName()),
                                destinationUser.getEmail());
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        callback.onError(fault);
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /**
     * Aceptación de un intercambio recibido.
     */
    private void doAcceptSwap(String swapId, final String bookId, final ServiceCallback<UiSwap> callback) {
        Intercambio swap = new Intercambio();
        swap.setObjectId(swapId);
        swap.setEstado(mSwapService.getStateAccepted());

        // Actualizamos el intercambio para marcar su estado como aceptado.
        mSwapService.updateSwap(swap, new ServiceCallback<Intercambio>() {
            @Override
            public void onSuccess(Intercambio response) {
                // Establecemos la relación del intercambio
                // con el libro solicitado al usuario origen
                mSwapService.setRelatedBook(response, bookId, new ServiceCallback<Intercambio>() {
                    @Override
                    public void onSuccess(Intercambio swapResponse) {
                        callback.onSuccess(swapMapper.from(swapResponse));

                        // Se envía un correo electrónico a ambos usuarios
                        // para informarles de que el intercambio ha sido
                        // aceptado y para facilitarles sus respectivas
                        // direcciones de email, para que se puedan poner
                        // en contacto entre ellos.
                        mUserService.findUserById(swapResponse.getOwnerId(), new ServiceCallback<BackendlessUser>() {
                            @Override
                            public void onSuccess(BackendlessUser userResponse) {
                                sendEmail(MailTemplate.SWAP_ACCEPTED_SRC.getSubject(),
                                        MailTemplate.SWAP_ACCEPTED_SRC.getMessage(getCurrentUserFullName(), getCurrentUserEmail()),
                                        userResponse.getEmail());

                                sendEmail(MailTemplate.SWAP_ACCEPTED_DST.getSubject(),
                                        MailTemplate.SWAP_ACCEPTED_DST.getMessage(userResponse.getProperty(Usuario.NAME_KEY).toString(),
                                                userResponse.getEmail()),
                                        getCurrentUserEmail());
                            }

                            @Override
                            public void onError(ServiceFault fault) {
                                Log.w("--->", "No se pudo enviar el mensaje: " + fault.getMessage());
                            }
                        });
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        callback.onError(fault);
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }
}
