package edu.uoc.jsev.b4b.ui.mybooks.bookdetail;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador de la pantalla de detalle de libro.
 */
public class BookDetailPresenter extends BasePresenter implements BookDetailContract.Presenter {

    // Datos del libro mostrado en la pantalla.
    private UiBook mBookData;

    // Flag que indica si el libro visualizado
    // es propiedad del usuario o es de otro usuario.
    private boolean mOwnBook;

    // En caso de venir desde la pantalla de detalle de intercambio,
    // almacena el id del intercambio que se aceptará en caso de
    // solicitarse el libro.
    private String mSwapId;

    // Vista e interactor asociados.
    private BookDetailContract.View mView;
    private BookDetailInteractor mInteractor;

    public BookDetailPresenter(BookDetailContract.View view, BookDetailInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see BookDetailContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Obtenemos los datos del libro,
        // solicitándolos a la vista.
        if(mBookData == null) {
            mBookData = mView.getBookData();
            mOwnBook = mInteractor.getCurrentUserId().equals(mBookData.getOwner());
            mSwapId = mView.getExtraSwapId();
        }

        // Mostramos los datos por pantalla.
        mView.setTitle(mBookData.getTitulo());
        mView.setPicture(mBookData.getFoto());
        mView.setAuthor(mBookData.getAutor());
        mView.setPublisher(mBookData.getEditorial());
        mView.setGenre(mBookData.getGenero());
        mView.setLanguage(mBookData.getIdioma());
        mView.setIsbn(mBookData.getIsbn());

        int state = mBookData.getEstado();
        if(state == mInteractor.getBookStateGood()) {
            mView.setStateGood();
        }
        else if(state == mInteractor.getBookStateRegular()) {
            mView.setStateRegular();
        }
        else {
            mView.setStateBad();
        }

        // Establece la acción principal, en función
        // de si se está visualizando un libro propio
        // o de otro usuario.
        if(mOwnBook) {
            // Libro propio: se puede eliminar.
            mView.setupDeleteFab();
        }
        else {
            // Libro de otro usuario: se puede solicitar.
            mView.setupSwapFab();
        }
    }

    /**
     * @see BookDetailContract.Presenter#backButtonPressed()
     */
    @Override
    public void backButtonPressed() {
        // Establecemos la navegación a la pantalla anterior
        // en función de si se está visualizando un libro propio
        // o de otro usuario.
        if(mOwnBook) {
            // Libro propio: volvemos al listado de "Mis libros".
            mView.navigateToMyBooks(null);
        }
        else {
            // Libro de otro usuario: volvemos al detalle del usuario.
            mView.navigateToUserDetail();
        }
    }

    /**
     * @see BookDetailContract.Presenter#mainActionClicked()
     */
    @Override
    public void mainActionClicked() {
        // La accción del botón flotante depende de
        // si se está visualizando un libro propio
        // o de otro usuario.
        if(mOwnBook) {
            // Libro propio: eliminar libro.
            mView.showDeletionDialog();
        }
        else {
            // Libro de otro usuario: solicitar libro.
            mView.showSwapDialog();
        }
    }

    /**
     * @see BookDetailContract.Presenter#delete()
     */
    @Override
    public void delete() {
        mView.showProgress(true);

        // Eliminamos el libro.
        mInteractor.deleteBook(mBookData, new ServiceCallback<Boolean>() {
            @Override
            public void onSuccess(Boolean deleted) {
                mView.showProgress(false);

                // Si el borrado es correcto mostramos un mensaje
                // y volvemos al listado de libros del usuario.
                if(deleted) {
                    mView.showBookDeletionOk();
                    mView.navigateToMyBooks(mBookData);
                }
                // Si el libro no se puede borrar porque se está
                // usando en un intercambio pendiente, mostramos
                // un mensaje.
                else {
                    mView.showBookDeletionNotPossible();
                }
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce un error al borrar,
                // mostramos un mensaje.
                mView.showProgress(false);
                mView.showBookDeletionError();
            }
        });
    }

    /**
     * @see BookDetailContract.Presenter#requestSwap()
     */
    @Override
    public void requestSwap() {
        // Si venimos de la pantalla de detalle de usuario significa
        // que vamos a iniciar un nuevo intercambio.
        if(Utils.isEmptyString(mSwapId)) {
            requestNewSwap();
        }
        // Si venimos de la pantalla de detalle de intercambio significa
        // que estamos aceptando un intercambio que nos han pedido.
        else {
            acceptSwap();
        }
    }

    /**
     * @see BookDetailContract.Presenter#deleteCanceled()
     */
    @Override
    public void deleteCanceled() {
        // Cerramos el diálogo y no hacemos nada.
        mView.dismissDeletionDialog();
    }

    /**
     * @see BookDetailContract.Presenter#requestSwapCanceled()
     */
    @Override
    public void requestSwapCanceled() {
        // Cerramos el diálogo y no hacemos nada.
        mView.dismissSwapDialog();
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Inicia un nuevo intercambio.
     */
    private void requestNewSwap() {
        mView.showProgress(true);
        String destinationUserId = mBookData.getOwner();
        String bookId = mBookData.getId();

        // Iniciamos un nuevo intercambio solicitando el libro al usuario.
        mInteractor.requestNewSwap(destinationUserId, bookId, new ServiceCallback<UiSwap>() {
            @Override
            public void onSuccess(UiSwap swapResponse) {
                mView.showProgress(false);

                // Si la inserción es correcta mostramos un mensaje
                // y volvemos al listado de intercambios del usuario.
                if(swapResponse != null) {
                    mView.showSwapRequestOk();
                    mView.navigateToSwaps(swapResponse);
                }
                else {
                    // Si el libro ya ha sido pedido y está aún
                    // pendiente de aceptar, mostramos un mensaje.
                    mView.showSwapRequestNotPossible();
                }
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce un error al registrar el intercambio,
                // mostramos un mensaje.
                mView.showProgress(false);
                mView.showSwapRequestError();
            }
        });
    }

    /**
     * Acepta un intercambio recibido por el usuario.
     */
    private void acceptSwap() {
        mView.showProgress(true);
        String bookId = mBookData.getId();

        // Aceptamos el intercambio que nos han solicitado.
        // La aceptación se produce al solicitar un libro al otro usuario.
        mInteractor.acceptSwap(mSwapId, bookId, new ServiceCallback<UiSwap>() {
            @Override
            public void onSuccess(UiSwap swapResponse) {
                mView.showProgress(false);

                // Si la actualización es correcta mostramos un mensaje
                // y volvemos al detalle del intercambio.
                if(swapResponse != null) {
                    String bookTitle = mBookData.getTitulo();
                    String bookPicture = mBookData.getFoto();

                    mView.showSwapRequestOk();
                    mView.navigateToSwapDetail(bookTitle, bookPicture);
                }
                else {
                    // Si el libro ya ha sido pedido y está aún
                    // pendiente de aceptar, mostramos un mensaje.
                    mView.showSwapRequestNotPossible();
                }
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce un error al actualizar el intercambio,
                // mostramos un mensaje.
                mView.showProgress(false);
                mView.showSwapRequestError();
            }
        });
    }
}
