package edu.uoc.jsev.b4b.ui.mybooks.newbook;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.NavUtils;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.zxing.integration.android.IntentIntegrator;
import com.google.zxing.integration.android.IntentResult;
import com.squareup.picasso.Picasso;

import java.util.List;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.util.ContextUtils;
import edu.uoc.jsev.b4b.Injection;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Actividad de la pantalla de registro de un nuevo libro.
 */
public class NewBookActivity extends BaseActivity implements NewBookContract.View {

    // Códigos para solicitar permisos y recibir respuestas de actividades.
    private static final int TAKE_PICTURE_REQUEST_CODE = 100;
    private static final int PICK_PICTURE_REQUEST_CODE = 101;
    private static final int CAMERA_PERMISSION_REQUEST_CODE = 102;

    // Tags para identificar los diálogos.
    private static final String TAG_PICTURE_SOURCE_DIALOG_FRAGMENT = "pictureSourceDialog";

    // Referencias a las vistas.
    private ImageView mPictureView;
    private TextInputLayout mIsbnLayout;
    private TextInputLayout mTitleLayout;
    private TextInputLayout mAuthorLayout;
    private TextInputLayout mPublisherLayout;
    private TextInputLayout mGenreLayout;
    private TextInputLayout mLanguageLayout;
    private TextView mStateLabelView;
    private ImageButton mStateBadView;
    private ImageButton mStateRegularView;
    private ImageButton mStateGoodView;

    // Adaptadores para mostrar las listas de selección
    // de géneros literarios e idiomas.
    private ArrayAdapter<String> mGenreAdapter;
    private ArrayAdapter<String> mLanguageAdapter;

    // Presentador.
    private NewBookContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_book);

        // Inyectamos el presentador.
        mPresenter = Injection.provideNewBookPresenter(this);

        // Configuramos las vistas.
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        mPictureView = findViewById(R.id.picture);
        mIsbnLayout = findViewById(R.id.isbn);
        mTitleLayout = findViewById(R.id.title);
        mAuthorLayout = findViewById(R.id.author);
        mPublisherLayout = findViewById(R.id.publisher);
        mGenreLayout = findViewById(R.id.genre);
        mLanguageLayout = findViewById(R.id.language);
        mStateLabelView = findViewById(R.id.state_label);

        // Lista de selección de géneros literarios.
        AutoCompleteTextView genreView = findViewById(R.id.genre_text);
        mGenreAdapter = new ArrayAdapter<>(this, android.R.layout.simple_spinner_dropdown_item);
        setupAutoCompleteTextView(genreView, mGenreAdapter);

        // Lista de selección de idiomas.
        AutoCompleteTextView languageView = findViewById(R.id.language_text);
        mLanguageAdapter = new ArrayAdapter<>(this, android.R.layout.simple_spinner_dropdown_item);
        setupAutoCompleteTextView(languageView, mLanguageAdapter);

        // Botón para seleccionar el origen de la fotografía del libro.
        ImageButton pictureSearchView = findViewById(R.id.picture_search);
        pictureSearchView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.selectPictureButtonClicked();
            }
        });

        // Botones de selección del estado del libro.
        mStateBadView = findViewById(R.id.state_bad);
        mStateBadView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.stateBadButtonClicked();
            }
        });

        mStateRegularView = findViewById(R.id.state_regular);
        mStateRegularView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.stateRegularButtonClicked();
            }
        });

        mStateGoodView = findViewById(R.id.state_good);
        mStateGoodView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.stateGoodButtonClicked();
            }
        });

        // La acción principal de la pantalla será buscar el libro por su ISBN,
        // para rellenar sus datos automáticamente.
        mProgressView = findViewById(R.id.progress);
        mFab = findViewById(R.id.fab);
        mFab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.searchIsbnButtonClicked(mIsbnLayout.getEditText().getText().toString());
            }
        });

        // Coloreamos los iconos de los TextView.
        TextView[] textViews = new TextView[] { mIsbnLayout.getEditText(), mTitleLayout.getEditText(),
                mAuthorLayout.getEditText(), mPublisherLayout.getEditText(), mGenreLayout.getEditText(),
                mLanguageLayout.getEditText(), mStateLabelView };
        ContextUtils.tintCompoundDrawables(textViews, R.color.colorPrimary);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Menú de la actividad.
        getMenuInflater().inflate(R.menu.menu_new_book, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        // Se pulsa sobre le opción añadir libro.
        if(itemId == R.id.action_add_book) {
            addBook();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(resultCode == RESULT_OK) {
            // Se vuelve desde la toma de imágenes con la cámara.
            if (requestCode == TAKE_PICTURE_REQUEST_CODE && data.getExtras() != null) {
                // Pasamos la imagen tomada al presentador.
                mPresenter.pictureSelected(data.getExtras().get("data"));
            }
            // Se vuelve desde la selección de imágenes en la galería.
            else if (requestCode == PICK_PICTURE_REQUEST_CODE) {
                // Pasamos la imagen seleccionada al presentador.
                mPresenter.pictureSelected(data.getData());
            }
            // Se vuelve de escanear el código ISBN del libro con la cámara.
            else if (requestCode == IntentIntegrator.REQUEST_CODE) {
                // Pasamos el código ISBN leído al presentador.
                IntentResult result = IntentIntegrator.parseActivityResult(requestCode, resultCode, data);
                mPresenter.searchIsbn(result.getContents());
            }
        }
    }

    @Override
    public boolean onSupportNavigateUp() {
        // Navegación al pulsar el botón "Up".
        mPresenter.upButtonPressed();
        return true;
    }

    @Override
    public void onBackPressed() {
        // Navegación al pulsar el botón "Back".
        mPresenter.backButtonPressed();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        // Se vuelve de solicitar el permiso para acceder a la cámara.
        if (requestCode == CAMERA_PERMISSION_REQUEST_CODE) {
            // El usuario da permiso de acceso a la cámara.
            if (grantResults.length == 1
                    && permissions[0].equals(Manifest.permission.CAMERA)
                    && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                mPresenter.cameraPermissionGranted();
            }
            // El usuario deniega el permiso de acceso a la cámara.
            else {
                mPresenter.cameraPermissionDenied();
            }
        }
        else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    /**
     * @see NewBookContract.View#setGenres(List)
     */
    @Override
    public void setGenres(List<String> genres) {
        mGenreAdapter.addAll(genres);
    }

    /**
     * @see NewBookContract.View#clearGenres()
     */
    @Override
    public void clearGenres() {
        mGenreAdapter.clear();
    }

    /**
     * @see NewBookContract.View#setLanguages(List)
     */
    @Override
    public void setLanguages(List<String> languages) {
        mLanguageAdapter.addAll(languages);
    }

    /**
     * @see NewBookContract.View#clearLanguages()
     */
    @Override
    public void clearLanguages() {
        mLanguageAdapter.clear();
    }

    /**
     * @see NewBookContract.View#checkStateBadButton()
     */
    @Override
    public void checkStateBadButton() {
        mStateBadView.setBackgroundColor(getResources().getColor(R.color.colorAccent));
    }

    /**
     * @see NewBookContract.View#uncheckStateBadButton()
     */
    @Override
    public void uncheckStateBadButton() {
        mStateBadView.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
    }

    /**
     * @see NewBookContract.View#checkStateRegularButton()
     */
    @Override
    public void checkStateRegularButton() {
        mStateRegularView.setBackgroundColor(getResources().getColor(R.color.colorAccent));
    }

    /**
     * @see NewBookContract.View#uncheckStateRegularButton()
     */
    @Override
    public void uncheckStateRegularButton() {
        mStateRegularView.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
    }

    /**
     * @see NewBookContract.View#checkStateGoodButton()
     */
    @Override
    public void checkStateGoodButton() {
        mStateGoodView.setBackgroundColor(getResources().getColor(R.color.colorAccent));
    }

    /**
     * @see NewBookContract.View#uncheckStateGoodButton()
     */
    @Override
    public void uncheckStateGoodButton() {
        mStateGoodView.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
    }

    /**
     * @see NewBookContract.View#navigateToIsbnSearch()
     */
    @Override
    public void navigateToIsbnSearch() {
        IntentIntegrator integrator = new IntentIntegrator(this);
        integrator.setPrompt(getString(R.string.prompt_scan_isbn));
        integrator.setOrientationLocked(true);
        integrator.setBeepEnabled(true);
        integrator.setBarcodeImageEnabled(false);
        integrator.initiateScan();
    }

    /**
     * @see NewBookContract.View#setPicture(String)
     */
    @Override
    public void setPicture(String url) {
        Picasso.get()
                .load(url)
                .placeholder(R.drawable.book_placeholder)
                .into(mPictureView);
    }

    /**
     * @see NewBookContract.View#setPicture(Object)
     */
    @Override
    public void setPicture(Object picture) {
        // Se ha tomado con la cámara una nueva foto.
        if(picture instanceof Bitmap) {
            mPictureView.setImageBitmap((Bitmap) picture);
        }
        // Se ha seleccionado la foto desde la galería de imágenes.
        else if (picture instanceof Uri) {
            mPictureView.setImageURI((Uri) picture);
        }
    }

    /**
     * @see NewBookContract.View#setTitle(String)
     */
    @Override
    public void setTitle(String title) {
        mTitleLayout.getEditText().setText(title);
    }

    /**
     * @see NewBookContract.View#setAuthor(String)
     */
    @Override
    public void setAuthor(String author) {
        mAuthorLayout.getEditText().setText(author);
    }

    /**
     * @see NewBookContract.View#setIsbn(String)
     */
    @Override
    public void setIsbn(String isbn) {
        mIsbnLayout.getEditText().setText(isbn);
    }

    /**
     * @see NewBookContract.View#setPublisher(String)
     */
    @Override
    public void setPublisher(String publisher) {
        mPublisherLayout.getEditText().setText(publisher);
    }

    /**
     * @see NewBookContract.View#showIsbnNotFoundMessage()
     */
    @Override
    public void showIsbnNotFoundMessage() {
        showToastMessage(R.string.message_no_isbn_found);
    }

    /**
     * @see NewBookContract.View#showIsbnScanningError()
     */
    @Override
    public void showIsbnScanningError() {
        showToastMessage(R.string.error_scanning_isbn);
    }

    /**
     * @see NewBookContract.View#showIsbnSearchError()
     */
    @Override
    public void showIsbnSearchError() {
        showToastMessage(R.string.error_searching_isbn);
    }

    /**
     * @see NewBookContract.View#resetValidationErrors()
     */
    @Override
    public void resetValidationErrors() {
        mTitleLayout.setErrorEnabled(false);
        mAuthorLayout.setErrorEnabled(false);
        mPublisherLayout.setErrorEnabled(false);
        mGenreLayout.setErrorEnabled(false);
        mLanguageLayout.setErrorEnabled(false);
    }

    /**
     * @see NewBookContract.View#showEmptyTitleError()
     */
    @Override
    public void showEmptyTitleError() {
        mTitleLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see NewBookContract.View#showEmptyAuthorError()
     */
    @Override
    public void showEmptyAuthorError() {
        mAuthorLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see NewBookContract.View#showEmptyPublisherError()
     */
    @Override
    public void showEmptyPublisherError() {
        mPublisherLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see NewBookContract.View#showEmptyGenreError()
     */
    @Override
    public void showEmptyGenreError() {
        mGenreLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see NewBookContract.View#showEmptyLanguageError()
     */
    @Override
    public void showEmptyLanguageError() {
        mLanguageLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see NewBookContract.View#showCameraNotAvailableError()
     */
    @Override
    public void showCameraNotAvailableError() {
        showToastMessage(R.string.error_camera_not_available);
    }

    /**
     * @see NewBookContract.View#showCameraPermissionDeniedError()
     */
    @Override
    public void showCameraPermissionDeniedError() {
        showToastMessage(R.string.error_camera_permission_denied);
    }

    /**
     * @see NewBookContract.View#showBookAlreadyRegistered()
     */
    @Override
    public void showBookAlreadyRegistered() {
        showToastMessage(R.string.message_book_already_registered);
    }

    /**
     * @see NewBookContract.View#showAddBookOk()
     */
    @Override
    public void showAddBookOk() {
        showToastMessage(R.string.message_book_added_ok);
    }

    /**
     * @see NewBookContract.View#showAddBookOkWithNoPicture()
     */
    @Override
    public void showAddBookOkWithNoPicture() {
        showToastMessage(R.string.message_book_added_ok_with_no_picture);
    }

    /**
     * @see NewBookContract.View#showAddBookError()
     */
    @Override
    public void showAddBookError() {
        showToastMessage(R.string.error_adding_book);
    }

    /**
     * @see NewBookContract.View#showPictureSourceDialog()
     */
    @Override
    public void showPictureSourceDialog() {
        int sources[] = new int[] { R.string.prompt_take_picture, R.string.prompt_pick_picture };
        showOptionsDialog(R.string.action_set_picture, sources, TAG_PICTURE_SOURCE_DIALOG_FRAGMENT);
    }

    /**
     * @see NewBookContract.View#dismissPictureSourceDialog()
     */
    @Override
    public void dismissPictureSourceDialog() {
        dismissDialog(TAG_PICTURE_SOURCE_DIALOG_FRAGMENT);
    }

    /**
     * @see NewBookContract.View#isCameraPermissionGranted()
     */
    @Override
    public boolean isCameraPermissionGranted() {
        return ContextUtils.isPermissionGranted(this, Manifest.permission.CAMERA);
    }

    /**
     * @see NewBookContract.View#requestCameraPermission()
     */
    @Override
    public void requestCameraPermission() {
        ActivityCompat.requestPermissions(this,
                new String[]{ Manifest.permission.CAMERA },
                CAMERA_PERMISSION_REQUEST_CODE);
    }

    /**
     * @see NewBookContract.View#navigateToTakePicture()
     */
    @Override
    public void navigateToTakePicture() {
        Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        if (takePictureIntent.resolveActivity(getPackageManager()) != null) {
            startActivityForResult(takePictureIntent, TAKE_PICTURE_REQUEST_CODE);
        }
        else {
            mPresenter.cameraNotAvailable();
        }
    }

    /**
     * @see NewBookContract.View#navigateToPickPicture()
     */
    @Override
    public void navigateToPickPicture() {
        Intent pickPictureIntent = new Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        startActivityForResult(Intent.createChooser(pickPictureIntent, getString(R.string.action_select_option)), PICK_PICTURE_REQUEST_CODE);
    }

    /**
     * @see NewBookContract.View#navigateToMyBooks(UiBook)
     */
    @Override
    public void navigateToMyBooks(UiBook book) {
        // Navegamos a la pantalla de listado de libros.
        Intent intent = NavUtils.getParentActivityIntent(this);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);

        // Informamos al listado de libros de si se debe actualizar.
        // Si se envía un libro significa que ha sido insertado. Se
        // lo enviamos a la actividad listado para que actualice sus datos
        // y lo muestre también en el listado.
        if(book != null) {
            intent.setAction(Intent.ACTION_INSERT);
            intent.putExtra(MainActivity.EXTRA_DATA_OBJECT, book);
        }

        NavUtils.navigateUpTo(this, intent);
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogItemClick(String, int)
     */
    @Override
    public void onDialogItemClick(String tag, int which) {
        if(tag.equals(TAG_PICTURE_SOURCE_DIALOG_FRAGMENT)) {
            // Se ha seleccionado tomar la imagen del libro con la cámara.
            if(which == 0) {
                mPresenter.takePictureOptionClicked();
            }
            // Se ha seleccionado tomar la imagen del libro desde la galería.
            else if(which == 1){
                mPresenter.pickPictureOptionClicked();
            }
        }
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        if(tag.equals(TAG_PICTURE_SOURCE_DIALOG_FRAGMENT)) {
            // El usuario cancela la selección de la imagen.
            mPresenter.selectPictureDialogCanceled();
        }
    }

    /*** MÉTODOS AUXILIARES***/

    /**
     * Establece el adaptador de datos en un campo de tipo autocompletado.
     */
    @SuppressLint("ClickableViewAccessibility")
    private void setupAutoCompleteTextView(AutoCompleteTextView view, ArrayAdapter<String> adapter) {
        view.setAdapter(adapter);
        view.setKeyListener(null);
        // Simulamos un desplegable sobrescribiendo el evento onTouch.
        // Cuando el usuario pulse el control se mostrará la lista de elementos.
        view.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if(event.getAction() == MotionEvent.ACTION_UP) {
                    ((AutoCompleteTextView) v).showDropDown();
                    return true;
                }
                return false;
            }
        });
    }

    /**
     * Método auxiliar para iniciar el proceso de guardado del libro,
     * enviando los datos introducidos por el usuario al presentador.
     */
    private void addBook() {
        String title = mTitleLayout.getEditText().getText().toString();
        String author = mAuthorLayout.getEditText().getText().toString();
        String isbn = mIsbnLayout.getEditText().getText().toString();
        String publisher = mPublisherLayout.getEditText().getText().toString();
        String genre = mGenreLayout.getEditText().getText().toString();
        String language = mLanguageLayout.getEditText().getText().toString();
        byte[] picture = Utils.getBytesFromImageView(mPictureView);

        mPresenter.addBook(title, author, isbn, publisher, genre, language, picture);
    }
}
