package edu.uoc.jsev.b4b.ui.mybooks.newbook;

import java.util.List;

import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla de alta de un nuevo libro.
 */
public interface NewBookContract {

    /**
     * Vista de la pantalla de alta de nuevo libro.
     */
    interface View extends BaseContract.View {
        /**
         * Establece la lista de géneros literarios
         * en su correspondiente desplegable.
         * @param genres La lista de géneros.
         */
        void setGenres(List<String> genres);

        /**
         * Elimina las opciones del desplagable de géneros literarios.
         */
        void clearGenres();

        /**
         * Establece la lista de idiomas
         * en su correspondiente desplegable.
         * @param languages La lista de idiomas.
         */
        void setLanguages(List<String> languages);

        /**
         * Elimina las opciones del desplagable de idiomas.
         */
        void clearLanguages();

        /**
         * Activa el botón de estado de libro "Malo".
         */
        void checkStateBadButton();

        /**
         * Desactiva el botón de estado de libro "Malo".
         */
        void uncheckStateBadButton();

        /**
         * Activa el botón de estado de libro "Regular".
         */
        void checkStateRegularButton();

        /**
         * Desactiva el botón de estado de libro "Regular".
         */
        void uncheckStateRegularButton();

        /**
         * Activa el botón de estado de libro "Bueno".
         */
        void checkStateGoodButton();

        /**
         * Desactiva el botón de estado de libro "Bueno".
         */
        void uncheckStateGoodButton();

        /**
         * Navega hacia la actividad para escanear
         * el código ISBN de un libro con la cámara del dispositivo.
         */
        void navigateToIsbnSearch();

        /**
         * Establece en la pantalla la fotografía del libro.
         * @param url La URL de descarga de la fotografía.
         */
        void setPicture(String url);

        /**
         * Establece en la pantalla la fotografía del libro.
         * @param picture Fotografía en formato binario.
         */
        void setPicture(Object picture);

        /**
         * Establece en la pantalla el título del libro.
         * @param title El título del libro.
         */
        void setTitle(String title);

        /**
         * Establece en la pantalla el autor del libro.
         * @param author El autor del libro.
         */
        void setAuthor(String author);

        /**
         * Establece en la pantalla el código ISBN del libro.
         * @param isbn El código ISBN del libro.
         */
        void setIsbn(String isbn);

        /**
         * Establece en la pantalla la editorial del libro.
         * @param publisher La editorial del libro.
         */
        void setPublisher(String publisher);

        /**
         * Muestra un mensaje indicando que no se ha encontrado
         * ningún libro al realizar la búsqueda por ISBN.
         */
        void showIsbnNotFoundMessage();

        /**
         * Muestra un mensaje indicando que se ha producido
         * un error al escanear el código ISBN del libro con
         * la cámara.
         */
        void showIsbnScanningError();

        /**
         * Muestra un mensaje indicando que se ha producido
         * un error al intentar obtener los datos del libro
         * por su ISBN.
         */
        void showIsbnSearchError();

        /**
         * Limpia los errores de validación
         * de los campos de texto de la pantalla.
         */
        void resetValidationErrors();

        /**
         * Muestra un mensaje de error indicando que
         * el campo título está vacío y es obligatorio.
         */
        void showEmptyTitleError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo autor está vacío y es obligatorio.
         */
        void showEmptyAuthorError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo editorial está vacío y es obligatorio.
         */
        void showEmptyPublisherError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo género está vacío y es obligatorio.
         */
        void showEmptyGenreError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo idioma está vacío y es obligatorio.
         */
        void showEmptyLanguageError();

        /**
         * Muestra un mensaje de error indicando que
         * el dispositivo no dispone de cámara o ésta
         * no está disponible.
         */
        void showCameraNotAvailableError();

        /**
         * Muestra un mensaje de error indicando que
         * el usuario ha denegado el permiso de acceso
         * a la cámara.
         */
        void showCameraPermissionDeniedError();

        /**
         * Muestra un mensaje de error indicando que
         * el libro no puede registrarse porque ya
         * existe uno con el mismo código ISBN.
         */
        void showBookAlreadyRegistered();

        /**
         * Muestra un mensaje de indicando que
         * el registro del libro se ha realizado
         * correctamente.
         */
        void showAddBookOk();

        /**
         * Muestra un mensaje de indicando que
         * el registro del libro se ha realizado
         * correctamente, pero no se ha podido
         * guardar la imagen proporcionada por el
         * usuario.
         */
        void showAddBookOkWithNoPicture();

        /**
         * Muestra un mensaje de error indicando que
         * no se ha podido registrar el libro.
         */
        void showAddBookError();

        /**
         * Muestra un diálogo para que el usuario
         * seleccione la fuente desde la que obtener
         * la imagen del libro (cámara o galería).
         */
        void showPictureSourceDialog();

        /**
         * Descarta el diálogo de selección de fuente
         * de la imagen.
         */
        void dismissPictureSourceDialog();

        /**
         * Comprueba si se tiene permiso para acceder
         * a la cámara del dispositivo.
         * @return true si se tiene permiso, false en
         * caso contrario.
         */
        boolean isCameraPermissionGranted();

        /**
         * Solicita permiso al usuario para acceder
         * a la cámara del dispositivo.
         */
        void requestCameraPermission();

        /**
         * Navega a la actividad para tomar una foto
         * con la cámara del dispositivo.
         */
        void navigateToTakePicture();

        /**
         * Navega a la actividad para seleccionar
         * una foto de la galería de imágenes.
         */
        void navigateToPickPicture();

        /**
         * Navega hacia el listado de libros.
         * @param book Datos del libro añadido, para
         *             que aparezca también en la actividad listado.
         *             Si no se ha añadido ningún libro,
         *             el parámetro es null y la actividad listado
         *             no realiza ninguna acción.
         */
        void navigateToMyBooks(UiBook book);
    }

    /**
     * Presentador de la pantalla de alta de nuevo libro.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * El usuario ha pulsado sobre el botón
         * de estado del libro "Malo".
         */
        void stateBadButtonClicked();

        /**
         * El usuario ha pulsado sobre el botón
         * de estado del libro "Regular".
         */
        void stateRegularButtonClicked();

        /**
         * El usuario ha pulsado sobre el botón
         * de estado del libro "Bueno".
         */
        void stateGoodButtonClicked();

        /**
         * El usuario ha pulsado sobre el botón
         * para añadir una fotografía al libro.
         */
        void selectPictureButtonClicked();

        /**
         * El usuario ha seleccionado la opción de
         * añadir una foto al libro haciéndola con
         * la cámara del dispositivo.
         */
        void takePictureOptionClicked();

        /**
         * El usuario ha seleccionado la opción de
         * añadir una foto al libro tomándola de
         * la galería de imágenes.
         */
        void pickPictureOptionClicked();

        /**
         * El usuario ha pulsado sobre el botón "Cancelar"
         * del diálogo de selección de fuente de la fotogorafía.
         */
        void selectPictureDialogCanceled();

        /**
         * El usuario ha dado permiso para acceder a la cámara.
         */
        void cameraPermissionGranted();

        /**
         * El usuario ha denegado el permiso para acceder a la cámara.
         */
        void cameraPermissionDenied();

        /**
         * La cámara del dispositivo no está disponible.
         */
        void cameraNotAvailable();

        /**
         * El usuario ha seleccionado una foto de la galería
         * o tomado una nueva con la cámara.
         * @param picture La foto seleccionada/tomada.
         */
        void pictureSelected(Object picture);

        /**
         * El usuario ha pulsado sobre el botón
         * de búsqueda de libro por ISBN.
         * @param isbn El ISBN a buscar.
         */
        void searchIsbnButtonClicked(String isbn);

        /**
         * Busca los datos de un libro por su ISBN.
         * @param isbn El ISBN a buscar.
         */
        void searchIsbn(String isbn);

        /**
         * El usuario pulsa sobre el botón de registro para crear el libro.
         * @param title El título del libro.
         * @param author El autor del libro.
         * @param isbn El código ISBN del libro.
         * @param publisher La editorial del libro.
         * @param genre El género literario del libro.
         * @param language El idioma del libro.
         * @param picture La fotografía del libro.
         */
        void addBook(String title, String author, String isbn,
                     String publisher, String genre, String language, byte[] picture);
    }
}
