package edu.uoc.jsev.b4b.ui.mybooks.newbook;

import android.util.Log;

import com.backendless.BackendlessUser;

import java.util.Map;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Libro;
import edu.uoc.jsev.b4b.data.backendless.model.Usuario;
import edu.uoc.jsev.b4b.data.backendless.service.BookService;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.openlibrary.model.OpenLibraryBook;
import edu.uoc.jsev.b4b.data.openlibrary.service.OpenLibraryService;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.base.BaseBooksInteractorImpl;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Clase de implementación del interactor de la pantalla de nuevo libro.
 */
public class NewBookInteractorImpl extends BaseBooksInteractorImpl implements NewBookInteractor {

    // Instancia singleton.
    private static NewBookInteractor instance;

    private NewBookInteractorImpl(UserService userService,
                                  CommonsService commonsService,
                                  BookService bookService) {
        super(userService, commonsService, bookService);
    }

    // Devuelve el singleton.
    public static NewBookInteractor getInstance(UserService userService,
                                                CommonsService commonsService,
                                                BookService bookService) {
        if (instance == null) {
            instance = new NewBookInteractorImpl(userService, commonsService, bookService);
        }
        return instance;
    }

    /**
     * @see NewBookInteractor#getBookByIsbn(String, ServiceCallback)
     */
    @Override
    public void getBookByIsbn(final String isbn, final ServiceCallback<UiBook> callback) {
        // Consultamos la base de datos OpenLibrary para
        // intentar obtener los datos del libro por su ISBN.
        final String queryParam = "ISBN:" + isbn;
        OpenLibraryService.getInstance().getBookByIsbn(queryParam).enqueue(new Callback<Map<String, OpenLibraryBook>>() {
            @Override
            public void onResponse(Call<Map<String, OpenLibraryBook>> call, Response<Map<String, OpenLibraryBook>> response) {
                if(response.isSuccessful()) {
                    OpenLibraryBook responseBook = response.body().get(queryParam);
                    UiBook book = openLibraryMapper.from(responseBook);
                    if(book != null) book.setIsbn(isbn);
                    callback.onSuccess(book);
                }
                else {
                    callback.onError(new ServiceFault("OpenLibrary response error"));
                }
            }

            @Override
            public void onFailure(Call<Map<String, OpenLibraryBook>> call, Throwable t) {
                callback.onError(new ServiceFault(t.getMessage()));
            }
        });
    }

    /**
     * @see NewBookInteractor#addBook(UiBook, byte[], ServiceCallback)
     */
    @Override
    public void addBook(final UiBook uiBook, final byte[] picture, final ServiceCallback<UiBook> callback) {
        // Comprobamos que el usuario no haya registrado
        // ya un libro con el mismo código ISBN.
        mBookService.bookExists(uiBook.getIsbn(), getCurrentUserId(), new ServiceCallback<Boolean>() {
            @Override
            public void onSuccess(Boolean exists) {
                if(exists) {
                    // Si ya existe un libro con ese ISBN, no lo guardamos.
                    callback.onSuccess(null);
                    return;
                }

                final Libro book = bookMapper.from(uiBook);
                final ServiceCallback<Libro> addBookCallback = buildAddBookCallback(callback);

                // Subimos la fotografía del libro al servidor.
                uploadPicture(picture, new ServiceCallback<String>() {
                    @Override
                    public void onSuccess(String pictureUrl) {
                        // Asociamos al libro la URL de descarga
                        // de la foto y lo insertamos.
                        book.setFoto(pictureUrl);
                        mBookService.saveBook(book, addBookCallback);
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        // Si se produce algún error al subir la foto
                        // guardamos el libro, pero sin foto.
                        mBookService.saveBook(book, addBookCallback);
                        Log.w("--->", "No se pudo guardar la foto del libro: " + fault.getMessage());
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Callback para la llamada de inserción de libro.
     */
    private ServiceCallback<Libro> buildAddBookCallback(final ServiceCallback<UiBook> callback) {
        return new ServiceCallback<Libro>() {
            @Override
            public void onSuccess(final Libro bookResponse) {
                // Si el libro se inserta correctamente,
                // incrementamos el contador de libros del usuario.
                final BackendlessUser currentUser = mUserService.getCurrentUser();
                int booksNumber = (Integer) currentUser.getProperty(Usuario.BOOKS_NUMBER_KEY) + 1;

                // Actualizamos el contador localmente.
                currentUser.setProperty(Usuario.BOOKS_NUMBER_KEY, booksNumber);

                // Actualizamos el contador en el servidor.
                mUserService.updateUser(currentUser, new ServiceCallback<BackendlessUser>() {
                    @Override
                    public void onSuccess(BackendlessUser response) {
                        mUserService.setCurrentUser(currentUser);
                        callback.onSuccess(bookMapper.from(bookResponse));
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        callback.onError(fault);
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        };
    }
}
