package edu.uoc.jsev.b4b.ui.mybooks.newbook;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador de la pantalla
 * de registro de un libro nuevo.
 */
public class NewBookPresenter extends BasePresenter implements NewBookContract.Presenter {

    // Vista e interactor asociados.
    private NewBookContract.View mView;
    private NewBookInteractor mInteractor;

    // Variable de respaldo para almacenar el estado
    // del libro actualmente seleccionado por el usuario.
    private int mCurrentSelectedState;

    public NewBookPresenter(NewBookContract.View view, NewBookInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see NewBookContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Establecemos la lista de géneros e idiomas.
        getGenres();
        getLanguages();

        // Marcamos, por defecto, el estado "Bueno".
        stateGoodButtonClicked();
    }

    /**
     * @see NewBookContract.Presenter#stateBadButtonClicked()
     */
    @Override
    public void stateBadButtonClicked() {
        // Marcamos el estado "Malo" y desmarcamos los demás.
        mView.checkStateBadButton();
        mView.uncheckStateRegularButton();
        mView.uncheckStateGoodButton();
        mCurrentSelectedState = mInteractor.getBookStateBad();
    }

    /**
     * @see NewBookContract.Presenter#stateRegularButtonClicked()
     */
    @Override
    public void stateRegularButtonClicked() {
        // Marcamos el estado "Regular" y desmarcamos los demás.
        mView.checkStateRegularButton();
        mView.uncheckStateBadButton();
        mView.uncheckStateGoodButton();
        mCurrentSelectedState = mInteractor.getBookStateRegular();
    }

    /**
     * @see NewBookContract.Presenter#stateGoodButtonClicked()
     */
    @Override
    public void stateGoodButtonClicked() {
        // Marcamos el estado "Bueno" y desmarcamos los demás.
        mView.checkStateGoodButton();
        mView.uncheckStateBadButton();
        mView.uncheckStateRegularButton();
        mCurrentSelectedState = mInteractor.getBookStateGood();
    }

    /**
     * @see NewBookContract.Presenter#selectPictureButtonClicked()
     */
    @Override
    public void selectPictureButtonClicked() {
        // Mostramos el diálogo de selección de la fuente de la foto.
        mView.showPictureSourceDialog();
    }

    /**
     * @see NewBookContract.Presenter#takePictureOptionClicked()
     */
    @Override
    public void takePictureOptionClicked() {
        // Si tenemos permiso para acceder a la cámara
        // abrimos la actividad para tomar la foto.
        if(mView.isCameraPermissionGranted()) {
            mView.navigateToTakePicture();
        }
        // Si no hay permiso, lo solicitamos.
        else {
            mView.requestCameraPermission();
        }
    }

    /**
     * @see NewBookContract.Presenter#pickPictureOptionClicked()
     */
    @Override
    public void pickPictureOptionClicked() {
        // Navegamos a la galería para seleccionar una imagen.
        mView.navigateToPickPicture();
    }

    /**
     * @see NewBookContract.Presenter#selectPictureDialogCanceled()
     */
    @Override
    public void selectPictureDialogCanceled() {
        // Descartamos el diálogo.
        mView.dismissPictureSourceDialog();
    }

    /**
     * @see NewBookContract.Presenter#cameraPermissionGranted()
     */
    @Override
    public void cameraPermissionGranted() {
        // Abrimos la actividad para tomar la foto con la cámara.
        mView.navigateToTakePicture();
    }

    /**
     * @see NewBookContract.Presenter#cameraPermissionDenied()
     */
    @Override
    public void cameraPermissionDenied() {
        // Mostramos un mensaje.
        mView.showCameraPermissionDeniedError();
    }

    /**
     * @see NewBookContract.Presenter#cameraNotAvailable()
     */
    @Override
    public void cameraNotAvailable() {
        // Mostramos un mensaje.
        mView.showCameraNotAvailableError();
    }

    /**
     * @see NewBookContract.Presenter#pictureSelected(Object)
     */
    @Override
    public void pictureSelected(Object picture) {
        // Establecemos la foto tomada/seleccionada en pantalla.
        if(picture != null) {
            mView.setPicture(picture);
        }
    }

    /**
     * @see NewBookContract.Presenter#searchIsbnButtonClicked(String)
     */
    @Override
    public void searchIsbnButtonClicked(String isbn) {
        mView.hideKeyboard();
        // Si se pulsa el botón y no hay ningún código
        // introducido por el usuario, abrimos la actividad
        // para escanear el código del libro con la cámara.
        if(Utils.isEmptyString(isbn)) {
            mView.navigateToIsbnSearch();
        }
        // Si se ha introducido algún código, lo buscamos directamente.
        else {
            searchIsbn(isbn);
        }
    }

    /**
     * @see NewBookContract.Presenter#searchIsbn(String)
     */
    @Override
    public void searchIsbn(String isbn) {
        if(!Utils.isEmptyString(isbn)) {
            mView.showProgress(true);
            // Intentamos obtener los datos del libro por su ISBN.
            mInteractor.getBookByIsbn(isbn, new ServiceCallback<UiBook>() {
                @Override
                public void onSuccess(UiBook response) {
                    mView.showProgress(false);
                    mView.resetValidationErrors();

                    // Si se encuentra el libro, cargamos sus datos en pantalla.
                    if (response != null) {
                        mView.setAuthor(response.getAutor());
                        mView.setIsbn(response.getIsbn());
                        mView.setPicture(response.getFoto());
                        mView.setPublisher(response.getEditorial());
                        mView.setTitle(response.getTitulo());
                    }
                    // Si no se encuentra, mostramos un mensaje.
                    else {
                        mView.showIsbnNotFoundMessage();
                    }
                }

                @Override
                public void onError(ServiceFault fault) {
                    // Si se produce un error al buscar,
                    // mostramos un mensaje.
                    mView.showProgress(false);
                    mView.showIsbnSearchError();
                }
            });
        }
        else {
            // Se ha producido un error al intentar escanear el ISBN del libro.
            mView.showIsbnScanningError();
        }
    }

    /**
     * @see NewBookContract.Presenter#addBook(String, String, String, String, String, String, byte[])
     */
    @Override
    public void addBook(String title, String author, String isbn, String publisher,
                        String genre, String language, byte[] picture) {
        boolean cancel = false;
        mView.resetValidationErrors();

        // Validamos la información introducida antes de registrar el libro:

        // El idioma es obligatorio.
        if (Utils.isEmptyString(language)) {
            mView.showEmptyLanguageError();
            cancel = true;
        }

        // El género literario es obligatorio.
        if (Utils.isEmptyString(genre)) {
            mView.showEmptyGenreError();
            cancel = true;
        }

        // La editorial es obligatoria.
        if(Utils.isEmptyString(publisher)) {
            mView.showEmptyPublisherError();
            cancel = true;
        }

        // El autor es obligatorio.
        if(Utils.isEmptyString(author)) {
            mView.showEmptyAuthorError();
            cancel = true;
        }

        // El título es obligatorio.
        if(Utils.isEmptyString(title)) {
            mView.showEmptyTitleError();
            cancel = true;
        }

        // Si los datos son correctos, intentamos el registro del libro.
        if(!cancel) {
            mView.hideKeyboard();
            doAddBook(title, author, isbn, publisher, genre, language, picture);
        }
    }

    /**
     * @see NewBookContract.Presenter#backButtonPressed()
     */
    @Override
    public void backButtonPressed() {
        // Volvemos al listado de libros.
        mView.navigateToMyBooks(null);
    }

    /*** MÉTODOS AUCILIARES ***/

    /**
     * Obtiene la lista de géneros literarios
     * y los carga en el campo desplegable de la pantalla.
     */
    private void getGenres() {
        final List<String> genres = new ArrayList<>();
        genres.add("");
        mView.clearGenres();
        mView.setGenres(genres);

        mInteractor.getAllGenres(new ServiceCallback<List<String>>() {
            @Override
            public void onSuccess(List<String> response) {
                // Carga la lista de géneros en el desplegable.
                genres.addAll(response);
                mView.clearGenres();
                mView.setGenres(genres);
            }

            @Override
            public void onError(ServiceFault fault) {

            }
        });
    }

    /**
     * Obtiene la lista de idiomas y los carga
     * en el campo desplegable de la pantalla.
     */
    private void getLanguages() {
        final List<String> languages = new ArrayList<>();
        languages.add("");
        mView.clearLanguages();
        mView.setLanguages(languages);

        mInteractor.getAllLanguages(new ServiceCallback<List<String>>() {
            @Override
            public void onSuccess(List<String> response) {
                // Carga la lista de idiomas en el desplegable.
                languages.addAll(response);
                mView.clearLanguages();
                mView.setLanguages(languages);
            }

            @Override
            public void onError(ServiceFault fault) {

            }
        });
    }

    /**
     * Solicita el registro del libro.
     */
    private void doAddBook(String title, String author, String isbn, String publisher,
                           String genre, String language, byte[] picture) {

        mView.showProgress(true);
        final UiBook newBook = new UiBook();
        newBook.setAutor(author);
        newBook.setEditorial(publisher);
        newBook.setEstado(mCurrentSelectedState);
        newBook.setIsbn(isbn);
        newBook.setOwner(mInteractor.getCurrentUserId());
        newBook.setTitulo(title);
        newBook.setGenero(genre);
        newBook.setIdioma(language);

        // Intentamos registrar el libro.
        mInteractor.addBook(newBook, picture, new ServiceCallback<UiBook>() {
            @Override
            public void onSuccess(UiBook bookResponse) {
                mView.showProgress(false);

                // Si ya existe un libro registrado por el usuario
                // con el mismo ISBN, mostramos un mensaje.
                if(bookResponse == null) {
                    mView.showBookAlreadyRegistered();
                    return;
                }

                // Si el libro se ha registrado pero ha habido algún
                // problema al guardar la foto, se muestra un mensaje
                // indicándolo.
                if(Utils.isEmptyString(bookResponse.getFoto())) {
                    mView.showAddBookOkWithNoPicture();
                }
                // Si el libro se ha registrado correctamente
                // se muestra un mensaje.
                else {
                    mView.showAddBookOk();
                }

                // Una vez registrado el libro, volvemos al listado de libros
                // y enviamos el libro insertado, para que aparezca también
                // en el listado.
                mView.navigateToMyBooks(bookResponse);
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce algún error al registrar
                // el libro, mostramos un mensaje.
                mView.showProgress(false);
                mView.showAddBookError();
            }
        });
    }
}
