package edu.uoc.jsev.b4b.ui.registration;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.util.TypedValue;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;

import com.google.android.gms.common.api.Status;
import com.google.android.gms.location.places.AutocompleteFilter;
import com.google.android.gms.location.places.Place;
import com.google.android.gms.location.places.ui.PlaceAutocompleteFragment;
import com.google.android.gms.location.places.ui.PlaceSelectionListener;
import com.google.android.gms.maps.model.LatLng;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.Injection;

/**
 * Actividad para gestionar la creación de una nueva cuenta de usuario.
 * Esta actividad utiliza la API Google Places para permitir que el usuario
 * introduzca de forma sencilla las coordenadas de su localización por defecto.
 * Estas coordenadas son útiles para poder realizar búsquedas de libros por
 * distancia cuando el usuario no activa la detección de la ubicación en su
 * dispositivo.
 */
public class RegistrationActivity extends BaseActivity implements RegistrationContract.View, PlaceSelectionListener {

    // Referencias a las vistas.
    private TextInputLayout mFullNameLayout;
    private TextInputLayout mEmailLayout;
    private TextInputLayout mPasswordLayout;
    private TextInputLayout mRepeatPasswordLayout;
    private EditText mLocationView;
    private View mLocationBorder;
    private TextView mLocationError;

    // Presentador.
    private RegistrationContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_registration);

        // Inyectamos el presentador.
        mPresenter = Injection.provideRegistrationPresenter(this);

        // Inicializamos las vistas.
        mFullNameLayout = findViewById(R.id.full_name);
        mEmailLayout = findViewById(R.id.email);
        mPasswordLayout = findViewById(R.id.password);
        mRepeatPasswordLayout = findViewById(R.id.repeat_password);
        mLocationBorder = findViewById(R.id.location_border);
        mLocationError = findViewById(R.id.location_error);

        // Configuramos el fragment a través del cual se integra la API de Google Places.
        setupPlaceFragment();

        Button registerButton = findViewById(R.id.register_button);
        registerButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                registration();
            }
        });

        TextView alreadyRegisteredLabel = findViewById(R.id.already_registered_label);
        alreadyRegisteredLabel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.login();
            }
        });

        mProgressView = findViewById(R.id.progress);
    }

    /**
     * @see RegistrationContract.View#resetValidationErrors()
     */
    @Override
    public void resetValidationErrors() {
        mFullNameLayout.setErrorEnabled(false);
        mEmailLayout.setErrorEnabled(false);
        mPasswordLayout.setErrorEnabled(false);
        mRepeatPasswordLayout.setErrorEnabled(false);
        disableLocationError();
    }

    /**
     * @see RegistrationContract.View#showEmptyFullNameError()
     */
    @Override
    public void showEmptyFullNameError() {
        mFullNameLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see RegistrationContract.View#showEmptyLocationError()
     */
    @Override
    public void showEmptyLocationError() {
        setLocationError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see RegistrationContract.View#showEmptyEmailError()
     */
    @Override
    public void showEmptyEmailError() {
        mEmailLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see RegistrationContract.View#showInvalidEmailError()
     */
    @Override
    public void showInvalidEmailError() {
        mEmailLayout.setError(getString(R.string.error_invalid_email));
    }

    /**
     * @see RegistrationContract.View#showEmptyPasswordError()
     */
    @Override
    public void showEmptyPasswordError() {
        mPasswordLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see RegistrationContract.View#showInvalidPasswordError()
     */
    @Override
    public void showInvalidPasswordError() {
        mPasswordLayout.setError(getString(R.string.error_invalid_password));
    }

    /**
     * @see RegistrationContract.View#showPasswordsDontMatchError()
     */
    @Override
    public void showPasswordsDontMatchError() {
        mRepeatPasswordLayout.setError(getString(R.string.error_password_matching));
    }

    /**
     * @see RegistrationContract.View#showRegistrationError()
     */
    @Override
    public void showRegistrationError() {
        showToastMessage(R.string.error_incorrect_registration);
    }

    /**
     * @see RegistrationContract.View#showLocationSearchError()
     */
    @Override
    public void showLocationSearchError() {
        showToastMessage(R.string.error_searching_location);
    }

    /**
     * @see RegistrationContract.View#navigateToLogin()
     */
    @Override
    public void navigateToLogin() {
        // Como venimos de la pantalla de login, para navegar a ella,
        // simplemente, cerramos la actividad de registro.
        finish();
    }

    /**
     * @see RegistrationContract.View#navigateToHome()
     */
    @Override
    public void navigateToHome() {
        // Navegamos a la pantalla principal y cerramos tanto
        // la pantalla de registro como la de login, de la que venimos.
        startActivity(new Intent(this, MainActivity.class));
        finishAffinity();
    }

    /*** CALLBACKS de PlaceSelectionListener ***/
    @Override
    public void onPlaceSelected(Place place) {
        // El usuario ha seleccionado un lugar.
        LatLng location = place.getLatLng();
        mPresenter.locationSelected(location.latitude, location.longitude);
    }

    @Override
    public void onError(Status status) {
        // Se ha producido un error en la selección del lugar.
        mPresenter.locationSearchError();
    }

    /**
     * Método auxiliar para configurar el fragmento de Google Places.
     */
    private void setupPlaceFragment() {
        PlaceAutocompleteFragment placeFragment = (PlaceAutocompleteFragment)
                getFragmentManager().findFragmentById(R.id.place_fragment);
        AutocompleteFilter filter = new AutocompleteFilter.Builder()
                .setCountry("ES")
                .build();
        placeFragment.setOnPlaceSelectedListener(this);
        placeFragment.setFilter(filter);

        // El fragment apenas ofrece opciones visuales de configuración.
        // Modificamos manualmente los estilos del EditText del buscador,
        // para que encajen con el estilo de la pantalla.
        View fragmentView = placeFragment.getView();
        ImageButton searchLocationView = fragmentView.findViewById(R.id.place_autocomplete_search_button);
        searchLocationView.setColorFilter(Color.WHITE);
        searchLocationView.setPadding(8, 0, 16, 4);

        ImageButton clearLocationView = fragmentView.findViewById(R.id.place_autocomplete_clear_button);
        clearLocationView.setColorFilter(Color.WHITE);
        clearLocationView.setPadding(16, 0, 8, 4);

        mLocationView = fragmentView.findViewById(R.id.place_autocomplete_search_input);
        mLocationView.setTextColor(Color.WHITE);
        mLocationView.setTextSize(TypedValue.COMPLEX_UNIT_SP, 18f);
        mLocationView.setHintTextColor(Color.WHITE);
        mLocationView.setHint("");
        mLocationView.setPadding(0, 0, 0, 4);
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Muestra un mensaje de error en el buscador de localizaciones.
     */
    private void setLocationError(String errorMessage) {
        mLocationError.setText(errorMessage);
        mLocationError.setVisibility(View.VISIBLE);
        mLocationBorder.setBackgroundColor(mLocationError.getCurrentTextColor());
    }

    /**
     * Desactiva el error en el buscador de localizaciones.
     */
    private void disableLocationError() {
        mLocationError.setVisibility(View.GONE);
        mLocationBorder.setBackgroundColor(mLocationView.getCurrentTextColor());
    }

    /**
     * Pasa los datos al presentador para que se registre la cuenta,
     * después de haber pulsado el usuario sobre el botón correspondiente.
     */
    private void registration() {
        String fullName = mFullNameLayout.getEditText().getText().toString();
        String email = mEmailLayout.getEditText().getText().toString();
        String password = mPasswordLayout.getEditText().getText().toString();
        String repeatPassword = mRepeatPasswordLayout.getEditText().getText().toString();
        String location = mLocationView.getText().toString();

        mPresenter.createAccount(fullName, email, password, repeatPassword, location);
    }
}
