package edu.uoc.jsev.b4b.ui.registration;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador de la pantalla de registro.
 */
public class RegistrationPresenter extends BasePresenter implements RegistrationContract.Presenter {

    // Vista e interactor asociados.
    private RegistrationContract.View mView;
    private RegistrationInteractor mInteractor;

    // Coordenadas de la localización seleccionada por el usuario.
    private Double mLocationLatitude;
    private Double mLocationLongitude;

    public RegistrationPresenter(RegistrationContract.View view, RegistrationInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see RegistrationContract.Presenter#locationSelected(double, double)
     */
    @Override
    public void locationSelected(double latitude, double longitude) {
        mLocationLatitude = latitude;
        mLocationLongitude = longitude;
    }

    /**
     * @see RegistrationContract.Presenter#locationSearchError()
     */
    @Override
    public void locationSearchError() {
        mView.showLocationSearchError();
    }

    /**
     * @see RegistrationContract.Presenter#createAccount(String, String, String, String, String)
     */
    @Override
    public void createAccount(String fullName, String email, String password, String repeatPassword, String location) {
        boolean cancelRegistration = false;
        mView.resetValidationErrors();

        // Validamos la información introducida antes de hacer el registro:

        // La contraseña es obligatoria, debe tener un mínimo de 6 caracteres
        // y debe coincidir con el campo de confirmación.
        if (Utils.isEmptyString(password)) {
            mView.showEmptyPasswordError();
            cancelRegistration = true;
        } else if (!Utils.hasMinLength(password, 6)) {
            mView.showInvalidPasswordError();
            cancelRegistration = true;
        } else if (!password.equals(repeatPassword)) {
            mView.showPasswordsDontMatchError();
            cancelRegistration = true;
        }

        // El email es obligatorio y debe tener el formato correcto.
        if (Utils.isEmptyString(email)) {
            mView.showEmptyEmailError();
            cancelRegistration = true;
        } else if (!Utils.isEmailValid(email)) {
            mView.showInvalidEmailError();
            cancelRegistration = true;
        }

        // La localización es obligatoria.
        if(Utils.isEmptyString(location)) {
            mView.showEmptyLocationError();
            cancelRegistration = true;
        }

        // El nombre del usuario es obligatorio.
        if(Utils.isEmptyString(fullName)) {
            mView.showEmptyFullNameError();
            cancelRegistration = true;
        }

        // Si los datos son correctos, intentamos el registro.
        if(!cancelRegistration) {
            doCreateAccount(fullName, email, password);
        }
    }

    /**
     * @see RegistrationContract.Presenter#login()
     */
    @Override
    public void login() {
        mView.navigateToLogin();
    }

    private void doCreateAccount(String fullName, String email, String password) {
        UiUser user = new UiUser();
        user.setNombre(fullName);
        user.setEmail(email);
        user.setPassword(password);
        user.setLatitud(mLocationLatitude);
        user.setLongitud(mLocationLongitude);

        mView.showProgress(true);
        mInteractor.createAccount(user, new ServiceCallback<Void>() {
            @Override
            public void onSuccess(Void response) {
                // Si el registro es correcto pasamos a la pantalla principal,
                // ya que también estamos autenticados.
                mView.showProgress(false);
                mView.navigateToHome();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si el registro no es correcto mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showRegistrationError();
            }
        });
    }
}
