package edu.uoc.jsev.b4b.ui.swaps;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewAdapter;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewItemClickListener;
import edu.uoc.jsev.b4b.ui.main.MainFragment;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailActivity;
import edu.uoc.jsev.b4b.Injection;

/**
 * Fragmento de la pantalla de listado de intercambios.
 * El listado se implementa a través de un RecyclerView
 * cuyos datos se pueden refrescar por medio de un
 * SwipeRefreshLayout.
 */
public class SwapsFragment extends MainFragment
        implements SwapsContract.View, RecyclerViewItemClickListener {

    // Presentador.
    private SwapsContract.Presenter mPresenter;

    public SwapsFragment() {

    }

    // Método fábrica.
    public static SwapsFragment newInstance() {
        return new SwapsFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Inyectamos el presentador.
        mPresenter = Injection.provideSwapsPresenter(this);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_swaps, container, false);

        // Configuramos el recyclerview.
        mAdapter = new RecyclerViewAdapter(mPresenter, this, RecyclerViewAdapter.SWAP_VIEW_TYPE);
        setupRecyclerView(view);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();

        // Solicitamos al presentador los intercambios del usuario.
        mPresenter.loadData();

        return view;
    }

    @Override
    public void onHiddenChanged(boolean hidden) {
        // Volvemos a inicializar la pantalla (título y acción del FAB)
        // cada vez que el fragmento vuelve a estar visible.
        // Como solo se carga la primera vez, solo entra una vez por
        // onCreateView.
        super.onHiddenChanged(hidden);
        if(!hidden) mPresenter.setup();
    }

    @Override
    public void onRefresh() {
        // Al solicitar el refresco por medio del SwipeRefreshLayout,
        // volvemos a pedir los datos al presentador.
        super.onRefresh();
        mPresenter.loadData();
    }

    /**
     * Añade al listado un nuevo intercambio.
     * @see MainFragment#addObjectToList(UiEntity)
     */
    @Override
    public void addObjectToList(UiEntity dataObject) {
        super.addObjectToList(dataObject);
        mPresenter.addData((UiSwap) dataObject);
    }

    /**
     * Actualiza los datos en el listado de un intercambio existente.
     * @see MainFragment#updateObjectInList(UiEntity)
     */
    @Override
    public void updateObjectInList(UiEntity dataObject) {
        super.updateObjectInList(dataObject);
        mPresenter.updateData((UiSwap) dataObject);
    }

    /**
     * @see SwapsContract.View#setTitle()
     */
    @Override
    public void setTitle() {
        // Delega en la actividad contenedora el cambio.
        mCallback.setTitle(getString(R.string.title_fragment_swaps));
    }

    /**
     * @see SwapsContract.View#showFabButton(boolean)
     */
    @Override
    public void showFabButton(boolean show) {
        // Delega en la actividad contenedora la configuración del botón.
        mCallback.showFabButton(show);
    }

    /**
     * @see SwapsContract.View#getRequestedSwapIcon()
     */
    @Override
    public int getRequestedSwapIcon() {
        return R.drawable.ic_arrow_up_bold_circle_white_48dp;
    }

    /**
     * @see SwapsContract.View#getDestinationSwapIcon()
     */
    @Override
    public int getDestinationSwapIcon() {
        return R.drawable.ic_arrow_down_bold_circle_white_48dp;
    }

    /**
     * @see SwapsContract.View#getPendingSwapColor()
     */
    @Override
    public int getPendingSwapColor() {
        return android.R.color.holo_orange_light;
    }

    /**
     * @see SwapsContract.View#getCanceledSwapColor()
     */
    @Override
    public int getCanceledSwapColor() {
        return android.R.color.holo_red_dark;
    }

    /**
     * @see SwapsContract.View#getAcceptedSwapColor()
     */
    @Override
    public int getAcceptedSwapColor() {
        return android.R.color.holo_green_dark;
    }

    /**
     * @see SwapsContract.View#getRequestedByLabel(String)
     */
    @Override
    public String getRequestedByLabel(String userName) {
        return getString(R.string.label_requested_by, userName);
    }

    /**
     * @see SwapsContract.View#getRequestedToLabel(String)
     */
    @Override
    public String getRequestedToLabel(String userName) {
        return getString(R.string.label_requested_to, userName);
    }

    /**
     * @see RecyclerViewContract.View#refreshData()
     */
    @Override
    public void refreshData() {
        // Notificamos al adaptador los cambios en los datos
        // y evaluamos si es necesario o no mostrar la vista
        // del listado vacío.
        mAdapter.notifyDataSetChanged();
        showEmptyList(mAdapter.isEmpty());
    }

    /**
     * @see RecyclerViewContract.View#showLoadDataError()
     */
    @Override
    public void showLoadDataError() {
        showToastMessage(R.string.error_getting_swaps);
    }

    /**
     * @see RecyclerViewItemClickListener#onItemClick(UiEntity)
     */
    @Override
    public void onItemClick(UiEntity item) {
        mPresenter.dataClicked(item);
    }

    /**
     * @see MainFragment#fabButtonClicked()
     */
    @Override
    public void fabButtonClicked() {
        mPresenter.mainActionClicked();
    }

    /**
     * @see SwapsContract.View#navigateToDetail(UiEntity, Object...)
     */
    @Override
    public void navigateToDetail(UiEntity item, Object... params) {
        // Pasamos a la actividad de detalle los datos del intercambio pulsado.
        Intent intent = new Intent(getContext(), SwapDetailActivity.class);
        intent.putExtra(SwapDetailActivity.EXTRA_SWAP_DATA, item);
        startActivity(intent);
    }
}
