package edu.uoc.jsev.b4b.ui.swaps;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewPresenter;

/**
 * Clase de implementación del presentador de la pantalla del listado de intercambios.
 */
public class SwapsPresenter extends RecyclerViewPresenter implements SwapsContract.Presenter {

    // Vista e interactor asociados.
    private SwapsContract.View mView;
    private SwapsInteractor mInteractor;

    public SwapsPresenter(SwapsContract.View view, SwapsInteractor interactor) {
        super(view);
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see SwapsContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Establecemos el título de la actividad contenedora
        // y ocultamos el botón flotante.
        mView.setTitle();
        mView.showFabButton(false);
    }

    /**
     * @see SwapsContract.Presenter#addData(UiSwap)
     */
    @Override
    public void addData(UiSwap swapData) {
        // Añadimos el intercambio y refrescamos el listado,
        // para que se muestre el nuevo elemento.
        List<UiSwap> swaps = (List<UiSwap>) mDataList;
        swaps.add(0, swapData);
        mView.refreshData();
    }

    /**
     * @see SwapsContract.Presenter#updateData(UiSwap)
     */
    @Override
    public void updateData(UiSwap swapData) {
        // Actualizamos el intercambio y refrescamos
        // el listado para que se reflejen los cambios.
        List<UiSwap> swaps = (List<UiSwap>) mDataList;
        swaps.set(swaps.indexOf(swapData), swapData);
        mView.refreshData();
    }

    /**
     * @see SwapsContract.Presenter#mainActionClicked()
     */
    @Override
    public void mainActionClicked() {
        // No hacemos nada, el FAB está oculto.
    }

    /**
     * @see SwapsContract.Presenter#loadData()
     */
    @Override
    public void loadData() {
        mView.showProgress(true);
        final String currentUserId = mInteractor.getCurrentUserId();
        // Consultamos los intercambios del usuario.
        mInteractor.getUserSwaps(currentUserId, new ServiceCallback<List<UiSwap>>() {
            @Override
            public void onSuccess(List<UiSwap> response) {
                // Si la consulta es correcta mostramos
                // los intercambios en el listado.
                mView.showProgress(false);
                mDataList = response;
                mView.refreshData();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce algún error en al consulta
                // mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showLoadDataError();
            }
        });
    }

    /**
     * @see RecyclerViewContract.Presenter#bindDataAtPosition(RecyclerViewContract.RowView, int)
     */
    @Override
    public void bindDataAtPosition(RecyclerViewContract.RowView rowView, int position) {
        // Configuramos la información a mostrar para cada intercambio.
        UiSwap data = (UiSwap) mDataList.get(position);
        boolean requestedSwap = data.isRequested(mInteractor.getCurrentUserId());
        int state = data.getEstado();
        String bookTitle = data.getTituloLibroDestino();

        // Establecemos el icono en función de si el usuario es destinatario u origen del intercambio.
        int iconResId = requestedSwap ? mView.getRequestedSwapIcon() : mView.getDestinationSwapIcon();

        // Establecemos el color del icono en función del estado del intercambio.
        int colorResId = state == mInteractor.getSwapStatePending()
                ? mView.getPendingSwapColor() : state == mInteractor.getSwapStateAccepted()
                ? mView.getAcceptedSwapColor() : mView.getCanceledSwapColor();

        // Se establece el nombre del otro usuario participante en el listado.
        String user = data.getNombreParticipante(mInteractor.getCurrentUserId());
        user = requestedSwap ? mView.getRequestedToLabel(user) : mView.getRequestedByLabel(user);

        // Mostramos los datos del intercambio en el elemento del listado.
        rowView.setData(data);
        rowView.setImage(iconResId, colorResId);
        rowView.setTitle(bookTitle);
        rowView.setSubtitle(user);
        rowView.setExtraInfo(data.getFecha());
    }
}
