package edu.uoc.jsev.b4b.ui.swaps.swapdetail;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.NavUtils;
import android.support.v4.content.ContextCompat;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailActivity;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.rating.RatingDialogFragment;
import edu.uoc.jsev.b4b.Injection;

/**
 * Actividad de la pantalla de detalle de intercambio.
 * Desde esta actividad, aparte de visualizar sus datos,
 * se puede cancelar un intercambio, aceptarlo (seleccionando
 * un libro del otro usuario participante) y valorarlo.
 */
public class SwapDetailActivity extends BaseActivity implements SwapDetailContract.View {

    // Nombre de los extras que puede recibir la actividad desde otras pantallas.
    public static final String EXTRA_SWAP_DATA = "swapData";
    public static final String EXTRA_REQUESTED_BOOK_TITLE = "bookTitle";
    public static final String EXTRA_REQUESTED_BOOK_PICTURE = "bookPicture";

    // Identificadores para las opciones de menú.
    private static final int CANCEL_MENU_ITEM = Menu.FIRST;
    private static final int RATE_MENU_ITEM = CANCEL_MENU_ITEM + 1;

    // Tags para identificar los diálogos.
    private static final String TAG_CANCELATION_DIALOG_FRAGMENT = "cancelationDialog";
    private static final String TAG_RATING_DIALOG_FRAGMENT = "ratingDialog";

    // Referencias a las vistas.
    private ImageView mPicture1View;
    private TextView mInfo1View;
    private TextView mRating1View;
    private TextView mTitle1View;
    private ImageView mPicture2View;
    private TextView mInfo2View;
    private TextView mRating2View;
    private TextView mTitle2View;
    private View mStateLayout;
    private ImageView mStateIcon;
    private TextView mStateText;
    private ImageButton mBookSearchView;

    // Presentador.
    private SwapDetailContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_swap_detail);

        // Inyectamos el presentador.
        mPresenter = Injection.provideSwapDetailPresenter(this);

        // Configuramos las vistas.
        mPicture1View = findViewById(R.id.book_picture_1);
        mInfo1View = findViewById(R.id.swap_info_1);
        mRating1View = findViewById(R.id.rating_1);
        mTitle1View = findViewById(R.id.book_title_1);
        mPicture2View = findViewById(R.id.book_picture_2);
        mInfo2View = findViewById(R.id.swap_info_2);
        mRating2View = findViewById(R.id.rating_2);
        mTitle2View = findViewById(R.id.book_title_2);
        mStateLayout = findViewById(R.id.state_layout);
        mStateIcon = findViewById(R.id.state_icon);
        mStateText = findViewById(R.id.state_text);

        mBookSearchView = findViewById(R.id.book_search);
        mBookSearchView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.bookSearchButtonPressed();
            }
        });

        mProgressView = findViewById(R.id.progress);

        // Solicitamos al presentador que inicialice la pantalla.
        mPresenter.setup();
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        // Las opciones del menú se establecen dinámicamente,
        // en función del estado en que se encuentre el intercambio.
        mMenu = menu;
        mPresenter.setupMenu();
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        // Se pulsa la opción de menú para cancelar el intercambio.
        if(itemId == CANCEL_MENU_ITEM) {
            mPresenter.cancelOptionItemSelected();
            return true;
        }
        // Se pulsa la opción de menú para valorar el intercambio.
        else if(itemId == RATE_MENU_ITEM) {
            mPresenter.rateOptionItemSelected();
            return true;
        }
        else {
            return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);

        // Cuando se recibe la acción ACTION_EDIT en el intent significa
        // que el usuario acaba de volver de seleccionar un libro del otro
        // usuario participante, es decir, acaba de aceptar el intercambio.
        // En ese caso, actualizamos la pantalla, mostrando los datos del
        // libro seleccionado.
        if (intent != null && intent.getAction() != null && intent.getAction().equalsIgnoreCase(Intent.ACTION_EDIT)) {
            String bookTitle = intent.getStringExtra(EXTRA_REQUESTED_BOOK_TITLE);
            String bookPicture = intent.getStringExtra(EXTRA_REQUESTED_BOOK_PICTURE);
            mPresenter.bookRequested(bookTitle, bookPicture);
        }
    }

    @Override
    public boolean onSupportNavigateUp() {
        // Navegación al pulsar el botón "Up".
        mPresenter.upButtonPressed();
        return true;
    }

    @Override
    public void onBackPressed() {
        // Navegación al pulsar el botón "Back".
        mPresenter.backButtonPressed();
    }

    /**
     * @see SwapDetailContract.View#getSwapData()
     */
    @Override
    public UiSwap getSwapData() {
        return (UiSwap) getIntent().getSerializableExtra(EXTRA_SWAP_DATA);
    }

    /**
     * @see SwapDetailContract.View#showSearchButton(boolean)
     */
    @Override
    public void showSearchButton(boolean show) {
        mBookSearchView.setVisibility(show ? View.VISIBLE : View.GONE);
    }

    /**
     * @see SwapDetailContract.View#clearOptionsMenu()
     */
    @Override
    public void clearOptionsMenu() {
        mMenu.clear();
    }

    /**
     * @see SwapDetailContract.View#addCancelMenuItem()
     */
    @Override
    public void addCancelMenuItem() {
        mMenu.add(0, CANCEL_MENU_ITEM, Menu.NONE, R.string.action_cancel_swap)
                .setIcon(R.drawable.ic_cancel_white_24dp)
                .setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
    }

    /**
     * @see SwapDetailContract.View#addRateMenuItem()
     */
    @Override
    public void addRateMenuItem() {
        mMenu.add(0, RATE_MENU_ITEM, Menu.NONE, R.string.action_rate_swap)
                .setIcon(R.drawable.ic_star_white_24dp)
                .setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
    }

    /**
     * @see SwapDetailContract.View#setBook1Picture(String)
     */
    @Override
    public void setBook1Picture(String url) {
        Picasso.get()
                .load(url)
                .placeholder(R.drawable.book_placeholder)
                .into(mPicture1View);
    }

    /**
     * @see SwapDetailContract.View#setBook1PictureCanceled()
     */
    @Override
    public void setBook1PictureCanceled() {
        mPicture1View.setImageResource(R.drawable.ic_close_circle_outline);
    }

    /**
     * @see SwapDetailContract.View#setBook1Requested(String)
     */
    @Override
    public void setBook1Requested(String user) {
        mInfo1View.setText(getString(R.string.label_requested_to_requested, user));
    }

    /**
     * @see SwapDetailContract.View#setBook1Pending(String)
     */
    @Override
    public void setBook1Pending(String user) {
        mInfo1View.setText(getString(R.string.label_requested_by_pending, user));
    }

    /**
     * @see SwapDetailContract.View#setBook1Canceled(String)
     */
    @Override
    public void setBook1Canceled(String user) {
        mInfo1View.setText(getString(R.string.label_requested_by_canceled, user));
    }

    /**
     * @see SwapDetailContract.View#setBook1Title(String)
     */
    @Override
    public void setBook1Title(String title) {
        mTitle1View.setText(title);
    }

    /**
     * @see SwapDetailContract.View#setRating1(String)
     */
    @Override
    public void setRating1(String rating) {
        if(mRating1View.getVisibility() == View.GONE) {
            mRating1View.setVisibility(View.VISIBLE);
        }
        mRating1View.setText(rating);
    }

    /**
     * @see SwapDetailContract.View#setBook2Picture(String)
     */
    @Override
    public void setBook2Picture(String url) {
        Picasso.get()
                .load(url)
                .placeholder(R.drawable.book_placeholder)
                .into(mPicture2View);
    }

    /**
     * @see SwapDetailContract.View#setBook2PictureCanceled()
     */
    @Override
    public void setBook2PictureCanceled() {
        mPicture2View.setImageResource(R.drawable.ic_close_circle_outline);
    }

    /**
     * @see SwapDetailContract.View#setBook2Requested(String)
     */
    @Override
    public void setBook2Requested(String user) {
        mInfo2View.setText(getString(R.string.label_requested_by_requested, user));
    }

    /**
     * @see SwapDetailContract.View#setBook2Pending(String)
     */
    @Override
    public void setBook2Pending(String user) {
        mInfo2View.setText(getString(R.string.label_requested_to_pending, user));
    }

    /**
     * @see SwapDetailContract.View#setBook2Canceled(String)
     */
    @Override
    public void setBook2Canceled(String user) {
        mInfo2View.setText(getString(R.string.label_requested_to_canceled, user));
    }

    /**
     * @see SwapDetailContract.View#setBook2Title(String)
     */
    @Override
    public void setBook2Title(String title) {
        mTitle2View.setText(title);
    }

    /**
     * @see SwapDetailContract.View#setRating2(String)
     */
    @Override
    public void setRating2(String rating) {
        if(mRating2View.getVisibility() == View.GONE) {
            mRating2View.setVisibility(View.VISIBLE);
        }
        mRating2View.setText(rating);
    }

    /**
     * @see SwapDetailContract.View#setSwapStatePending()
     */
    @Override
    public void setSwapStatePending() {
        mStateIcon.setImageResource(R.drawable.ic_clock_outline_white_48dp);
        mStateText.setText(R.string.label_swap_state_pending);
        mStateLayout.setBackgroundColor(ContextCompat.getColor(this, android.R.color.holo_orange_light));
    }

    /**
     * @see SwapDetailContract.View#setSwapStateAccepted()
     */
    @Override
    public void setSwapStateAccepted() {
        mStateIcon.setImageResource(R.drawable.ic_check_circle_outline_white_48dp);
        mStateText.setText(R.string.label_swap_state_accepted);
        mStateLayout.setBackgroundColor(ContextCompat.getColor(this, android.R.color.holo_green_dark));
    }

    /**
     * @see SwapDetailContract.View#setSwapStateCanceled()
     */
    @Override
    public void setSwapStateCanceled() {
        mStateIcon.setImageResource(R.drawable.ic_cancel_white_48dp);
        mStateText.setText(R.string.label_swap_state_canceled);
        mStateLayout.setBackgroundColor(ContextCompat.getColor(this, android.R.color.holo_red_dark));
    }

    /**
     * @see SwapDetailContract.View#showCancelationDialog()
     */
    @Override
    public void showCancelationDialog() {
        showAlertDialog(R.string.action_cancel_swap, R.string.prompt_swap_cancelation, TAG_CANCELATION_DIALOG_FRAGMENT);
    }

    /**
     * @see SwapDetailContract.View#dismissCancelationDialog()
     */
    @Override
    public void dismissCancelationDialog() {
        dismissDialog(TAG_CANCELATION_DIALOG_FRAGMENT);
    }

    /**
     * @see SwapDetailContract.View#showRatingDialog()
     */
    @Override
    public void showRatingDialog() {
        RatingDialogFragment dialog = RatingDialogFragment.newInstance(R.string.action_rate_swap,
                R.string.prompt_swap_rating, R.layout.dialog_rate);
        dialog.show(getSupportFragmentManager(), TAG_RATING_DIALOG_FRAGMENT);
    }

    /**
     * @see SwapDetailContract.View#dismissRatingDialog()
     */
    @Override
    public void dismissRatingDialog() {
        dismissDialog(TAG_RATING_DIALOG_FRAGMENT);
    }

    /**
     * @see SwapDetailContract.View#showCancelationOk()
     */
    @Override
    public void showCancelationOk() {
        showToastMessage(R.string.message_swap_cancelation_ok);
    }

    /**
     * @see SwapDetailContract.View#showCancelationError()
     */
    @Override
    public void showCancelationError() {
        showToastMessage(R.string.error_canceling_swap);
    }

    /**
     * @see SwapDetailContract.View#showRatingOk()
     */
    @Override
    public void showRatingOk() {
        showToastMessage(R.string.message_swap_rating_ok);
    }

    /**
     * @see SwapDetailContract.View#showRatingError()
     */
    @Override
    public void showRatingError() {
        showToastMessage(R.string.error_rating_swap);
    }

    /**
     * @see SwapDetailContract.View#navigateToSwaps(UiSwap)
     */
    @Override
    public void navigateToSwaps(UiSwap swap) {
        // Navegamos a la lista de intercambios.
        Intent intent = NavUtils.getParentActivityIntent(this);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);

        // Informamos al listado de intercambios de si se debe actualizar.
        // Si se envía un intercambio significa que ha sido editado. Se
        // lo enviamos a la actividad listado para que actualice sus datos
        // y los muestre con la información actualizada.
        if(swap != null) {
            intent.setAction(Intent.ACTION_EDIT);
            intent.putExtra(MainActivity.EXTRA_DATA_OBJECT, swap);
        }

        NavUtils.navigateUpTo(this, intent);
    }

    /**
     * @see SwapDetailContract.View#navigateToUserDetail(UiUser, String)
     */
    @Override
    public void navigateToUserDetail(UiUser user, String swapId) {
        // Navegamos al detalle de usuario,
        // para seleccionar alguno de sus libros.
        Intent intent = new Intent(this, UserDetailActivity.class);
        intent.putExtra(UserDetailActivity.EXTRA_USER_DATA, user);
        intent.putExtra(UserDetailActivity.EXTRA_SWAP_ID, swapId);
        startActivity(intent);
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogPositiveClick(String, Object...)
     */
    @Override
    public void onDialogPositiveClick(String tag, Object... values) {
        // Se ha pulsado sobre "Aceptar" en el diálogo
        // de confirmación de cancelación del intercambio.
        if(tag.equals(TAG_CANCELATION_DIALOG_FRAGMENT)) {
            mPresenter.cancelationDialogOk();
        }
        // Se ha pulsado sobre "Aceptar" en el diálogo
        // de valoración del intercambio.
        else if(tag.equals(TAG_RATING_DIALOG_FRAGMENT)) {
            float rating = values.length > 0 ? (float) values[0] : 1f;
            mPresenter.ratingDialogOk(rating);
        }
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        // Se ha pulsado sobre "Cancelar" en el diálogo
        // de confirmación de cancelación del intercambio.
        if (tag.equals(TAG_CANCELATION_DIALOG_FRAGMENT)) {
            mPresenter.cancelationDialogCancel();
        }
        // Se ha pulsado sobre "Cancelar" en el diálogo
        // de valoración del intercambio.
        else if(tag.equals(TAG_RATING_DIALOG_FRAGMENT)) {
            mPresenter.ratingDialogCancel();
        }
    }
}
