package edu.uoc.jsev.b4b.ui.swaps.swapdetail;

import android.util.Log;

import com.backendless.BackendlessUser;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Intercambio;
import edu.uoc.jsev.b4b.data.backendless.model.Usuario;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.SwapService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.messaging.MailTemplate;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseSwapsInteractorImpl;

/**
 * Clase de implementación del interactor de la pantalla de detalle de intercambio.
 */
public class SwapDetailInteractorImpl extends BaseSwapsInteractorImpl implements SwapDetailInteractor {

    // Instancia singleton.
    private static SwapDetailInteractor instance;

    private SwapDetailInteractorImpl(UserService userService,
                                     CommonsService commonsService,
                                     SwapService swapService) {
        super(userService, commonsService, swapService);
    }

    // Devuelve el singleton.
    public static SwapDetailInteractor getInstance(UserService userService,
                                                   CommonsService commonsService,
                                                   SwapService swapService) {
        if (instance == null) {
            instance = new SwapDetailInteractorImpl(userService, commonsService, swapService);
        }
        return instance;
    }

    /**
     * @see SwapDetailInteractor#findSwapParticipant(String, String, ServiceCallback)
     */
    @Override
    public void findSwapParticipant(String srcUserId, String dstUserId, final ServiceCallback<UiUser> callback) {
        String currentUserId = getCurrentUserId();
        String participantId = currentUserId.equals(srcUserId) ? dstUserId : srcUserId;
        mUserService.findUserById(participantId, new ServiceCallback<BackendlessUser>() {
            @Override
            public void onSuccess(BackendlessUser response) {
                callback.onSuccess(userMapper.from(response));
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /**
     * @see SwapDetailInteractor#cancelSwap(String, ServiceCallback)
     */
    @Override
    public void cancelSwap(String swapId, final ServiceCallback<Void> callback) {
        Intercambio swap = new Intercambio();
        swap.setObjectId(swapId);
        swap.setEstado(getSwapStateCanceled());

        // Actualizamos el estado del intercambio a "Cancelado".
        mSwapService.updateSwap(swap, new ServiceCallback<Intercambio>() {
            @Override
            public void onSuccess(Intercambio swapResponse) {
                callback.onSuccess(null);

                // Enviamos un mensaje de correo al usuario que inició el intercambio,
                // para informarle de que el destinatario lo ha cancelado.
                mUserService.findUserById(swapResponse.getOwnerId(), new ServiceCallback<BackendlessUser>() {
                    @Override
                    public void onSuccess(BackendlessUser userResponse) {
                        sendEmail(MailTemplate.SWAP_CANCELED.getSubject(),
                                MailTemplate.SWAP_CANCELED.getMessage(getCurrentUserFullName()),
                                userResponse.getEmail());
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        Log.w("--->", "No se pudo enviar el mensaje: " + fault.getMessage());
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }

    /**
     * @see SwapDetailInteractor#rateSwap(String, String, int, int, ServiceCallback)
     */
    @Override
    public void rateSwap(String swapId, final String userId, int srcRating, int dstRating, final ServiceCallback<Void> callback) {
        final Intercambio swap = new Intercambio();
        swap.setObjectId(swapId);
        swap.setEstado(getSwapStateAccepted());
        swap.setValoracionOrigen(srcRating);
        swap.setValoracionDestino(dstRating);

        // Actualizamos el intercambio con la valoración añadida.
        mSwapService.updateSwap(swap, new ServiceCallback<Intercambio>() {
            @Override
            public void onSuccess(Intercambio response) {
                // Calculamos la nueva valoración media del usuario.
                mSwapService.getUserRating(userId, new ServiceCallback<Double>() {
                    @Override
                    public void onSuccess(final Double avgRating) {
                        // Actualizamos los datos del usuario para establecer su nueva valoración media.
                        mUserService.findUserById(userId, new ServiceCallback<BackendlessUser>() {
                            @Override
                            public void onSuccess(final BackendlessUser userResponse) {
                                userResponse.setProperty(Usuario.RATING_KEY, avgRating);
                                mUserService.updateUser(userResponse, new ServiceCallback<BackendlessUser>() {
                                    @Override
                                    public void onSuccess(BackendlessUser response) {
                                        // Finalmente, enviamos un correo al usuario,
                                        // informándole de que ha recibido una nueva valoración.
                                        sendEmail(MailTemplate.SWAP_RATED.getSubject(),
                                                MailTemplate.SWAP_RATED.getMessage(getCurrentUserFullName()),
                                                userResponse.getEmail());
                                        callback.onSuccess(null);
                                    }

                                    @Override
                                    public void onError(ServiceFault fault) {
                                        callback.onError(fault);
                                    }
                                });
                            }

                            @Override
                            public void onError(ServiceFault fault) {
                                callback.onError(fault);
                            }
                        });
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        callback.onError(fault);
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }
}
