package edu.uoc.jsev.b4b.ui.swaps.swapdetail;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;

/**
 * Clase de implementación del presentador de la pantalla de detalle de intercambio.
 */
public class SwapDetailPresenter extends BasePresenter implements SwapDetailContract.Presenter {

    // Datos del intercambio mostrado en la pantalla.
    private UiSwap mSwapData;

    // Datos del otro usuario participante en el intercambio,
    // el que no es el usuario que está usando la aplicación.
    private UiUser mSwapParticipant;

    // Flag para controlar si los datos del intercambio
    // han sido actualizados en la pantalla:
    //      - Se ha cancelado.
    //      - Se ha aceptado, seleccionando un libro del otro usuario.
    //      - Se ha valorado.
    private boolean mUpdated = false;

    // Vista e interactor asociados.
    private SwapDetailContract.View mView;
    private SwapDetailInteractor mInteractor;

    public SwapDetailPresenter(SwapDetailContract.View view, SwapDetailInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see SwapDetailContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Obtenemos los datos del intercambio,
        // solicitándolos a la vista, y obtenemos
        // los datos del otro usuario participante.
        if(mSwapData == null) {
            mSwapData = mView.getSwapData();
            findParticipant();
        }

        // Mostramos los datos del intercambio.
        setupData(false);
    }

    /**
     * @see SwapDetailContract.Presenter#setupMenu()
     */
    @Override
    public void setupMenu() {
        String currentUser = mInteractor.getCurrentUserId();
        mView.clearOptionsMenu();
        // Si el intercambio está pendiente y el usuario es el destinatario
        // mostramos en el menú el icono de cancelación.
        if(isPending() && !mSwapData.isRequested(currentUser)) {
            mView.addCancelMenuItem();
        }
        // Si el intercambio está aceptado y aún no ha sido valorado
        // por el usuariom mostramos en el menú el icono de valoración.
        else if(isAccepted() && !mSwapData.isRated(currentUser)) {
            mView.addRateMenuItem();
        }
    }

    /**
     * @see SwapDetailContract.Presenter#cancelOptionItemSelected()
     */
    @Override
    public void cancelOptionItemSelected() {
        mView.showCancelationDialog();
    }

    /**
     * @see SwapDetailContract.Presenter#rateOptionItemSelected()
     */
    @Override
    public void rateOptionItemSelected() {
        mView.showRatingDialog();
    }

    /**
     * @see SwapDetailContract.Presenter#cancelationDialogOk()
     */
    @Override
    public void cancelationDialogOk() {
        mView.showProgress(true);
        String swapId = mSwapData.getId();

        // Cancelamos el intercambio.
        mInteractor.cancelSwap(swapId, new ServiceCallback<Void>() {
            @Override
            public void onSuccess(Void response) {
                // Si se cancela correctamente, actualizamos
                // los datos del intercambio en la pantalla.
                mView.showProgress(false);
                mSwapData.setEstado(mInteractor.getSwapStateCanceled());
                setupData(true);
                mView.showCancelationOk();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si hay algún error en la cancelación,
                // mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showCancelationError();
            }
        });
    }

    /**
     * @see SwapDetailContract.Presenter#cancelationDialogCancel()
     */
    @Override
    public void cancelationDialogCancel() {
        mView.dismissCancelationDialog();
    }

    /**
     * @see SwapDetailContract.Presenter#ratingDialogOk(float)
     */
    @Override
    public void ratingDialogOk(final float rating) {
        mView.showProgress(true);
        final String currentUser = mInteractor.getCurrentUserId();
        boolean isRequestedSwap = mSwapData.isRequested(currentUser);
        int srcRating = isRequestedSwap ? mSwapData.getValoracionOrigen(): (int) rating;
        int dstRating = isRequestedSwap ? (int) rating : mSwapData.getValoracionDestino();
        String swapId = mSwapData.getId();
        String userId = mSwapParticipant.getId();

        // Añadimos la valoración al intercambio.
        mInteractor.rateSwap(swapId, userId, srcRating, dstRating, new ServiceCallback<Void>() {
            @Override
            public void onSuccess(Void response) {
                // Si la valoración se añade correctamente, actualizamos
                // los datos del intercambio en la pantalla.
                mView.showProgress(false);
                mSwapData.setRating((int) rating, currentUser);
                setupData(true);
                mView.showRatingOk();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si hay algún error al registrar la valoración,
                // mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showRatingError();
            }
        });

    }

    /**
     * @see SwapDetailContract.Presenter#ratingDialogCancel()
     */
    @Override
    public void ratingDialogCancel() {
        mView.dismissRatingDialog();
    }

    /**
     * @see SwapDetailContract.Presenter#bookSearchButtonPressed()
     */
    @Override
    public void bookSearchButtonPressed() {
        mView.navigateToUserDetail(mSwapParticipant, mSwapData.getId());
    }

    /**
     * @see SwapDetailContract.Presenter#bookRequested(String, String)
     */
    @Override
    public void bookRequested(String bookTitle, String bookPicture) {
        // Se ha seleccionado un libro del otro usuario.
        // Actualizamos los datos del intercambio, para incluir
        // el libro seleccionado y colocar su estado como aceptado.
        mSwapData.setFotoLibroOrigen(bookPicture);
        mSwapData.setTituloLibroOrigen(bookTitle);
        mSwapData.setEstado(mInteractor.getSwapStateAccepted());

        // Actualizamos el flag que indica
        // si el intercambio ha sido actualizado.
        setupData(true);
    }

    /**
     * @see SwapDetailContract.Presenter#backButtonPressed()
     */
    @Override
    public void backButtonPressed() {
        // Si el intercambio ha sido actualizado enviamos
        // los datos actualizados a la actividad destino.
        UiSwap swapData = mUpdated ? mSwapData : null;
        mView.navigateToSwaps(swapData);
    }

    /*** MÉTODOS AUXILIARES ***/

    /**
     * Obtiene los datos del otro usuario participante.
     */
    private void findParticipant() {
        mView.showProgress(true);
        String srcUserId = mSwapData.getUsuarioLibroOrigen();
        String dstUserId = mSwapData.getUsuarioLibroDestino();

        mInteractor.findSwapParticipant(srcUserId, dstUserId, new ServiceCallback<UiUser>() {
            @Override
            public void onSuccess(UiUser response) {
                mView.showProgress(false);
                mSwapParticipant = response;
            }

            @Override
            public void onError(ServiceFault fault) {
                mView.showProgress(false);
            }
        });
    }

    /**
     * Indica si el intercambio está en estado pendiente.
     * @return true si el intercambio está pendiente,
     * false en caso contrario.
     */
    private boolean isPending() {
        return mSwapData.getEstado() == mInteractor.getSwapStatePending();
    }

    /**
     * Indica si el intercambio está en estado cancelado.
     * @return true si el intercambio está cancelado,
     * false en caso contrario.
     */
    private boolean isCanceled() {
        return mSwapData.getEstado() == mInteractor.getSwapStateCanceled();
    }

    /**
     * Indica si el intercambio está en estado aceptado.
     * @return true si el intercambio está aceptado,
     * false en caso contrario.
     */
    private boolean isAccepted() {
        return mSwapData.getEstado() == mInteractor.getSwapStateAccepted();
    }

    /**
     * Configura la vista para mostrar los datos del intercambio.
     * @param swapUpdated Booleano que indica si los datos
     *                    del intercambio han sido actualizados.
     */
    private void setupData(boolean swapUpdated) {
        if(mSwapData.isRequested(mInteractor.getCurrentUserId())) {
            setupRequestedSwap();
        }
        else {
            setupDestinationSwap();
        }

        setupState();
        setupBookSearchButton();

        if(swapUpdated) {
            mUpdated = true;
            setupMenu();
        }
    }

    /**
     * Configura los datos del intercambio cuando
     * el usuario es el que lo ha iniciado.
     */
    private void setupRequestedSwap() {
        int state = mSwapData.getEstado();
        String user = mSwapData.getNombreParticipante(mInteractor.getCurrentUserId());
        mView.setBook1Picture(mSwapData.getFotoLibroDestino());
        mView.setBook1Requested(user);
        mView.setBook1Title(mSwapData.getTituloLibroDestino());

        if(state == mInteractor.getSwapStatePending()) {
            mView.setBook2Pending(user);
        }
        else if(state == mInteractor.getSwapStateCanceled()) {
            mView.setBook2PictureCanceled();
            mView.setBook2Canceled(user);
        }
        else {
            mView.setBook2Requested(user);
            mView.setBook2Picture(mSwapData.getFotoLibroOrigen());
            mView.setBook2Title(mSwapData.getTituloLibroOrigen());

            if(mSwapData.getValoracionDestino() > 0) {
                mView.setRating1(String.valueOf(mSwapData.getValoracionDestino()));
            }

            if(mSwapData.getValoracionOrigen() > 0) {
                mView.setRating2(String.valueOf(mSwapData.getValoracionOrigen()));
            }
        }
    }

    /**
     * Configura los datos del intercambio cuando
     * el usuario es el destinatario del mismo.
     */
    private void setupDestinationSwap() {
        int state = mSwapData.getEstado();
        String user = mSwapData.getNombreParticipante(mInteractor.getCurrentUserId());
        mView.setBook2Picture(mSwapData.getFotoLibroDestino());
        mView.setBook2Requested(user);
        mView.setBook2Title(mSwapData.getTituloLibroDestino());

        if(state == mInteractor.getSwapStatePending()) {
            mView.setBook1Pending(user);
        }
        else if(state == mInteractor.getSwapStateCanceled()) {
            mView.setBook1PictureCanceled();
            mView.setBook1Canceled(user);
        }
        else {
            mView.setBook1Requested(user);
            mView.setBook1Picture(mSwapData.getFotoLibroOrigen());
            mView.setBook1Title(mSwapData.getTituloLibroOrigen());

            if(mSwapData.getValoracionDestino() > 0) {
                mView.setRating2(String.valueOf(mSwapData.getValoracionDestino()));
            }

            if(mSwapData.getValoracionOrigen() > 0) {
                mView.setRating1(String.valueOf(mSwapData.getValoracionOrigen()));
            }
        }
    }

    /**
     * Habilita el botón para que el usuario seleccione
     * un libro del otro usuario participante,
     * si el intercambio está pendiente y el usuario
     * es el destinatario del intercambio
     */
    private void setupBookSearchButton() {
        boolean isRequestedSwap = mSwapData.isRequested(mInteractor.getCurrentUserId());
        mView.showSearchButton(!isRequestedSwap && isPending());
    }

    /**
     * Muestra por pantalla el estado del intermcambio.
     */
    private void setupState() {
        int state = mSwapData.getEstado();
        if(state == mInteractor.getSwapStateAccepted()) {
            mView.setSwapStateAccepted();
        }
        else if(state == mInteractor.getSwapStateCanceled()) {
            mView.setSwapStateCanceled();
        }
        else {
            mView.setSwapStatePending();
        }
    }
}
