package edu.uoc.jsev.b4b.util;

import android.content.Context;
import android.content.pm.PackageManager;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.provider.Settings;
import android.support.v4.content.ContextCompat;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.util.Log;
import android.widget.TextView;

/**
 * Clase auxiliar de utilidades dependientes de un contexto Android.
 */
public class ContextUtils {

    /**
     * Indica si el servicio de localización del dispositivo
     * está o no activado.
     *
     * @param context Contexto
     * @return true si el servicio de localización está activado,
     * false en caso contrario.
     */
    public static boolean isLocationEnabled(Context context) {
        try {
            int locationMode = Settings.Secure.getInt(context.getContentResolver(), Settings.Secure.LOCATION_MODE);
            return locationMode != Settings.Secure.LOCATION_MODE_OFF;
        }
        catch(Settings.SettingNotFoundException e) {
            Log.e("Exception: %s", e.getMessage());
            return false;
        }
    }

    /**
     * Indica si se le ha concedido a la aplicación el acceso a un determinado permiso.
     *
     * @param context Contexto
     * @param permission Permiso a consultar
     * @return true si la aplicación tiene el permiso, false en caso contrario.
     */
    public static boolean isPermissionGranted(Context context, String permission) {
        return ContextCompat.checkSelfPermission(context, permission)
                == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * Colorea los iconos asociados a un conjunto de TextViews.
     * El método colorea los cuatro iconos, en caso de existir:
     * superior, derecho, inferior e izquierdo.
     *
     * @param textViews Array con los TextViews cuyos iconos se quiere colorear.
     * @param colorResId Código de color a utilizar.
     */
    public static void tintCompoundDrawables(TextView[] textViews, int colorResId) {
        for(TextView textView : textViews) {
            tintCompoundDrawables(textView, colorResId);
        }
    }

    /**
     * Colorea los iconos asociados a un TextView.
     * El método colorea los cuatro iconos, en caso de existir:
     * superior, derecho, inferior e izquierdo.
     *
     * @param textView TextView cuyos iconos se quiere colorear.
     * @param colorResId Código de color a utilizar.
     */
    public static void tintCompoundDrawables(TextView textView, int colorResId) {
        Context context = textView.getContext();
        Drawable[] drawables = textView.getCompoundDrawablesRelative();
        Drawable[] tintedDrawables = new Drawable[drawables.length];

        for(int i=0; i<drawables.length; i++) {
            Drawable drawable = drawables[i];
            if(drawable != null) {
                tintedDrawables[i] = tintDrawable(context, drawable, colorResId);
            }
        }

        textView.setCompoundDrawablesWithIntrinsicBounds(tintedDrawables[0],
                tintedDrawables[1], tintedDrawables[2], tintedDrawables[3]);
    }

    /**
     * Colorea un recurso de imagen con el color indicado.
     *
     * @param context Contexto
     * @param drawable Recurso de imagen a colorear.
     * @param colorResId Código de color a utilizar.
     * @return El recurso de imagen coloreado en el color indicado.
     */
    public static Drawable tintDrawable(Context context, Drawable drawable, int colorResId) {
        Drawable tintedDrawable = DrawableCompat.wrap(drawable);
        DrawableCompat.setTint(tintedDrawable, ContextCompat.getColor(context, colorResId));
        DrawableCompat.setTintMode(tintedDrawable, PorterDuff.Mode.SRC_IN);

        return drawable;
    }
}
