package edu.uoc.jsev.b4b.util;

import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.widget.ImageView;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Clase auxiliar con utilidades genéricas.
 */
public class Utils {

    // Formateador para fechas.
    private static DateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());

    /**
     * Indica si una cadena de texto es vacía o nula.
     *
     * @param str Cadena de texto a evaluar.
     * @return true si la cadena es nula o vacía, false en caso contrario.
     */
    public static boolean isEmptyString(String str) {
        return str == null || str.isEmpty();
    }

    /**
     * Indica si una dirección de correo electrónico tiene
     * un formato correcto (usuario@dominio.extension).
     *
     * @param email Cadena de texto con la dirección de correo.
     * @return true si la dirección de correo tiene un formato correcto,
     * false en caso contrario.
     */
    public static boolean isEmailValid(String email) {
        Pattern pattern = Pattern.compile("^[\\w\\.-]+@([\\w\\-]+\\.)+[A-Z]{2,4}$", Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(email);
        return matcher.matches();
    }

    /**
     * Indica si una cadena de texto tiene, como mínimo, el número
     * de caracteres pasado como parámetro.
     *
     * @param str La cadena a evaluar.
     * @param minLength Número mínimo de caracteres que la cadena debe tener.
     * @return true si la cadena contiene, como mínimo, el número de caracteres
     * indicado, false en caso contrario.
     */
    public static boolean hasMinLength(String str, int minLength) {
        return str.length() >= minLength;
    }

    /**
     * Formatea la fecha pasada como parámetro con el patrón dd/mm/aaaa.
     *
     * @param date La fecha a formatear.
     * @return La fecha formateada con el patrón dd/mm/aaaa. Si la fecha
     * es nula, el método devuelve null.
     */
    public static String formatDate(Date date) {
        return date != null ? dateFormatter.format(date) : null;
    }

    /**
     * Obtiene una fecha (objeto Date) a partir de la cadena pasada como parámetro.
     * La cadena debe tener un formato dd/mm/aaaa, en caso contrario, se devolverá
     * null.
     *
     * @param strDate Cadena con la fecha a parsear, en formato dd/mm/aaaa.
     * @return Fecha parseada o null si el formato de la cadena de entrada
     * no es dd/mm/aaaa.
     */
    public static Date parseDate(String strDate) {
        try {
            return dateFormatter.parse(strDate);
        }
        catch (ParseException e) {
            return null;
        }
    }

    /**
     * Devuelve una cadena de texto con los elementos de una lista unidos,
     * separados por el carácter ";". Es decir, si la lista contiene los
     * elementos [1,2,3,4], el método devuelve la cadena "1;2;3;4".
     * Si la lista es nula o vacía, se devuelve una cadena vacía.
     *
     * @param list La lista cuyos elementos se quiere unir.
     * @return Cadena de texto con los elementos de la lista unidos,
     * separados por el carácter ";", o una cadena vacía si la lista
     * es nula o vacía.
     */
    public static String join(List list) {
        if(list == null) return "";

        StringBuilder sb = new StringBuilder();
        Iterator<?> it = list.iterator();
        if (it.hasNext()) {
            sb.append(it.next());
            while (it.hasNext()) {
                sb.append(";");
                sb.append(it.next());
            }
        }
        return sb.toString();
    }

    /**
     * Formatea un número con precisión doble devolviendo una cadena
     * con solo una posición decimal.
     *
     * @param number Número a formatear.
     * @return Cadena con el número formateado, con solo un decimal.
     */
    public static String formatDouble(double number) {
        return String.format(Locale.getDefault(), "%.1f", number);
    }

    /**
     * Formatea un número con precisión doble devolviendo una cadena
     * con solo una posición decimal. En caso de que el número sea menor
     * que 1, se devuelve la cadena "-".
     *
     * @param number Número a formatear.
     * @return Cadena con el número formateado, con solo un decimal,
     * o la cadena "-" si el número es menor que 1.
     */
    public static String formatDoubleIfGreaterThan1(double number) {
        return number >= 1d ? formatDouble(number) : "-";
    }

    /**
     * Transforma una imagen en formato bitmap a otra en formato PNG,
     * que se devuelve como array de bytes.
     *
     * @param bitmap Imagen en formato bitmap a transformar.
     * @return Array de bytes con la imagen transformada en PNG.
     */
    public static byte[] getBytesFromBitmap(Bitmap bitmap) {
        ByteArrayOutputStream os = null;
        byte[] picture = null;

        try {
            if(bitmap != null) {
                os = new ByteArrayOutputStream();
                bitmap.compress(Bitmap.CompressFormat.PNG, 100, os);
                picture = os.toByteArray();
            }
        }
        catch(Exception e) {

        }
        finally {
            try {
                if (os != null) {
                    os.close();
                }
            }
            catch (IOException e) {

            }
        }

        return picture;
    }

    /**
     * Transforma la imagen contenida en una ImageView en otra en formato PNG,
     * que se devuelve como array de bytes. La imagen almacenada en la ImageView
     * debe estar en formato bitmap, en caso contrario, se devuelve null.
     *
     * @param imageView ImageView con la imagen a transformar.
     * @return Array de bytes con la imagen transformada en PNG.
     */
    public static byte[] getBytesFromImageView(ImageView imageView) {
        byte[] picture = null;
        if(imageView != null && imageView.getDrawable() != null && imageView.getDrawable() instanceof BitmapDrawable) {
            picture = getBytesFromBitmap(((BitmapDrawable) imageView.getDrawable()).getBitmap());
        }

        return picture;
    }
}
