package edu.uoc.jsev.b4b;

import android.graphics.Bitmap;

import org.junit.Test;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;

import edu.uoc.jsev.b4b.util.Utils;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * Tests unitarios para la clase de utilidades
 * @see Utils
 */
public class UtilsTest {

    @Test
    public void emptyString_isEmpty() {
        assertTrue(Utils.isEmptyString(""));
    }

    @Test
    public void emptyString_isNull() {
        assertTrue(Utils.isEmptyString(null));
    }

    @Test
    public void emptyString_notEmpty() {
        assertFalse(Utils.isEmptyString("string"));
    }

    @Test
    public void emailValid_isValid() {
        assertTrue(Utils.isEmailValid("j@s.com"));
    }

    @Test
    public void emailValid_isNotValidWithSpaces() {
        assertFalse(Utils.isEmailValid("j @s.com"));
    }

    @Test
    public void emailValid_isNotValidWithoutDomain() {
        assertFalse(Utils.isEmailValid("j@s."));
    }

    @Test
    public void emailValid_isNotValidWithoutAt() {
        assertFalse(Utils.isEmailValid("js.com"));
    }

    @Test
    public void minLength_hasMinLength() {
        assertTrue(Utils.hasMinLength("string", "string".length()));
    }

    @Test
    public void minLength_hasNotMinLength() {
        assertFalse(Utils.hasMinLength("string", "string".length() + 1));
    }

    @Test
    public void parseDate_isCorrect() {
        int day = 1, month = 1, year = 1970;
        Date date = Utils.parseDate(day + "/" + month + "/" + year);
        assertNotNull(date);

        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        assertTrue(cal.get(Calendar.DAY_OF_MONTH) == day);
        assertTrue(cal.get(Calendar.MONTH) == month - 1);
        assertTrue(cal.get(Calendar.YEAR) == year);
    }

    @Test
    public void formatDate_isNull() {
        assertNull(Utils.formatDate(null));
    }

    @Test
    public void formatDate_dateFormatted() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.set(Calendar.MONTH, 0);
        cal.set(Calendar.YEAR, 1980);

        assertEquals("01/01/1980", Utils.formatDate(cal.getTime()));
    }

    @Test
    public void parseDate_isNull() {
        assertNull(Utils.parseDate("1"));
    }

    @Test
    public void join_nullElement() {
        assertEquals("", Utils.join(null));
    }

    @Test
    public void join_noElements() {
        assertEquals("", Utils.join(new ArrayList()));
    }

    @Test
    public void join_oneElement() {
        assertEquals("elem1", Utils.join(Arrays.asList("elem1")));
    }

    @Test
    public void join_threeElements() {
        assertEquals("elem1;elem2;elem3", Utils.join(Arrays.asList("elem1", "elem2", "elem3")));
    }

    @Test
    public void formatDistance_roundDown() {
        assertEquals("2,4", Utils.formatDouble(2.415));
    }

    @Test
    public void formatDistance_roundUp() {
        assertEquals("2,5", Utils.formatDouble(2.45));
    }

    @Test
    public void formatDistance_integer() {
        assertEquals("2,0", Utils.formatDouble(2));
    }

    @Test
    public void formatRating_roundDown() {
        assertEquals("2,4", Utils.formatDoubleIfGreaterThan1(2.415));
    }

    @Test
    public void formatRating_roundUp() {
        assertEquals("2,5", Utils.formatDoubleIfGreaterThan1(2.45));
    }

    @Test
    public void formatRating_integer() {
        assertEquals("1,0", Utils.formatDoubleIfGreaterThan1(1));
    }

    @Test
    public void formatRating_zero() {
        assertEquals("-", Utils.formatDoubleIfGreaterThan1(0));
    }

    @Test
    public void bytesFromBitmap_isCorrect() {
        final byte[] expectedBytes = new byte[]{'a', 'b', 'c', 'd'};
        Bitmap bmp = mock(Bitmap.class);

        when(bmp.compress(any(Bitmap.CompressFormat.class), anyInt(), any(ByteArrayOutputStream.class)))
                .thenAnswer(new Answer<Void>() {
                    @Override
                    public Void answer(InvocationOnMock invocation) throws IOException {
                        ((ByteArrayOutputStream) invocation.getArguments()[2]).write(expectedBytes);
                        return null;
                    }
                });
        byte[] actualBytes = Utils.getBytesFromBitmap(bmp);
        assertArrayEquals(expectedBytes, actualBytes);
    }
}
